"""

	NAME
    EcatLib_2094S_Demo.py - a python implementation motion controll demo

  DESCRIPTION
    This program implements a motion controll interface for a stepper driver ECAT-2094S.
    User should first click "Start" Button to Start network configuration, and then click
    "McInit" to init motion setting when device is in OP state.
    
"""   

import wx
from EcatLib import *
import time

McReady = 0
LoopRun = 0
LoopDir = 0
WaitCnt = 0


def AddTimeStr(str):
    timeStamp = int(time.time()) 
    timeArray = time.localtime(timeStamp)
    strtime = time.strftime("%Y-%m-%d %H:%M:%S: ", timeArray) + str + "\n"
    return strtime


class MyFrame(wx.Frame):
    def __init__(self, *args, **kw):
        wx.Frame.__init__(self, *args, **kw)
        panel = wx.Panel(self)
        vbox = wx.BoxSizer(wx.VERTICAL)

        it = wx.StaticBox(panel, -1, 'Initial:', size=(600, -1))
        itSizer = wx.StaticBoxSizer(it, wx.VERTICAL)

        self.btnStart = wx.Button(panel, -1, label="Start")
        self.btnStop = wx.Button(panel, -1, label="Stop")
        self.btnMcInit = wx.Button(panel, -1, label="McInit")
        self.text_AlState = wx.TextCtrl(
            panel, -1, value="", style=wx.TE_READONLY)

        itbox = wx.BoxSizer(wx.HORIZONTAL)
        itbox.Add(self.btnStart, 0, wx.ALL | wx.CENTER, 5)
        itbox.Add(self.btnStop, 0, wx.ALL | wx.CENTER, 5)
        itbox.Add(self.btnMcInit, 0, wx.ALL | wx.CENTER, 5)
        itbox.Add(self.text_AlState, 0, wx.ALL | wx.CENTER, 5)
        itSizer.Add(itbox, 0, wx.ALL | wx.CENTER, 10)

        mc = wx.StaticBox(panel, -1, 'Motion Control:', size=(600, -1))
        mcSizer = wx.StaticBoxSizer(mc, wx.VERTICAL)

        self.txAxNo = wx.StaticText(panel, -1, label="AxisNo: ")
        self.btnServoOn = wx.Button(panel, -1, label="ServoOn")
        self.btnResetErr = wx.Button(panel, -1, label="Reset Err")
        self.btnResetPos = wx.Button(panel, -1, label="Reset Pos")

        AxisList = ["0", "1", "2", "3", "All"]
        self.cmbAxNo = wx.Choice(panel, -1, choices=AxisList)

        mcbox1 = wx.BoxSizer(wx.HORIZONTAL)
        mcbox1.Add(self.txAxNo, 0, wx.ALL | wx.CENTER, 5)
        mcbox1.Add(self.cmbAxNo, 0, wx.ALL | wx.CENTER, 5)
        mcbox1.Add(self.btnServoOn, 0, wx.ALL | wx.CENTER, 5)
        mcbox1.Add(self.btnResetErr, 0, wx.ALL | wx.CENTER, 5)
        mcbox1.Add(self.btnResetPos, 0, wx.ALL | wx.CENTER, 5)
        mcSizer.Add(mcbox1, 0, wx.ALL | wx.CENTER, 10)

        self.btnMoveP = wx.Button(panel, -1, label="Move+")
        self.btnMoveN = wx.Button(panel, -1, label="Move-")
        self.btnMcStop = wx.Button(panel, -1, label="McStop")
        self.btnLoopRun = wx.Button(panel, -1, label="LoopRun")
        mcbox2 = wx.BoxSizer(wx.HORIZONTAL)
        mcbox2.Add(self.btnMoveP, 0, wx.ALL | wx.CENTER, 5)
        mcbox2.Add(self.btnMoveN, 0, wx.ALL | wx.CENTER, 5)
        mcbox2.Add(self.btnMcStop, 0, wx.ALL | wx.CENTER, 5)
        mcbox2.Add(self.btnLoopRun, 0, wx.ALL | wx.CENTER, 5)
        mcSizer.Add(mcbox2, 0, wx.ALL | wx.CENTER, 10)

        ast = wx.StaticBox(panel, -1, 'Axis Status:', size=(600, -1))
        asSizer = wx.StaticBoxSizer(ast, wx.VERTICAL)

        self.txBlank = wx.StaticText(panel, -1, size=(60, -1), label="")
        self.txAx0 = wx.StaticText(panel, -1, size=(100, -1), label="Axis0")
        self.txAx1 = wx.StaticText(panel, -1, size=(100, -1), label="Axis1")
        self.txAx2 = wx.StaticText(panel, -1, size=(100, -1), label="Axis2")
        self.txAx3 = wx.StaticText(panel, -1, size=(100, -1), label="Axis3")
        asbox1 = wx.BoxSizer(wx.HORIZONTAL)
        asbox1.Add(self.txBlank, 0, wx.ALL | wx.LEFT, 5)
        asbox1.Add(self.txAx0, 0, wx.ALL | wx.CENTER, 5)
        asbox1.Add(self.txAx1, 0, wx.ALL | wx.CENTER, 5)
        asbox1.Add(self.txAx2, 0, wx.ALL | wx.CENTER, 5)
        asbox1.Add(self.txAx3, 0, wx.ALL | wx.CENTER, 5)
        asSizer.Add(asbox1, 0, wx.ALL | wx.LEFT, 10)

        self.txPos = wx.StaticText(panel, -1, label="Pos ")
        self.tx_pos_ax0 = wx.TextCtrl(
            panel, -1, size=(100, -1), value="", style=wx.TE_READONLY)
        self.tx_pos_ax1 = wx.TextCtrl(
            panel, -1, size=(100, -1), value="", style=wx.TE_READONLY)
        self.tx_pos_ax2 = wx.TextCtrl(
            panel, -1, size=(100, -1), value="", style=wx.TE_READONLY)
        self.tx_pos_ax3 = wx.TextCtrl(
            panel, -1, size=(100, -1), value="", style=wx.TE_READONLY)
        asbox2 = wx.BoxSizer(wx.HORIZONTAL)
        asbox2.Add(self.txPos, 0, wx.ALL | wx.LEFT, 5)
        asbox2.Add(self.tx_pos_ax0, 0, wx.ALL | wx.LEFT, 5)
        asbox2.Add(self.tx_pos_ax1, 0, wx.ALL | wx.LEFT, 5)
        asbox2.Add(self.tx_pos_ax2, 0, wx.ALL | wx.LEFT, 5)
        asbox2.Add(self.tx_pos_ax3, 0, wx.ALL | wx.LEFT, 5)
        asSizer.Add(asbox2, 0, wx.ALL | wx.LEFT, 10)

        self.txVel = wx.StaticText(panel, -1, label="Vel ")
        self.tx_vel_ax0 = wx.TextCtrl(
            panel, -1, size=(100, -1), value="", style=wx.TE_READONLY)
        self.tx_vel_ax1 = wx.TextCtrl(
            panel, -1, size=(100, -1), value="", style=wx.TE_READONLY)
        self.tx_vel_ax2 = wx.TextCtrl(
            panel, -1, size=(100, -1), value="", style=wx.TE_READONLY)
        self.tx_vel_ax3 = wx.TextCtrl(
            panel, -1, size=(100, -1), value="", style=wx.TE_READONLY)
        asbox3 = wx.BoxSizer(wx.HORIZONTAL)
        asbox3.Add(self.txVel, 0, wx.ALL | wx.LEFT, 5)
        asbox3.Add(self.tx_vel_ax0, 0, wx.ALL | wx.LEFT, 5)
        asbox3.Add(self.tx_vel_ax1, 0, wx.ALL | wx.LEFT, 5)
        asbox3.Add(self.tx_vel_ax2, 0, wx.ALL | wx.LEFT, 5)
        asbox3.Add(self.tx_vel_ax3, 0, wx.ALL | wx.LEFT, 5)
        asSizer.Add(asbox3, 0, wx.ALL | wx.LEFT, 10)

        self.txSt = wx.StaticText(panel, -1, label="State")
        self.tx_st_ax0 = wx.TextCtrl(
            panel, -1, size=(100, -1), value="", style=wx.TE_READONLY)
        self.tx_st_ax1 = wx.TextCtrl(
            panel, -1, size=(100, -1), value="", style=wx.TE_READONLY)
        self.tx_st_ax2 = wx.TextCtrl(
            panel, -1, size=(100, -1), value="", style=wx.TE_READONLY)
        self.tx_st_ax3 = wx.TextCtrl(
            panel, -1, size=(100, -1), value="", style=wx.TE_READONLY)
        asbox4 = wx.BoxSizer(wx.HORIZONTAL)
        asbox4.Add(self.txSt, 0, wx.ALL | wx.LEFT, 5)
        asbox4.Add(self.tx_st_ax0, 0, wx.ALL | wx.LEFT, 5)
        asbox4.Add(self.tx_st_ax1, 0, wx.ALL | wx.LEFT, 5)
        asbox4.Add(self.tx_st_ax2, 0, wx.ALL | wx.LEFT, 5)
        asbox4.Add(self.tx_st_ax3, 0, wx.ALL | wx.LEFT, 5)
        asSizer.Add(asbox4, 0, wx.ALL | wx.LEFT, 10)

        vbox.Add(itSizer, 0, wx.ALL | wx.LEFT, 5)
        vbox.Add(mcSizer, 0, wx.ALL | wx.LEFT, 5)
        vbox.Add(asSizer, 0, wx.ALL | wx.LEFT, 5)
        self.text_Log = wx.TextCtrl(
            panel, -1, size=(600, 75), value="", style=wx.TE_MULTILINE)
        vbox.Add(self.text_Log, 0, wx.ALL | wx.EXPAND, 5)

        panel.SetSizer(vbox)
        self.Centre()
        panel.Fit()

        self.Bind(wx.EVT_BUTTON, self.OnButtonStart, self.btnStart)
        self.Bind(wx.EVT_BUTTON, self.OnButtonStop, self.btnStop)
        self.Bind(wx.EVT_BUTTON, self.OnButtonMcInit, self.btnMcInit)
        self.Bind(wx.EVT_BUTTON, self.OnButtonServoOn, self.btnServoOn)
        self.Bind(wx.EVT_BUTTON, self.OnButtonResetErr, self.btnResetErr)
        self.Bind(wx.EVT_BUTTON, self.OnButtonResetPos, self.btnResetPos)
        self.Bind(wx.EVT_BUTTON, self.OnButtonMoveP, self.btnMoveP)
        self.Bind(wx.EVT_BUTTON, self.OnButtonMoveN, self.btnMoveN)
        self.Bind(wx.EVT_BUTTON, self.OnButtonMcStop, self.btnMcStop)
        self.Bind(wx.EVT_BUTTON, self.OnButtonLoopRun, self.btnLoopRun)
        self.Bind(wx.EVT_BUTTON, self.OnButtonLoopRun, self.btnLoopRun)

        self.timer = wx.Timer(self) 
        self.Bind(wx.EVT_TIMER, self.OnTimer, self.timer) 

    def OnButtonStart(self, event):
        ret = EcatLib.OpenDevice(0)
        str = "OpenDevice: {}".format(ret)
        self.text_Log.AppendText(AddTimeStr(str))

        if ret != 0:
            return

        time.sleep(3)
        
        EcatLib.StopDeviceOpTask(0)
        WaitCnt = 0     
        while (WaitCnt<100):
            ret, LinkUp, SlavesResp, AlStates, Wc = EcatLib.GetDeviceState(0)
            if AlStates.value == ECAT_AS_PREOP:
                self.text_Log.AppendText(AddTimeStr("Device state: PreOP"))
                break
            WaitCnt += 1
            if WaitCnt == 100:
                self.text_Log.AppendText(AddTimeStr("Wait device state PreOP timeout!"))
                ret = -1
                return
            time.sleep(0.1)        
        if ret != 0 or AlStates.value != ECAT_AS_PREOP:
            ret = -1
            self.text_Log.AppendText(AddTimeStr("ECAT_StopDeviceOpTask failed!"))
            return
        ret = EcatLib.SetSlaveNoType(0, SLAVE_NO_TYPE_ALIAS_ID)
        if ret != 0:
            str = "Set SlaveNo Type failed, ret: {}".format(ret)
            self.text_Log.AppendText(AddTimeStr(str))
            return

        ret = EcatLib.StartDeviceOpTask(0, 0, 0, 3)

        str = "StartDeviceOpTask: {}".format(ret)
        self.text_Log.AppendText(AddTimeStr(str))

        if ret != 0:
            return

        self.timer.Start(100)  # 設定時間間隔為100毫秒,並啟動定時器

    def OnButtonStop(self, event):
        global McReady
        ret = EcatLib.CloseDevice(0)
        str = "CloseDevice: {}".format(ret)
        self.text_Log.AppendText(AddTimeStr(str))
        McReady = 0
        self.text_AlState.SetLabel("preOP")
        self.timer.Stop()

    def OnButtonMcInit(self, event):
        global McReady
        AxisCnt = (c_ushort)(0)        
        SlaveNo = (c_ushort * MC_AXIS_NO_MAX)(*[0 for x in range(MC_AXIS_NO_MAX)])
        SubAxisNo = (c_ushort * MC_AXIS_NO_MAX)(*[0 for x in range(MC_AXIS_NO_MAX)])

        
        ret, SlaveNo, SubAxisNo, AxisCnt = EcatLib.McInit_Ex(0, "./Auto.motcfg".encode())  
        str = "McInitMcInit: {}".format(ret)
        self.text_Log.AppendText(AddTimeStr(str))
        if ret == 0:
            McReady = 1

    def OnButtonServoOn(self, event):
        AxNo = self.cmbAxNo.GetSelection()
        if AxNo >= 4:
            for AxisNo in range(0, 4):
                ret = EcatLib.McSetAxisServoOn(0, AxisNo, 1)
                str = "Axis[{}] McSetAxisServoOn: {}".format(AxisNo, ret)
                self.text_Log.AppendText(AddTimeStr(str))
        else:
            ret = EcatLib.McSetAxisServoOn(0, AxNo, 1)
            str = "Axis[{}] McSetAxisServoOn: {}".format(AxNo, ret)
            self.text_Log.AppendText(AddTimeStr(str))

    def OnButtonResetErr(self, event):
        AxNo = self.cmbAxNo.GetSelection()
        if AxNo >= 4:
            for AxisNo in range(0, 4):
                ret = EcatLib.McAxisErrorReset(0, AxisNo)
                str = "Axis[{}] McAxisErrorReset: {}".format(AxisNo, ret)
                self.text_Log.AppendText(AddTimeStr(str))
        else:
            ret = EcatLib.McAxisErrorReset(0, AxNo)
            str = "Axis[{}] McAxisErrorReset: {}".format(AxNo, ret)
            self.text_Log.AppendText(AddTimeStr(str))

    def OnButtonResetPos(self, event):
        AxNo = self.cmbAxNo.GetSelection()
        if AxNo >= 4:
            for AxisNo in range(0, 4):
                ret = EcatLib.McSetAxisHomeMethod(0, AxisNo, 35)
                str = "Axis[{}] McSetAxisHomeMethod: {}".format(AxisNo, ret)
                self.text_Log.AppendText(AddTimeStr(str))
                ret = EcatLib.McAxisHome(0, AxisNo)
                str = "Axis[{}] McAxisHome: {}".format(AxisNo, ret)
                self.text_Log.AppendText(AddTimeStr(str))
        else:
            ret = EcatLib.McSetAxisHomeMethod(0, AxNo, 35)
            str = "Axis[{}] McSetAxisHomeMethod: {}".format(AxNo, ret)
            self.text_Log.AppendText(AddTimeStr(str))
            ret = EcatLib.McAxisHome(0, AxNo)
            str = "Axis[{}] McAxisHome: {}".format(AxNo, ret)
            self.text_Log.AppendText(AddTimeStr(str))

    def OnButtonMoveP(self, event):
        AxNo = self.cmbAxNo.GetSelection()
        Vel = (c_double)(100000)
        if AxNo >= 4:
            for AxisNo in range(0, 4):
                ret = EcatLib.McAxisMoveVel(0, AxisNo, Vel)
                str = "Axis[{}] McAxisMoveVel(+): {}".format(AxisNo, ret)
                self.text_Log.AppendText(AddTimeStr(str))
        else:
            ret = EcatLib.McAxisMoveVel(0, AxNo, Vel)
            str = "Axis[{}] McAxisMoveVel(+): {}".format(AxNo, ret)
            self.text_Log.AppendText(AddTimeStr(str))

    def OnButtonMoveN(self, event):
        AxNo = self.cmbAxNo.GetSelection()
        Vel = (c_double)(-100000)
        if AxNo >= 4:
            for AxisNo in range(0, 4):
                ret = EcatLib.McAxisMoveVel(0, AxisNo, Vel)
                str = "Axis[{}] McAxisMoveVel(-): {}".format(AxisNo, ret)
                self.text_Log.AppendText(AddTimeStr(str))
        else:
            ret = EcatLib.McAxisMoveVel(0, AxNo, Vel)
            str = "Axis[{}] McAxisMoveVel(-): {}".format(AxNo, ret)
            self.text_Log.AppendText(AddTimeStr(str))

    def OnButtonMcStop(self, event):
        global LoopRun
        AxNo = self.cmbAxNo.GetSelection()
        if AxNo >= 4:
            for AxisNo in range(0, 4):
                ret = EcatLib.McAxisStop(0, AxisNo)
                str = "Axis[{}] McAxisStop: {}".format(AxisNo, ret)
                self.text_Log.AppendText(AddTimeStr(str))
        else:
            ret = EcatLib.McAxisStop(0, AxNo)
            str = "Axis[{}] McAxisStop: {}".format(AxNo, ret)
            self.text_Log.AppendText(AddTimeStr(str))
        if LoopRun == 1:
            LoopRun = 0
            str = "Stop LoopRun!"
            self.text_Log.AppendText(AddTimeStr(str))

    def OnButtonLoopRun(self, event):
        global LoopRun
        global LoopDir
        global WaitCnt
        LoopRun = 1
        LoopDir = 0
        WaitCnt = 0
        str = "Start LoopRun!"
        self.text_Log.AppendText(AddTimeStr(str))

    def OnTimer(self, evt):  
        global McReady
        global LoopRun
        global LoopDir
        global WaitCnt

        ret, LinkUp, SlavesResp, AlStates, Wc = EcatLib.GetDeviceState(0)
        str = ""

        if AlStates.value == ECAT_AS_INIT:
            str = "Init"
        elif AlStates.value == ECAT_AS_PREOP:
            str = "PreOP"
        elif AlStates.value == ECAT_AS_SAFEOP:
            str = "SafeOP"
        elif AlStates.value == ECAT_AS_OP:
            str = "OP"
        else:
            str = "{}".format(AlStates)
        self.text_AlState.SetLabel(str)

        nAxisReady = 0

        if McReady == 1:
            ret, Pos = EcatLib.McGetAxisActualPos_Ex(0, 65535)
            str = "{}".format(Pos[0])
            self.tx_pos_ax0.SetLabel(str)
            str = "{}".format(Pos[1])
            self.tx_pos_ax1.SetLabel(str)
            str = "{}".format(Pos[2])
            self.tx_pos_ax2.SetLabel(str)
            str = "{}".format(Pos[3])
            self.tx_pos_ax3.SetLabel(str)

            ret, Vel = EcatLib.McGetAxisActualVel_Ex(0, 65535)
            str = "{}".format(Vel[0])
            self.tx_vel_ax0.SetLabel(str)
            str = "{}".format(Vel[1])
            self.tx_vel_ax1.SetLabel(str)
            str = "{}".format(Vel[2])
            self.tx_vel_ax2.SetLabel(str)
            str = "{}".format(Vel[3])
            self.tx_vel_ax3.SetLabel(str)

            ret, State = EcatLib.McGetAxisState_Ex(0, 65535)
            str = "{:d}".format(State[0])
            self.tx_st_ax0.SetLabel(str)
            str = "{:d}".format(State[1])
            self.tx_st_ax1.SetLabel(str)
            str = "{:d}".format(State[2])
            self.tx_st_ax2.SetLabel(str)
            str = "{:d}".format(State[3])
            self.tx_st_ax3.SetLabel(str)

            for AxisNo in range(0, 4):
                if State[AxisNo] == 1:
                    nAxisReady += 1

        if LoopRun == 1 and nAxisReady == 4:
            if WaitCnt < 2:
                WaitCnt += 1
            else:
                WaitCnt = 0
                if LoopDir == 1:
                    LoopDir = 0
                    AxisPos = (c_double)(2160000)
                    AxisVel = (c_double)(100000)
                    for AxisNo in range(0, 4):
                        ret = EcatLib.McAxisMoveAbs(
                            0, AxisNo, AxisPos, AxisVel)
                else:
                    LoopDir = 1
                    AxisPos = (c_double)(0)
                    AxisVel = (c_double)(100000)
                    for AxisNo in range(0, 4):
                        ret = EcatLib.McAxisMoveAbs(
                            0, AxisNo, AxisPos, AxisVel)


app = wx.App()
frame = MyFrame(None, title="ECAT-2094S Test Demo", size=(600, 650))
frame.Show()
app.MainLoop()
