# -*- coding: utf-8 -*-
"""
NAME
    EcatLib_Iotest.py - a python implementation test code for EtherCAT I/O slaves which are controlled by ECAT-M801

DESCRIPTION
    This code test some I/O controll related functions in EcatLib. 
	The 1st ,2nd slave are EtherCAT AIO modules and the 3rd slave is an EtherCAT DIO module in this demo.
	The network topology shold be built via the ECAT_Utility of ECAT-M801 before running this demo.
    
Created on Fri Feb  5 09:55:37 2021

@author: Eric
"""

from EcatLib import *
import time

def main():
    # scan and open device
    DeviceNo = EcatOpen()
    
    while True:
        # StartOP, most of the I/O and motion functions are available in state OPERATIONAL
        ret = EcatStartOP(DeviceNo)
        if ret != 0: 
            break
            
        # set slave DO / get slave DI
        DIO_Test(DeviceNo) 
    
        # set slave AO / get slave AI
        AIO_Test(DeviceNo)  
        break
    
    # close device
    EcatClose(DeviceNo)
    
def EcatOpen():
    ret, DeviceCnt, CardID = EcatLib.GetDeviceCnt()
    str = "GetDeviceCnt: {}, DeviceCnt: {}".format(ret, DeviceCnt)
    print(AddTimeStr(str))    
    DeviceNo = CardID[0] 
    
    if ret == 0 and DeviceCnt.value > 0:
        DeviceNo = CardID[0]
        ret = EcatLib.OpenDevice(DeviceNo)        
        str = "OpenDevice: {}".format(ret)
        print(AddTimeStr(str)) 
    else:
        DeviceNo = 0
    
    return DeviceNo

def EcatStartOP(DeviceNo):          
    ret = EcatLib.StartDeviceOpTask(DeviceNo, (c_ushort)(0), c_ubyte(0), c_uint(3))
    str = "StartDeviceOpTask: {}".format(ret)
    print(AddTimeStr(str))
    
    if ret < 0:
        return ret
    
    time.sleep(1)
    WaitCnt = 0   
    
    while ( WaitCnt<50):
        ret, LinkUp, SlavesResp, AlStates, Wc = EcatLib.GetDeviceState(DeviceNo)
        if AlStates.value == ECAT_AS_OP:
            print(AddTimeStr("Device state: OP"))
            break;  
        WaitCnt += 1
        if WaitCnt == 50:
            print(AddTimeStr("Wait device state OP timeout!"))
            return -1;
        time.sleep(0.1)
    return 0
    
def DIO_Test(DeviceNo): 
    SetValue = (c_ushort)(55)
    SlaveNo = (c_ushort)(2)
    
    # set value to DO module
    ret = EcatLib.SetSlaveDO(DeviceNo, SlaveNo, SetValue)
    if ret < 0:
        str = "Slave[{}] SetSlaveDO falied: {}".format(SlaveNo, ret)
    else:
        str = "Slave[{}] SetSlaveDO value: {}".format(SlaveNo, SetValue)
    print(AddTimeStr(str))
        
    time.sleep(1)
    
    # get value from DI module
    ret, GetValue = EcatLib.GetSlaveDI(DeviceNo, SlaveNo)
    if ret < 0:
        str = "Slave[{}] GetSlaveDI falied: {}".format(SlaveNo, ret)
    else:
        str = "Slave[{}] GetSlaveDI value: {}".format(SlaveNo, GetValue)
    print(AddTimeStr(str))
    
def AIO_Test(DeviceNo): 
    SetValue = (c_short)(20000)
    SlaveNo = (c_ushort)(1)
    ChannelNo = (c_ubyte)(0)
    
    # set value to AO module
    # Note: This function only supports ECAT-2024 and ECAT-2028
    ret = EcatLib.SetSlaveAoRawData(DeviceNo, SlaveNo, ChannelNo, SetValue)
    if ret < 0:
        str = "Slave[{}] SetSlaveAo falied: {}".format(SlaveNo, ret)
    else:
        str = "Slave[{}] SetSlaveAo value: {}".format(SlaveNo, SetValue.value)
    print(AddTimeStr(str))
        
    time.sleep(1)
    
    SlaveNo = (c_ushort)(0)
    # set value to AI module
    # Note: This function only supports ECAT-2011H and ECAT-2012H
    ret, GetValue = EcatLib.GetSlaveAiRawData(DeviceNo, SlaveNo, ChannelNo)
    if ret < 0:
        str = "Slave[{}] GetSlaveAI falied: {}".format(SlaveNo, ret)
    else:
        str = "Slave[{}] GetSlaveAI value: {}".format(SlaveNo, GetValue.value)
    print(AddTimeStr(str))    
        
def EcatClose(DeviceNo):
    ret = EcatLib.CloseDevice(DeviceNo)
    str = "CloseDevice: {}".format(ret)
    print(AddTimeStr(str))        

def AddTimeStr(str):
    timeStamp = int(time.time())      
    timeArray = time.localtime(timeStamp)
    strtime = time.strftime("%Y-%m-%d %H:%M:%S: ", timeArray) + str + "\n"
    return strtime
    
#
# Run main()
#
if __name__ == "__main__":
	main()    