# -*- coding: utf-8 -*-
"""
NAME
    EcatLib_Mctest.py - a python implementation test code for EtherCAT driver slaves which are controlled by ECAT-M801

DESCRIPTION
    This code test some motion controll related functions in EcatLib. 	
	The network topology shold be built via the ECAT_Utility of ECAT-M801 before running this demo.
    If the number of EtherCAT driver is less than 3, set the other axis to virtaul(SlaveNo: 65535).
    
Created on Fri Feb  5 09:55:37 2021

@author: Eric
"""

from EcatLib import *
import time

def main():
    # scan and open device
    DeviceNo = EcatOpen()
    bStateOp = 0
    
    while True:
        # StartOP, most of the I/O and motion functions are available in state OPERATIONAL
        ret = EcatStartOP(DeviceNo)        
        if ret != 0: 
            break
        bStateOp = 1
        
        # McInit
        ret = EcatMcInit(DeviceNo)
        if ret != 0: 
            break
        
        time.sleep(0.5)
        
         # Home
        ret = McTest_Home(DeviceNo)
        if ret != 0: 
            break
        
        # Init group parameter
        ret = McGroupInit(DeviceNo)
        if ret != 0:
            break
        
        time.sleep(0.5)
        
        # Group LineMove test
        ret = McGroupMoveLine(DeviceNo)
        if ret != 0: 
            break
        
        # Group 2d CircularMove test
        ret = McGroupMoveCircular2D(DeviceNo)
        if ret != 0: 
            break
        
        # Group 3d CircularMove test
        ret = McGroupMoveCircular3D(DeviceNo)
        break
    
    # Stop cyclic operation task
    if (bStateOp == 1):
        EcatStopOP(DeviceNo)
        
    # Close device
    EcatClose(DeviceNo)
    
def EcatOpen():
    ret, DeviceCnt, CardID = EcatLib.GetDeviceCnt()
    str = "GetDeviceCnt: {}, DeviceCnt: {}".format(ret, DeviceCnt.value)
    print(AddTimeStr(str))    
    DeviceNo = CardID[0]
    
    if ret == 0 and DeviceCnt.value > 0:
        DeviceNo = CardID[0]
        ret = EcatLib.OpenDevice(DeviceNo)        
        str = "OpenDevice: {}".format(ret)
        print(AddTimeStr(str)) 
        if ret == 0:
            EcatLib.StopDeviceOpTask(DeviceNo)
            WaitCnt = 0     
            while (WaitCnt<100):
                ret, LinkUp, SlavesResp, AlStates, Wc = EcatLib.GetDeviceState(DeviceNo)
                if AlStates.value == ECAT_AS_PREOP:
                    print(AddTimeStr("Device state: PreOP"))
                    break
                WaitCnt += 1
                if WaitCnt == 100:
                    print(AddTimeStr("Wait device state PreOP timeout!"))
                    ret = -1
                    break
                time.sleep(0.1)        
            if ret != 0 or AlStates.value != ECAT_AS_PREOP:
                ret = -1
                print(AddTimeStr("ECAT_StopDeviceOpTask failed!"))
            ret = EcatLib.SetSlaveNoType(DeviceNo, SLAVE_NO_TYPE_ALIAS_ID)
            if ret != 0:
                str = "Set SlaveNo Type failed, ret: {}".format(ret)
                print(AddTimeStr(str))

    else:
        DeviceNo = 0
    
    return DeviceNo

def EcatStartOP(DeviceNo):          
    ret = EcatLib.StartDeviceOpTask(DeviceNo, (c_ushort)(0), c_ubyte(0), c_uint(3))
    str = "StartDeviceOpTask: {}".format(ret)
    print(AddTimeStr(str))
    
    if ret < 0:
        return ret
    
    time.sleep(1)
    WaitCnt = 0   
    ret = 0
    
    while ( WaitCnt<100):
        ret, LinkUp, SlavesResp, AlStates, Wc = EcatLib.GetDeviceState(DeviceNo)
        if AlStates.value == ECAT_AS_OP:
            print(AddTimeStr("Device state: OP"))
            break;  
        WaitCnt += 1
        if WaitCnt == 100:
            print(AddTimeStr("Wait device state OP timeout!"))
            ret = -1
            break;
        time.sleep(0.1)        
        pass
    
    return ret

def EcatStopOP(DeviceNo):          
    ret = EcatLib.StopDeviceOpTask(DeviceNo)
    str = "StopDeviceOpTask: {}".format(ret)
    print(AddTimeStr(str))
    return ret
    
def EcatMcInit(DeviceNo):
    AxisCnt = (c_ushort)(0)        
    SlaveNo = (c_ushort * MC_AXIS_NO_MAX)(*[0 for x in range(MC_AXIS_NO_MAX)])
    SubAxisNo = (c_ushort * MC_AXIS_NO_MAX)(*[0 for x in range(MC_AXIS_NO_MAX)])

       
    ret, SlaveNo, SubAxisNo, AxisCnt = EcatLib.McInit_Ex(DeviceNo, "./Auto.motcfg".encode())  
    str = "McInitMcInit: {}".format(ret)
    print(AddTimeStr(str))
    if ret < 0:
        return ret
    
    time.sleep(0.5)
    
    for AxisNo in range(0, 3):
        AxisPPU = (c_double)(10000.0)
        ret = EcatLib.McSetAxisPPU(DeviceNo, AxisNo, AxisPPU) 
        str = "Axis[{}] McSetAxisPPU: {}".format(AxisNo, ret) 
        print(AddTimeStr(str))
        if ret < 0: 
            break
        
        while 1:
            ret = EcatLib.McSetAxisServoOn(DeviceNo, AxisNo, 1) 
            # when error occurred on master or motor driver
            # it will take few ms to process error
            # in the meantime, ServoOn or ErrorReset will return -1137

            # P.S. ServoOn also has the function of clearing errors

            if ret == -1137:
                continue
            break

        str = "Axis[{}] McSetAxisServoOn: {}".format(AxisNo, ret) 
        print(AddTimeStr(str))  
        if ret < 0: 
            break
        
        for i in range(10):
            ret, state = EcatLib.McGetAxisState_Ex(DeviceNo, AxisNo)
            if ret == 0 and state.value == MC_AS_STANDSTILL:
                break
            time.sleep(0.1)
        if ret != 0 or state.value != MC_AS_STANDSTILL:
            str = "AxisNo[{}]:Failed to Servo On:ret:{}, State:{}\n".format(AxisNo, ret, state.value) 
            print(AddTimeStr(str))
        
    return ret

def McTest_Home(DeviceNo):
    for AxisNo in range(0, 3):
        ret = EcatLib.McSetAxisHomeMethod(DeviceNo, AxisNo, 35) 
        str = "Axis[{}] McSetAxisHomeMethod: {}".format(AxisNo, ret) 
        print(AddTimeStr(str))
        if ret < 0: 
            return ret
        
    for AxisNo in range(0, 3):
        ret = EcatLib.McAxisHome(DeviceNo, AxisNo) 
        str = "Axis[{}] McAxisHome: {}".format(AxisNo, ret) 
        print(AddTimeStr(str))
        if ret < 0: 
            return ret
    
    time.sleep(0.5)
    while True:
        McFinishCnt = 0
        ret, State = EcatLib.McGetAxisState(DeviceNo, 65535)
        for AxisNo in range(0, 3):
            if State[AxisNo] == MC_AS_STANDSTILL:
                McFinishCnt += 1
                
        if McFinishCnt == 3:
            break 
    return ret

def McGroupInit(DeviceNo):
    GroupNo = (c_ushort)(0)
    for AxisNo in range(0, 3):
        ret = EcatLib.McAddAxisToGroup_Ex(DeviceNo, GroupNo, AxisNo) 
        str = "Axis[{}] McAddAxisToGroup[{}]: {}".format(AxisNo, GroupNo.value, ret) 
        print(AddTimeStr(str))
        if ret < 0: 
            print(AddTimeStr("McTest failed at McAddAxisToGroup"))
            return ret
    
    Time_ms = (c_ushort)(500)
    ret = EcatLib.McSetGroupAccTime_Ex(DeviceNo, GroupNo, Time_ms.value)
    str = "McSetGroupAccTime[{}]: {}".format(GroupNo.value, ret) 
    print(AddTimeStr(str))
    if ret < 0: 
        print(AddTimeStr("McGroupInit failed at McSetGroupAccTime"))
        return ret
        
    CmdMode = (c_ushort)(1) # 0: Aborting, 1: Buffered, 2: Blending
    ret = EcatLib.McSetGroupCmdMode_Ex(DeviceNo, GroupNo.value, CmdMode.value)
    str = "McSetGroupCmdMode[{}]: {}".format(GroupNo, ret) 
    print(AddTimeStr(str))
    if ret < 0: 
        print(AddTimeStr("McGroupInit failed at McSetGroupCmdMode"))
    
    return ret
    
        
def McGroupMoveLine(DeviceNo):
    GroupNo = (c_ushort)(0)
    GroupPos = (c_double * 3)(*[10,10,10])
    GroupVel = (c_double)(2)
    
    ret = EcatLib.McGroupMoveLineAbs_Ex(DeviceNo, GroupNo, GroupPos, GroupVel)
    str = "McGroupMoveLineAbs[{}]: {}".format(GroupNo.value, ret) 
    print(AddTimeStr(str))
    if ret < 0: 
        print(AddTimeStr("McGroupMoveLine failed at McGroupMoveLineAbs"))
        return ret
    
    time.sleep(4)
    
    ret = EcatLib.McGroupStop(DeviceNo, GroupNo);
    str = "McGroupStop[{}]: {}".format(GroupNo.value, ret) 
    print(AddTimeStr(str))
    if ret < 0: 
        print(AddTimeStr("McGroupMoveLine failed at McGroupStop"))
        
    while True:        
        ret, State = EcatLib.McGetGroupState(0, GroupNo)
        str = "GroupState: {}".format(State.value) 
        print(AddTimeStr(str))
        if State.value == MC_GS_STANDBY:
            break
        time.sleep(0.5)
        
    print(AddTimeStr("McGroupMoveLine finished"))
    return 0

def McGroupMoveCircular2D(DeviceNo):
    GroupNo = (c_ushort)(0)
    CircCenterPos  = (c_double * 2)(*[10,0])
    CircAngle = (c_double)(135)
    GroupVel = (c_double)(2)
    
    ret = EcatLib.McGroupMoveCircularRel_CP_Angle_Ex(DeviceNo, GroupNo, GroupVel, CircAngle, CircCenterPos)
    str = "McGroupMoveCircularRel_CP_Angle[{}]: {}".format(GroupNo.value, ret) 
    print(AddTimeStr(str))
    if ret < 0: 
        print(AddTimeStr("McGroupMoveLine failed at McGroupMoveCircularRel_CP_Angle"))
        return ret
    
    while True:
        ret, State = EcatLib.McGetGroupState(DeviceNo, GroupNo)
        if State.value == MC_GS_STANDBY:
            break
        time.sleep(0.1)
        
    print(AddTimeStr("McGroupMoveCircular2D finished"))
    return 0

def McGroupMoveCircular3D(DeviceNo):
    GroupNo = (c_ushort)(0)
    CircCenterPos  = (c_double * 3)(*[3.5355,3.5355,0])
    CircNormalVector  = (c_double * 3)(*[-0.7071,0.7071,0])
    CircAngle = (c_double)(135)
    GroupVel = (c_double)(2)
    
    ret = EcatLib.McGroupMove3DCircularRel_CP_Angle_Ex(DeviceNo, GroupNo, GroupVel, CircAngle, CircCenterPos, CircNormalVector)
    str = "McGroupMove3DCircularRel_CP_Angle[{}]: {}".format(GroupNo.value, ret) 
    print(AddTimeStr(str))
    if ret < 0: 
        print(AddTimeStr("McGroupMoveLine failed at McGroupMove3DCircularRel_CP_Angle"))
        return ret
    
    while True:
        ret, State = EcatLib.McGetGroupState(DeviceNo, GroupNo)
        if State.value == 1:  #MC_GS_STANDBY
            break
        time.sleep(0.1)
        
    print(AddTimeStr("McGroupMoveCircular3D finished"))
    return 0
            
        
def EcatClose(DeviceNo):
    ret = EcatLib.CloseDevice(DeviceNo)
    str = "CloseDevice: {}".format(ret)
    print(AddTimeStr(str))        

def AddTimeStr(str):
    timeStamp = int(time.time())      
    timeArray = time.localtime(timeStamp)
    strtime = time.strftime("%Y-%m-%d %H:%M:%S: ", timeArray) + str + "\n"
    return strtime
    
#
# Run main()
#
if __name__ == "__main__":
	main()    