# -*- coding: utf-8 -*-
"""
NAME
    EcatLib_Mctest.py - a python implementation test code for EtherCAT driver slaves which are controlled by ECAT-M801

DESCRIPTION
    This code test some motion controll related functions in EcatLib. 	
	The network topology shold be built via the ECAT_Utility of ECAT-M801 before running this demo.
    If the number of EtherCAT driver is less than 3, set the other axis to virtaul(SlaveNo: 65535).
    
Created on Fri Feb  5 09:55:37 2021

@author: Eric
"""

from EcatLib import *
import time

def main():
    # scan and open device
    DeviceNo = EcatOpen()
    bStateOp = 0
    
    while True:
        # StartOP, most of the I/O and motion functions are available in state OPERATIONAL
        ret = EcatStartOP(DeviceNo)
        if ret != 0: 
            break
        bStateOp = 1
            
        # McInit
        ret = EcatMcInit(DeviceNo)
        if ret != 0: 
            break
        
        time.sleep(0.5)
        
         # Home
        ret = McTest_Home(DeviceNo)
        if ret != 0: 
            break
        
        time.sleep(0.5)
        
        # Single motion test
        for TestCnt in range(0, 5):
            ret = McTest_Single(DeviceNo)
            if ret != 0: 
                break
            else:
                if TestCnt == 4:
                    print(AddTimeStr("Motion test finished!"))
        break
    
    # Stop cyclic operation task
    if (bStateOp == 1):
        EcatStopOP(DeviceNo)
        
    # Close device
    EcatClose(DeviceNo)
    
def EcatOpen():
    ret, DeviceCnt, CardID = EcatLib.GetDeviceCnt()
    str = "GetDeviceCnt: {}, DeviceCnt: {}".format(ret, DeviceCnt.value)
    print(AddTimeStr(str))    
    DeviceNo = CardID[0] 
    
    if ret == 0 and DeviceCnt.value > 0:
        DeviceNo = CardID[0]
        ret = EcatLib.OpenDevice(DeviceNo)        
        str = "OpenDevice: {}".format(ret)
        print(AddTimeStr(str)) 
        if ret == 0:
            EcatLib.StopDeviceOpTask(DeviceNo)
            WaitCnt = 0     
            while (WaitCnt<100):
                ret, LinkUp, SlavesResp, AlStates, Wc = EcatLib.GetDeviceState(DeviceNo)
                if AlStates.value == ECAT_AS_PREOP:
                    print(AddTimeStr("Device state: PreOP"))
                    break
                WaitCnt += 1
                if WaitCnt == 100:
                    print(AddTimeStr("Wait device state PreOP timeout!"))
                    ret = -1
                    break
                time.sleep(0.1)        
            if ret != 0 or AlStates.value != ECAT_AS_PREOP:
                ret = -1
                print(AddTimeStr("ECAT_StopDeviceOpTask failed!"))
            ret = EcatLib.SetSlaveNoType(DeviceNo, SLAVE_NO_TYPE_ALIAS_ID)
            if ret != 0:
                str = "Set SlaveNo Type failed, ret: {}".format(ret)
                print(AddTimeStr(str))

    else:
        DeviceNo = 0
    
    return DeviceNo

def EcatStartOP(DeviceNo):          
    ret = EcatLib.StartDeviceOpTask(DeviceNo, (c_ushort)(0), c_ubyte(0), c_uint(3))
    str = "StartDeviceOpTask: {}".format(ret)
    print(AddTimeStr(str))
    
    if ret < 0:
        return ret
    
    time.sleep(1)
    WaitCnt = 0   
    ret = 0
    
    while ( WaitCnt<100):
        ret, LinkUp, SlavesResp, AlStates, Wc = EcatLib.GetDeviceState(DeviceNo)
        if AlStates.value == ECAT_AS_OP:
            print(AddTimeStr("Device state: OP"))
            break;  
        WaitCnt += 1
        if WaitCnt == 100:
            print(AddTimeStr("Wait device state OP timeout!"))
            ret = -1
            break;
        time.sleep(0.1)        
        pass
    
    return ret
    
def EcatStopOP(DeviceNo):
    ret = EcatLib.StopDeviceOpTask(DeviceNo)
    str = "StopDeviceOpTask: {}".format(ret)
    print(AddTimeStr(str))
    return ret

def EcatMcInit(DeviceNo):
    AxisCnt = (c_ushort)(0)        
    SlaveNo = (c_ushort * MC_AXIS_NO_MAX)(*[0 for x in range(MC_AXIS_NO_MAX)])
    SubAxisNo = (c_ushort * MC_AXIS_NO_MAX)(*[0 for x in range(MC_AXIS_NO_MAX)])

       
    ret, SlaveNo, SubAxisNo, AxisCnt = EcatLib.McInit_Ex(DeviceNo, "./Auto.motcfg".encode())  
    str = "McInitMcInit: {}".format(ret)
    print(AddTimeStr(str))
    if ret < 0:
        if ret == -1027:
            print(AddTimeStr("Please use ECAT_Utility.exe to create Motion Control Parameter File(.motcfg)"))
        return ret
    
    time.sleep(0.5)  
    
    for AxisNo in range(0, 3):
        state = (c_uint32)(0)
        AxisPPU = (c_double)(10000.0)
        ret = EcatLib.McSetAxisPPU(DeviceNo, AxisNo, AxisPPU) 
        str = "Axis[{}] McSetAxisPPU: {}".format(AxisNo, ret) 
        print(AddTimeStr(str))
        if ret < 0: 
            break
        while 1:
            ret = EcatLib.McSetAxisServoOn(DeviceNo, AxisNo, 1) 
            # when error occurred on master or motor driver
            # it will take few ms to process error
            # in the meantime, ServoOn or ErrorReset will return -1137

            # P.S. ServoOn also has the function of clearing errors

            if ret == -1137:
                continue
            break

        str = "Axis[{}] McSetAxisServoOn: {}".format(AxisNo, ret) 
        print(AddTimeStr(str))  
        if ret < 0: 
            break
        
        for i in range(10):
            ret, state = EcatLib.McGetAxisState_Ex(DeviceNo, AxisNo)
            if ret == 0 and state.value == MC_AS_STANDSTILL:
                break
            time.sleep(0.1)
        if ret != 0 or state.value != MC_AS_STANDSTILL:
            str = "AxisNo[{}]:Failed to Servo On:ret:{}, State:{}\n".format(AxisNo, ret, state.value) 
            print(AddTimeStr(str))
        
    return ret

def McTest_Home(DeviceNo):
    for AxisNo in range(0, 3):
        ret = EcatLib.McSetAxisHomeMethod(DeviceNo, AxisNo, 35) 
        str = "Axis[{}] McSetAxisHomeMethod: {}".format(AxisNo, ret) 
        print(AddTimeStr(str))
        if ret < 0: 
            return ret
        
    for AxisNo in range(0, 3):
        ret = EcatLib.McAxisHome(DeviceNo, AxisNo) 
        str = "Axis[{}] McAxisHome: {}".format(AxisNo, ret) 
        print(AddTimeStr(str))
        if ret < 0: 
            return ret
        
    time.sleep(0.5)
    while True:
        McFinishCnt = 0
        ret, State = EcatLib.McGetAxisState_Ex(DeviceNo, 65535)
        for AxisNo in range(0, 3):
            if State[AxisNo] == MC_AS_STANDSTILL:
                McFinishCnt += 1
                
        if McFinishCnt == 3:
            break        
    return ret

def McTest_Single(DeviceNo):
    AxisVel = (c_double)(10)
    for AxisNo in range(0, 3):
        ret = EcatLib.McAxisMoveVel(DeviceNo, AxisNo, AxisVel) 
        str = "Axis[{}] McAxisMoveVel: {}".format(AxisNo, ret) 
        print(AddTimeStr(str))
        if ret < 0: 
            print(AddTimeStr("McTest failed at McAxisMoveVel"))
            return ret
        
    time.sleep(3)
    
    for AxisNo in range(0, 3):
        ret = EcatLib.McAxisStop(DeviceNo, AxisNo) 
        str = "Axis[{}] McAxisStop: {}".format(AxisNo, ret) 
        print(AddTimeStr(str))
        if ret < 0: 
            print(AddTimeStr("McTest failed at McAxisStop"))
            return ret
            
    while True:
        McFinishCnt = 0
        ret, State = EcatLib.McGetAxisState_Ex(DeviceNo, 65535)
        for AxisNo in range(0, 3):
            if State[AxisNo] == MC_AS_STANDSTILL:
                McFinishCnt += 1
                
        if McFinishCnt == 3:
            break
    
    AxisPos = (c_double)(0)
    AxisVel = (c_double)(10)
    for AxisNo in range(0, 3):
        ret = EcatLib.McAxisMoveAbs(DeviceNo, AxisNo, AxisPos, AxisVel) 
        str = "Axis[{}] McAxisMoveAbs: {}".format(AxisNo, ret) 
        print(AddTimeStr(str))
        if ret < 0: 
            print(AddTimeStr("McTest failed at McAxisMoveAbs"))
            return ret
        
    time.sleep(1)
            
    while 1:
        McFinishCnt = 0
        ret, State = EcatLib.McGetAxisState_Ex(DeviceNo, 65535)
        for AxisNo in range(0, 3):
            if State[AxisNo] == MC_AS_STANDSTILL:
                McFinishCnt += 1
                
        if McFinishCnt == 3:        
            print(AddTimeStr("McAxisMoveAbs finished"))
            break
        
    return 0
            
        
def EcatClose(DeviceNo):
    ret = EcatLib.CloseDevice(DeviceNo)
    str = "CloseDevice: {}".format(ret)
    print(AddTimeStr(str))        

def AddTimeStr(str):
    timeStamp = int(time.time())      
    timeArray = time.localtime(timeStamp)
    strtime = time.strftime("%Y-%m-%d %H:%M:%S: ", timeArray) + str + "\n"
    return strtime
    
#
# Run main()
#
if __name__ == "__main__":
	main()    