# -*- coding: utf-8 -*-
"""
NAME
    EcatLib_test.py - a python implementation test code of EcatLib class 

DESCRIPTION
  This program implements test code in cells that divided by In[*]
  Users can debug code in cell by clicking the "Run current cell" button in Spyder    
  The 1st slave in this demo is the ECAT-2094S, which is an EtherCAT 4-axis stepper motor driver developed by ICPDAS.
  The network topology shold be built via the ECAT_Utility of ECAT-M801 before running this demo.
    
Created on Tue Jan  5 15:27:01 2021

@author: Eric
"""

# In[*]
# Get and open device
from EcatLib import *
from ctypes import *

ret, DeviceCnt, CardID = EcatLib.GetDeviceCnt()
str = "GetDeviceCnt: {}, DeviceCnt: {}".format(ret, DeviceCnt)
print(str)
str = "CardID: "
for el in CardID:
    str += "{}, ".format(el)
print(str)

if ret == 0 and DeviceCnt.value > 0:     
    ret = EcatLib.OpenDevice(CardID[0])        
    str = "OpenDevice: {}".format(ret)
    print(str)

# In[*]
# Close device
ret = EcatLib.CloseDevice(0)        
str = "CloseDevice: {}".format(ret)
print(str)

# In[*]
# StartOP, most of the I/O and motion functions are available in state OPERATIONAL
ret = EcatLib.StartDeviceOpTask(0, (c_ushort)(0), c_ubyte(0), c_uint(3))
str = "StartDeviceOpTask: {}".format(ret)
print(str)

# In[*]
# Get device state
ret, LinkUp, SlavesResp, AlStates, Wc = EcatLib.GetDeviceState(0)
str = "ret: {}, LinkUp: {}, SlavesResp: {}, AlStates: {}, Wc: {}".format(ret, LinkUp, SlavesResp, AlStates, Wc)
print(str)

# Get slave info
for SlaveNo in range(0, SlavesResp.value):
    ret, Alias, ProductCode, VendorID, RevisionNo, SerialNo, AlState, SlaveType, Slavename = EcatLib.GetSlaveInfo(0, SlaveNo)
    str = "Slave[{}]->  Alias: {}, ProductCode: {}, VendorID: {}, RevisionNo: {}, SerialNo: {}, AlState: {}, SlaveType: {}, \
    Slavename: {}".format(SlaveNo, Alias.value, ProductCode.value, VendorID.value, RevisionNo.value, SerialNo.value, AlState.value, SlaveType.value, Slavename.value)
    print(str)

# In[*]
# McInit is necessary before motion control
SlaveNo = (c_ushort * 4)(*[0,0,0,0])
SubAxisNo = (c_ushort * 4)(*[0,1,2,3])
AxisCnt = 4
ret = EcatLib.McInit(0, SlaveNo, SubAxisNo, AxisCnt)  
str = "McInitMcInit: {}".format(ret)
print(str) 
 
# In[*]
# Set axis servo on
for AxisNo in range(0, 4):
    ret = EcatLib.McSetAxisServoOn(0, AxisNo, 1) 
    str = "Axis[{}] McSetAxisServoOn: {}".format(AxisNo, ret) 
    print(str)
    
# In[*]
# Set axis servo off
for AxisNo in range(0, 4):
    ret = EcatLib.McSetAxisServoOn(0, AxisNo, 0) 
    str = "Axis[{}] McSetAxisServoOn: {}".format(AxisNo, ret) 
    print(str) 
    
# In[*]
# Set axis home method
for AxisNo in range(0, 4):
    ret = EcatLib.McSetAxisHomeMethod(0, AxisNo, 35) 
    str = "Axis[{}] McSetAxisHomeMethod: {}".format(AxisNo, ret) 
    print(str) 
    
# In[*]
# Start home motion of an axis
for AxisNo in range(0, 4):
    ret = EcatLib.McAxisHome(0, AxisNo) 
    str = "Axis[{}] McAxisHome: {}".format(AxisNo, ret) 
    print(str) 
    
# In[*]
# Start a never ending movement with a specified velocity
for AxisNo in range(0, 4):
    ret = EcatLib.McAxisMoveVel(0, AxisNo, (c_double)(1000))
    str = "Axis[{}] McAxisMoveVel: {}".format(AxisNo, ret) 
    print(ret)
    
# In[*]
# Stop an axis with deceleration
for AxisNo in range(0, 4):
    ret = EcatLib.McAxisStop(0, AxisNo)
    str = "Axis[{}] McAxisStop: {}".format(AxisNo, ret) 
    print(ret)
    
# In[*]
# Start an absolute position motion of an axis
AxisPos = (c_double)(21600)
AxisVel = (c_double)(1000)
for AxisNo in range(0, 4):
    ret = EcatLib.McAxisMoveAbs(0, AxisNo, AxisPos, AxisVel) 
    str = "Axis[{}] McAxisMoveAbs: {}".format(AxisNo, ret) 
    print(ret)
    
# In[*]
# Get postion, velocity and state of axes
ret, Pos = EcatLib.McGetAxisActualPos(0, 65535)
ret, Vel = EcatLib.McGetAxisActualVel(0, 65535)
ret, State = EcatLib.McGetAxisState(0, 65535)

for AxisNo in range(0, 4):   
    str = "Axis[{}] pos: {:.2f}".format(AxisNo, Pos[AxisNo]) 
    print(str)

for AxisNo in range(0, 4):   
    str = "Axis[{}] vel: {:.2f}".format(AxisNo, Vel[AxisNo]) 
    print(str)

for AxisNo in range(0, 4):   
    str = "Axis[{}] state: {}".format(AxisNo, State[AxisNo]) 
    print(str)    
    
# In[*]
# Get SDO data of a slave. Read a data object by means of service data object communication.
ret, ObjectVal, AbortCode = EcatLib.GetSlaveSdoObject(0, (c_ushort)(0), (c_ushort)(0x8010), (c_ubyte)(1), (c_ushort)(2))
str = "GetSlaveSdoObject: ret[{}], ObjectVal[{}], AbortCode[{}]".format(ret, ObjectVal.value, AbortCode.value) 
print(str)

# In[*]
# Set SDO data of a slave. Write a data object by means of service data object communication.
ObjectVal = (c_uint)(1)
ret, AbortCode = EcatLib.SetSlaveSdoObject(0, (c_ushort)(0), (c_ushort)(0x8010), (c_ubyte)(1), (c_ushort)(2), ObjectVal)
str = "SetSlaveSdoObject: ret[{}], ObjectVal[{}], AbortCode[{}]".format(ret, ObjectVal.value, AbortCode.value) 
print(str)

# In[*]
exit(0)
print("exit")
