﻿"""

	NAME
		EcatLib.py - ECAT-M801 control API function prototypes & macros for python

	DESCRIPTION
		Include this class library in your python source to provide prototypes
		for the control API functions of ECAT-M801.

		from EcatLib.py import *

"""

from ctypes import *
import os
#import numpy as np

#
# Max Macro
#
DEV_NETWORK_INFO_NO_MAX		= 8
MC_AXIS_NO_MAX				= 64
MC_AXIS_NO_LIMIT			= 64
MC_GROUP_NO_MAX				= 8
MC_AXIS_ADVANCE_NO_MAX		= 4
MC_GROUP_CMD_BUFFER_MAX		= 7000
MC_PROFILE_NO_MAX			= 16
MC_PROFILE_DATA_MAX			= 3000
RW_PDO_DATA_SIZE_MAX		= 512
NETWORK_SLAVE_MAX			= 80
CARD_DEVICE_NO_MAX			= 16
DEVICE_SERIAL_NO_SIZE		= 8
Motiondata_Max				= 100000
FILE_BUFFER_SIZE			= 512
PID_NO_MAX					= 10
EVENT_NO_MAX				= 32
RECORDDATA_GET_COUNT_MAX	= 64
MAX_SLAVE_NAME_LENGTH		= 64
MC_PATH_DATA_NO_MAX			= 3
MC_PATH_DATA_ARGS_MAX		= 5
PDO_BUFFER_CHANNEL_MAX		= 3
PDO_BUFFER_DATA_MAX			= 128
MC_AXIS_CMD_BUFFER_MAX		= 100
AI_FILTER_CHANNEL_MAX		= 8
PDO_INTOOUT_CHANNEL_MAX		= 8
LACK_PDOS_GET_COUNT_MAX		= 16
ENC_BUFFER_CHANNEL_MAX		= 6
ENC_BUFFER_MAX				= 128
MULTI_SLAVE_DO_MAX			= 32
SLAVE_DIO_MAP_MAX			= 32
SLAVE_DIO_MAX_EX			= 64
SLAVE_AIO_CHANNEL_MAX		= 16
#
# Slave Type
#
SLAVE_TYPE_GENERIC				= 0				# Generic
SLAVE_TYPE_CiA402				= 1				# Profile No.402
SLAVE_TYPE_STEPPER_MOTOR		= 2				# ICPDAS 1 Axis Stepper Motor
SLAVE_TYPE_4_AXIS_STEPPER_MOTOR	= 3				# ICPDAS 4 Axis Stepper Motor
SLAVE_TYPE_2610					= 4				# ICPDAS Modbus To Ethercat
#
# Slave AO range code
#
SLAVE_AO_UNI_5V				= 0					# Unipolar 5V (0-5V)
SLAVE_AO_BI_5V				= 1					# Bipolar 5V (+-5V)
SLAVE_AO_UNI_10V			= 2					# Unipolar 10V (0-10V)
SLAVE_AO_BI_10V				= 3					# Bipolar 10V (+-10V)
#
# Slave AI range code
#
SLAVE_AI_BI_10V				= 0					# Bipolar 10V (+-10V)
SLAVE_AI_BI_5V				= 1					# Bipolar 5V (+-5V)
SLAVE_AI_BI_2_5V			= 2					# Bipolar 2.5V (+-2.5V)
SLAVE_AI_UNI_10V			= 3					# Unipolar 10V (0-10V)
SLAVE_AI_UNI_20mA			= 4					# Unipolar 20mA (0-20mA)
SLAVE_AI_UNI_4_20mA			= 5					# Unipolar 4-20mA(4-20mA)
SLAVE_AI_BI_20mA			= 6					# Bipolar 20mA(+-20mA)
SLAVE_AI_BI_4_20mA			= 7					# Bipolar 4-20mA(+-4-20mA)
SLAVE_AI_BI_10V_UNI_20mA	= 8					# ch0~3 Bipolar 10V (+-10V) ch4~7 Unipolar 20mA (0-20mA)
SLAVE_AI_BI_10V_UNI_4_20mA	= 9					# ch0~3 Bipolar 10V (+-10V) ch4~7 Unipolar 4-20mA(4-20mA)
SLAVE_AI_BI_10V_BI_20mA		= 10				# ch0~3 Bipolar 10V (+-10V) ch4~7 Bipolar 20mA(+-20mA)
SLAVE_AI_BI_10V_BI_4_20mA	= 11				# ch0~3 Bipolar 10V (+-10V) ch4~7 Bipolar 4-20mA(+-4-20mA)
#
# ECAT2016N AI range code
#
ECAT2016_AI_BI_1_25V    = 0		 		# Bipolar 1.25V (+-1.25V)
ECAT2016_AI_BI_600mV    = 1 			# Bipolar 600mV (+-600mV)
ECAT2016_AI_BI_300mV    = 2				# Bipolar 300mV (+-300mV)
ECAT2016_AI_BI_125mV    = 3				# Bipolar 125mV (+-125mV)
ECAT2016_AI_BI_80mV     = 4				# Bipolar 80mV (+-80mV)
ECAT2016_AI_BI_60mV	    = 5				# Bipolar 60mV (+-60mV)
ECAT2016_AI_BI_40mV	    = 6				# Bipolar 40mV (+-40mV)
ECAT2016_AI_BI_30mV	    = 7				# Bipolar 30mV (+-30mV)
ECAT2016_AI_BI_20mV	    = 8				# Bipolar 20mV (+-20mV)
ECAT2016_AI_BI_15mV	    = 9				# Bipolar 15mV (+-15mV)
ECAT2016_AI_BI_10mV	    = 10			# Bipolar 10mV (+-10mV)
#
# ECAT2016N-3 AI range code
#
ECAT2016_3_AI_BI_10V       = 0			# Bipolar 10V (+-10V)
ECAT2016_3_AI_BI_5V		   = 1			# Bipolar 5V (+-5V)
ECAT2016_3_AI_BI_2_5V      = 2			# Bipolar 2.5V (+-2.5V)
ECAT2016_3_AI_BI_1_25V     = 3			# Bipolar 1.25V (+-1.25V) 
ECAT2016_3_AI_BI_625mV     = 4			# Bipolar 625mV (+-625mV)
ECAT2016_3_AI_BI_312mV     = 5			# Bipolar 312mV (+-312mV)
ECAT2016_3_AI_BI_200mV     = 6			# Bipolar 200mV (+-200mV)
ECAT2016_3_AI_BI_100mV     = 7			# Bipolar 100mV (+-100mV)
ECAT2016_3_AI_BI_50mV      = 8			# Bipolar 50mV (+-50mV)
ECAT2016_3_AI_BI_25mV	   = 9			# Bipolar 25mV (+-25mV)
#
# EtherCAT AL State
#
ECAT_AS_INIT				= 0x00
ECAT_AS_PREOP				= 0x02
ECAT_AS_SAFEOP				= 0x04
ECAT_AS_OP					= 0x08
#
# Device OP task cycle time
#
DEV_OP_CYCLE_TIME_1MS		= 0					# 1ms
DEV_OP_CYCLE_TIME_2MS		= 1					# 2ms
DEV_OP_CYCLE_TIME_3MS		= 2					# 3ms
DEV_OP_CYCLE_TIME_4MS		= 3					# 4ms
DEV_OP_CYCLE_TIME_5MS		= 4					# 5ms
DEV_OP_CYCLE_TIME_6MS		= 5					# 6ms
DEV_OP_CYCLE_TIME_7MS		= 6					# 7ms
DEV_OP_CYCLE_TIME_8MS		= 7					# 8ms
DEV_OP_CYCLE_TIME_9MS		= 8					# 9ms
DEV_OP_CYCLE_TIME_10MS		= 9					# 10ms
DEV_OP_CYCLE_TIME_11MS		= 10				# 11ms
DEV_OP_CYCLE_TIME_12MS		= 11				# 12ms
DEV_OP_CYCLE_TIME_13MS		= 12				# 13ms
DEV_OP_CYCLE_TIME_14MS		= 13				# 14ms
DEV_OP_CYCLE_TIME_15MS		= 14				# 15ms
DEV_OP_CYCLE_TIME_16MS		= 15				# 16ms
#
# Motion Control Group command mode
#
MS_GRP_CM_ABORTING			= 0
MS_GRP_CM_BUFFERED			= 1
MS_GRP_CM_BLENDING			= 2
#
# Motion Control Axis Synchronization source
#
MC_AXIS_SYNC_SOURCE_SET_VALUE		= 0
MC_AXIS_SYNC_SOURCE_ACTUAL_VALUE	= 1
#
# Motion Control Axis State
#
MC_AS_DISABLED				= 0
MC_AS_STANDSTILL			= 1
MC_AS_ERRORSTOP				= 2
MC_AS_STOPPING				= 3
MC_AS_HOMING				= 4
MC_AS_DISCRETEMOTION		= 5
MC_AS_CONTINUOUSMOTION		= 6
MC_AS_SYNCHRONIZEDMOTION	= 7
MC_AS_STEERWHEELMOTION		= 8
#
# Motion Control Axis State
#
MC_AS_CONSTANT_VEL			= 0
MC_AS_ACC					= 1
MC_AS_DEC					= 2
#
# Motion Control Motion State
#
MC_AS_CONSTANT_VEL			= 0
MC_AS_ACC					= 1
MC_AS_DEC					= 2
#
# Motion Control Group State
#
MC_GS_DISABLED				= 0
MC_GS_STANDBY				= 1
MC_GS_ERRORSTOP				= 2
MC_GS_STOPPING				= 3
MC_GS_HOMING				= 4
MC_GS_MOVING				= 5
MC_GS_HOLD					= 6
MC_GS_PAUSE					= 7
#
# Device Encoder Mode
#
DEV_ENC_MODE_CW_CCW			= 1
DEV_ENC_MODE_PLS_DIR		= 2
DEV_ENC_MODE_AB_PHASE		= 3
#
# Device Encoder Low Pass Filter
#
DEV_ENC_LPF_4_MHZ			= 0
DEV_ENC_LPF_3P6_MHZ			= 1
DEV_ENC_LPF_1P8_MHZ			= 2
DEV_ENC_LPF_950_KHZ			= 4
DEV_ENC_LPF_480_KHZ			= 8
DEV_ENC_LPF_240_KHZ			= 16
DEV_ENC_LPF_120_KHZ			= 32
DEV_ENC_LPF_60_KHZ			= 64
DEV_ENC_LPF_30_KHZ			= 128
#
# Device emergency source
#
DEV_EMG_SOURCE_OB_DI		= 0					# On board DI
DEV_EMG_SOURCE_SLAVE_DI		= 1					# Slave DI
#
# Motion record parameters
#
MC_RECORD_POSITION			= 0
MC_RECORD_VELOCITY			= 1
MC_RECORD_COMMAND_POSITION	= 2
MC_RECORD_COMMAND_VELOCITY	= 3
#
# Event Type
#
EV_COMPARE_POSITION			= 1
EV_DIBIT_CHANGE				= 2
EV_MOTION_CPMPLETE			= 3
EV_DI_CHANGE				= 4
EV_MOTION_STATUS			= 5
EV_COMPARE_AI				= 6
#
# Comparison Operators
#
GREATER_THAN				= 0
GREATER_THAN_OR_EQUAL_TO	= 1
LESS_THAN					= 2
LESS_THAN_OR_EQUAL_TO		= 3
#
# Home Settings
#
MC_AS_HOME_SPEED_SW			= 1
MC_AS_HOME_SPEED_ZR			= 2
MC_AS_HOME_ACC				= 4
MC_AS_HOME_OFFSET			= 8
#
# Path command type
#
MC_PATH_CMD_TYPE_MOVE_LINE				= 1
MC_PATH_CMD_TYPE_MOVE_CIRC_CP_ANGLE		= 2
MC_PATH_CMD_TYPE_MOVE_CIRC_CP_EP		= 3
MC_PATH_CMD_TYPE_MOVE_CIRC_BP_EP		= 4
MC_PATH_CMD_TYPE_MOVE_3D_CIRC_CP_ANGLE	= 5
MC_PATH_CMD_TYPE_MOVE_3D_CIRC_CP_EP		= 6
MC_PATH_CMD_TYPE_MOVE_3D_CIRC_BP_EP		= 7
MC_PATH_CMD_TYPE_MOVE_HELICAL			= 8
MC_PATH_CMD_TYPE_MOVE_3D_HELICAL		= 9
MC_PATH_CMD_TYPE_MOVE_CONICAL_HELIX		= 10
MC_PATH_CMD_TYPE_MOVE_3D_CONICAL_HELIX	= 11
MC_PATH_CMD_TYPE_SET_ACCDEC_TIME		= 12
MC_PATH_CMD_TYPE_SET_ACCDEC_TYPE		= 13
MC_PATH_CMD_TYPE_SET_CMD_MODE			= 14
MC_PATH_CMD_TYPE_SET_BLEND_PERCENT		= 15
MC_PATH_CMD_TYPE_DWELL					= 16
MC_PATH_CMD_TYPE_SET_DO					= 17
MC_PATH_CMD_TYPE_SET_AO_VOLT			= 18
#
# Acceleration deceleration mode
#
MC_ACC_DEC_MODE_RATE		= 0
MC_ACC_DEC_MODE_TIME		= 1
#
# Filter type
#
FILTER_NOTCH				= 1
FILTER_LOW_PASS				= 2
FILTER_HIGH_PASS			= 4
#
# Slave No Type
#
SLAVE_NO_TYPE_POSITION		= 0					# Position as SlaveNo
SLAVE_NO_TYPE_ALIAS			= 1					# Alias as SlaveNo
SLAVE_NO_TYPE_ALIAS_ID			= 2					# Switch as SlaveNo
#
# Group interpolation Type
#
MC_DEFAULT_INTERPOLATION	= 0
MC_POLAR_INTERPOLATION		= 1
#
# Slave Register read type
#
REGISTER_TYPE_ERR			= 0					# Rx Err / Crc Err
REGISTER_TYPE_FWD_CRC_ERR	= 1					# Forward Crc Err
REGISTER_TYPE_LOST_LINK		= 2					# Lost Link
#
# Slave Register read type
#
MC_MODE_CSP					= 8					# Cyclic synchronous position mode
MC_MODE_CSV					= 9					# Cyclic synchronous velocity mode
MC_MODE_CST					= 10				# Cyclic synchronous torque mode
#
# Error codes
#
ECAT_ERR_REQUEST_MASTER				= -1001					# Request Master
ECAT_ERR_ETHERNET_LINK_DOWN			= -1002					# Ethernet link is down
ECAT_ERR_SLAVES_STATE				= -1003					# All slaves al state != OP
ECAT_ERR_WORKING_COUNTER			= -1004					# Working counter
ECAT_ERR_SLAVE_CNT_EXCEEDED			= -1005					# Slave count exceed maximum support count
ECAT_ERR_CREATE_DOMAIN				= -1006					# Create domain
ECAT_ERR_ALLOCATE_SLAVE_DATA		= -1007					# Allocate slave data
ECAT_ERR_CONFIG_SLAVE				= -1008					# Configure slave
ECAT_ERR_NETWORK_MISMATCH			= -1009					# Network configuration mismatch
ECAT_ERR_MASTER_ACTIVATE			= -1010					# Master activate
ECAT_ERR_GET_PROCESS_DATA			= -1011					# Get ProcessData
ECAT_ERR_CONFIG_CYCLIC_TASK			= -1012					# Configure cyclic task
ECAT_ERR_RUN_CYCLIC_TASK			= -1013					# Run cyclic task
ECAT_ERR_INVALID_SLAVE_TYPE			= -1014					# Invalid slave type
ECAT_ERR_SAME_SLAVE_NO				= -1015					# Same slave number
ECAT_ERR_INVALID_SLAVE_NO			= -1016					# Invalid slave number
ECAT_ERR_INVALID_PARAM				= -1017					# Invalid input parameter
ECAT_ERR_INVALID_DATA_SIZE			= -1018					# Invalid data size
ECAT_ERR_SDO_REQUEST_BUSY			= -1019					# Sdo request busy
ECAT_ERR_SDO_REQUEST_ERROR			= -1020					# Sdo request error
ECAT_ERR_ALLOCATE_PDO_QUEUE			= -1021					# Allocate PDO queue
ECAT_ERR_INVALID_OFFSET				= -1022					# Invalid offset
ECAT_ERR_INIT_MOTION				= -1023					# Initialize motion data
ECAT_ERR_GET_SLAVE_INFO				= -1024					# Get slave information
ECAT_ERR_OPEN_FILE					= -1025					# open file
ECAT_ERR_WRITE_FILE					= -1026					# write data to file
ECAT_ERR_READ_FILE					= -1027					# read data from file
ECAT_ERR_FUNC_NOT_SUPPORT			= -1028					# Function not support
ECAT_ERR_INVALID_CHANNEL			= -1029					# Invalid channel
ECAT_ERR_EMG_HAPPENED				= -1030					# EMG happened
ECAT_ERR_INVALID_PID_NO				= -1031					# Invalid PID number
ECAT_ERR_TIMER_NOT_ACTIVATED		= -1032					# Timer not activated
ECAT_ERR_ALL_EVENT_CREATE			= -1033					# All events are created
ECAT_ERR_EVENT_NOT_CREATE			= -1034					# EventID is not created
ECAT_ERR_INVALID_EVENTID			= -1035					# Invalid EventID
ECAT_ERR_INVALID_FILTER_TYPE		= -1036					# Invalid filter type
ECAT_ERR_SLAVES_ALIAS				= -1037					# repeating alias or alias == 0
ECAT_ERR_SLAVES_ALIAS_NOT_EXIST		= -1038					# alias is not exist
ECAT_ERR_OPTASK						= -1039					# start op task
ECAT_ERR_SLAVES_REQUEST_BUSY		= -1040					# slave request busy
ECAT_ERR_SLAVES_REQUEST_ERROR		= -1041					# slave request error
ECAT_ERR_ALL_BUFFER_USED			= -1042					# all buffer are used
ECAT_ERR_BUFFER_NOT_ENABLE			= -1043					# buffer not enable
ECAT_ERR_CYCLETIME_EXCEEDED			= -1044					# cycle time lower than minimum support
ECAT_ERR_SLAVE_NOT_CONFIGURED		= -1045					# slave not configured
ECAT_ERR_DEACTIVATE_SLAVES			= -1046					# slaves changing state from OP to PreOP
ECAT_ERR_MASTER_BUSY_SCANNING		= -1047					# Master Busy Scanning
ECAT_ERR_SLAVE_EEPROM_ERROR			= -1048					# Slave EEPROM error

ECAT_ERR_MC_NOT_ENABLE_DC					= -1100					# Slave not enable dc
ECAT_ERR_MC_TIME_OUT						= -1101					# Motion control timeout
ECAT_ERR_MC_AXIS_CNT_EXCEEDED				= -1102					# axis count exceed maximum support count
ECAT_ERR_MC_NOT_INITIALIZED					= -1103					# Motion not initialized
ECAT_ERR_MC_INVALID_AXIS_NO					= -1104					# Invalid axis number
ECAT_ERR_MC_NOT_AXIS_SERVO_ON				= -1105					# The axis's servo is not on
ECAT_ERR_MC_INVALID_AXIS_STATE 				= -1106					# Invalid axis state
ECAT_ERR_MC_DRIVE_FAULT 					= -1107					# Drive fault occurred
ECAT_ERR_MC_DRIVE_WARNING 					= -1108					# Drive warning occurred
ECAT_ERR_MC_INVALID_PARAM 					= -1109					# Invalid input parameter
ECAT_ERR_MC_HOMING 							= -1110					# Homing error
ECAT_ERR_MC_LIMIT_ACTIVE 					= -1111					# Limit active
ECAT_ERR_MC_INVALID_ACC_TIME 				= -1112					# Invalid Acc Time
ECAT_ERR_MC_INVALID_GROUP_NO 				= -1113					# Invalid group number
ECAT_ERR_MC_INVALID_GROUP_STATE 			= -1114					# Invalid group state
ECAT_ERR_MC_AXIS_WAS_IN_GROUP 				= -1115					# Invalid group state
ECAT_ERR_MC_AXIS_IN_OTHER_GROUP 			= -1116					# Axis was in group
ECAT_ERR_MC_GROUP_CMD_ALLOCATE 				= -1117					# Allocate group command
ECAT_ERR_MC_GROUP_CMD_BUFFER_OVERFLOW 		= -1118					# group command buffer overflow
ECAT_ERR_MC_INVALID_AXIS_SYNC_MODE 			= -1119					# Invalid axis synchronization mode
ECAT_ERR_MC_INVALID_PROFILE_NO 				= -1120					# Invalid profile number
ECAT_ERR_MC_INVALID_GROUP_MOVE_CMD 			= -1121					# Invalid group move command
ECAT_ERR_MC_GROUP_CMD_MODE_NOT_SUPPORT 		= -1122					# Group command mode not support
ECAT_ERR_MC_INVALID_ACC_DEC_TYPE 			= -1123					# Invalid Acc Dec Type
ECAT_ERR_MC_INVALID_VEL 					= -1124					# Invalid input velocity
ECAT_ERR_MC_INVALID_ANGLE 					= -1125					# Invalid input angle
ECAT_ERR_MC_INVALID_RADIUS 					= -1126					# Invalid input radius
ECAT_ERR_MC_INVALID_END_POS 				= -1127					# Invalid input end position
ECAT_ERR_MC_INVALID_ECAM_TABLE_NO 			= -1128					# Invalid table number
ECAT_ERR_MC_INVALID_NORMAL_VECTOR 			= -1129					# Invalid normal vector
ECAT_ERR_MC_NOT_SETUP 						= -1130					# stewart platform not setup
ECAT_ERR_MC_GREATER_THAN_MAX_RODLENGTH 		= -1131					# greater than max rodlength
ECAT_ERR_MC_LESS_THAN_RODLENGTH 			= -1132					# less than rodlength
ECAT_ERR_MC_GREATER_THAN_RECORD_COUNT 		= -1133					# greater than record count
ECAT_ERR_MC_SOFTWARE_LIMIT_ACTIVE 			= -1134					# Position Software Limit active
ECAT_ERR_MC_GANTRY_POS_EXCESSIVE_DEVIATION 	= -1135					# Gantry Position deviation greater than set value
ECAT_ERR_MC_GROUP_NO_NOT_SUPPORT 			= -1136					# Group number not support
ECAT_ERR_MC_INVALID_MOVE_CMD 				= -1137					# Invalid move command
ECAT_ERR_MC_QUEUE_IS_FULL 					= -1138					# Queue is full
ECAT_ERR_MC_COORDINATE_TRANS_ON 			= -1139					# Coordinate transform is on
ECAT_ERR_MC_NOT_SET_BEFORE 					= -1140					# not set before
ECAT_ERR_MC_HOMIE_NOT_DONE 					= -1141					# Homing not done
ECAT_ERR_MC_INHIBITED_FUNCTION 				= -1142					# inhibited functions
ECAT_ERR_MC_LACK_PDOS						= -1143					# Lack of necessary pdos
ECAT_ERR_MC_SAFETY_STOP						= -1144					# Force by other axis etc.-1004
ECAT_ERR_MC_GANTRY_SYNC_ERR					= -1145					# Master or slave error / servo off / homing

ECAT_ERR_IPC_INVALID_DEVICE_NO 		= -1201					# Invalid device No
ECAT_ERR_IPC_DEVICE_IS_OPEN 		= -1202					# Device is open
ECAT_ERR_IPC_DEVICE_NOT_OPEN 		= -1203					# Device not open
ECAT_ERR_IPC_CREATE_HANDLE 			= -1204					# Create IPC handle object
ECAT_ERR_IPC_BUSY 					= -1205					# IPC busy
ECAT_ERR_IPC_TIME_OUT 				= -1206					# IPC time out
ECAT_ERR_IPC_INVALID_CMD 			= -1207					# Invalid command No
ECAT_ERR_IPC_WRITE_SHM 				= -1208					# write data to shared memory
ECAT_ERR_IPC_READ_SHM 				= -1209					# read data frome shared memory
ECAT_ERR_IPC_RUN_DOWN_UP_LOAD 		= -1210					# IPC Process run Download/Upload Command
ECAT_ERR_IPC_INVALID_SHM 			= -1211					# Invalid shared memory
ECAT_ERR_IPC_DEVICE_NOT_READY 		= -1212					# IPC not ready

ECAT_ERR_DRV_GET_INFO 				= -1301					# Get driver information
ECAT_ERR_DRV_CREATE_HANDLE 			= -1302					# Create driver handle object
ECAT_ERR_DRV_IOCTL 					= -1303					# Driver I/O control
ECAT_ERR_DRV_DEVICE_NOT_FOUND 		= -1304					# Cannot find device
ECAT_ERR_IXE_VERSION 				= -1305					# Cannot read/write ixecat version
ECAT_ERR_DRV_DEVICE_BEEN_OPENED		= -1306					# Device has been opened by other process

ECAT_ERR_ST_NOT_POSITION_MODE 		= -1401					# Not in the position mode
ECAT_ERR_ST_TRIP_ALARM 				= -1402					# trips a alarm
ECAT_ERR_ST_INVALID_TABLE_NO 		= -1403					# Invalid Table No
ECAT_ERR_ST_LIB_NOT_EXIST 			= -1404					# wheelControl lib not exist
ECAT_ERR_ST_NOT_DISABLE_MODE 		= -1405					# Not in the disable mode

if os.name == "nt":
	dll = windll.LoadLibrary("/icpdas/Ecat-M801/API/x64/libecatdevice.dll")
else:
	dll = cdll.LoadLibrary("/opt/icpdas/ecat_m801/lib/libecatdevice.so")

#dll = windll.LoadLibrary("libecatdevice.dll")  # 載入連結庫

#
# The main EcatLib object (all the public access is in this class)
#
class EcatLib:
	# EtherCAT master card control API 
	def __init__(self):     
		#
		# EcatLib function argument and return type declarations. It would be nice if these could be in their 
		# respective member function but there is about a 12% performance penalty if they are run every time a
		# function is called (because python is interpreted). In __init__ they only run once.
		#
		dll.ECAT_GetDllVersion.argtypes = [c_char_p, POINTER(c_ushort)]
		dll.ECAT_GetDllVersion.restype = c_int
		dll.ECAT_GetFirmwareVersion.argtypes = [c_ushort, c_char_p, POINTER(c_ushort)]
		dll.ECAT_GetFirmwareVersion.restype = c_int
		dll.ECAT_GetDeviceCnt.argtypes = [POINTER(c_ushort), POINTER(c_ubyte)]
		dll.ECAT_GetDeviceCnt.restype = c_int
		dll.ECAT_OpenDevice.argtypes = [c_ushort]
		dll.ECAT_OpenDevice.restype = c_int        
		dll.ECAT_CloseDevice.argtypes = [c_ushort]
		dll.ECAT_CloseDevice.restype = c_int
		dll.ECAT_GetDeviceSerialNo.argtypes = [c_ushort, POINTER(c_ubyte)]
		dll.ECAT_GetDeviceSerialNo.restype = c_int
		dll.ECAT_GetDeviceDI.argtypes = [c_ushort, POINTER(c_uint)]
		dll.ECAT_GetDeviceDI.restype = c_int
		dll.ECAT_GetDeviceDIBit.argtypes = [c_ushort, c_ushort, POINTER(c_uint)]
		dll.ECAT_GetDeviceDIBit.restype = c_int           
		dll.ECAT_GetDeviceDO.argtypes = [c_ushort, POINTER(c_uint)]
		dll.ECAT_GetDeviceDO.restype = c_int
		dll.ECAT_GetDeviceDOBit.argtypes = [c_ushort, c_ushort, POINTER(c_uint)]
		dll.ECAT_GetDeviceDOBit.restype = c_int        
		dll.ECAT_SetDeviceDO.argtypes = [c_ushort, c_uint]
		dll.ECAT_SetDeviceDO.restype = c_int
		dll.ECAT_SetDeviceDOBit.argtypes = [c_ushort, c_ushort, c_uint]
		dll.ECAT_SetDeviceDOBit.restype = c_int        
		dll.ECAT_GetDeviceState.argtypes = [c_ushort, c_ushort, POINTER(c_uint), POINTER(c_uint), POINTER(c_uint), POINTER(c_uint)]
		dll.ECAT_GetDeviceState.restype = c_int
		dll.ECAT_GetSlaveInfo.argtypes = [c_ushort, c_ushort, POINTER(c_ushort), POINTER(c_uint), POINTER(c_uint), POINTER(c_uint), POINTER(c_uint), POINTER(c_ubyte), POINTER(c_uint), POINTER(c_char_p)]
		dll.ECAT_GetSlaveInfo.restype = c_int                
		dll.ECAT_GetSlaveSdoObject.argtypes = [c_ushort, c_ushort, c_ushort, c_ubyte, c_ushort, POINTER(c_uint), POINTER(c_uint)]
		dll.ECAT_GetSlaveSdoObject.restype = c_int
		dll.ECAT_SetSlaveSdoObject.argtypes = [c_ushort, c_ushort, c_ushort, c_ubyte, c_ushort, c_uint, POINTER(c_uint)]
		dll.ECAT_SetSlaveSdoObject.restype = c_int
		dll.ECAT_StartDeviceOpTask.argtypes = [c_ushort, c_ushort, c_ubyte, c_uint]
		dll.ECAT_StartDeviceOpTask.restype = c_int
		dll.ECAT_StopDeviceOpTask.argtypes = [c_ushort]
		dll.ECAT_StopDeviceOpTask.restype = c_int
		dll.ECAT_SetSlaveRxPdoData.argtypes = [c_ushort, c_ushort, c_ushort, c_ushort, POINTER(c_ubyte)]
		dll.ECAT_SetSlaveRxPdoData.restype = c_int
		dll.ECAT_GetSlaveRxPdoData.argtypes = [c_ushort, c_ushort, c_ushort, c_ushort, POINTER(c_ubyte)]
		dll.ECAT_GetSlaveRxPdoData.restype = c_int
		dll.ECAT_GetSlaveTxPdoData.argtypes = [c_ushort, c_ushort, c_ushort, c_ushort, POINTER(c_ubyte)]
		dll.ECAT_GetSlaveTxPdoData.restype = c_int
		dll.ECAT_GetSlaveDI.argtypes = [c_ushort, c_ushort, POINTER(c_uint)]
		dll.ECAT_GetSlaveDI.restype = c_int
		dll.ECAT_GetSlaveDIBit.argtypes = [c_ushort, c_ushort, c_ushort, POINTER(c_uint)]
		dll.ECAT_GetSlaveDIBit.restype = c_int
		dll.ECAT_GetSlaveDO.argtypes = [c_ushort, c_ushort, POINTER(c_uint)]
		dll.ECAT_GetSlaveDO.restype = c_int
		dll.ECAT_GetSlaveDOBit.argtypes = [c_ushort, c_ushort, c_ushort, POINTER(c_uint)]
		dll.ECAT_GetSlaveDOBit.restype = c_int
		dll.ECAT_SetSlaveDO.argtypes = [c_ushort, c_ushort, c_uint]
		dll.ECAT_SetSlaveDO.restype = c_int
		dll.ECAT_SetSlaveDOBit.argtypes = [c_ushort, c_ushort, c_ushort, c_uint]
		dll.ECAT_SetSlaveDOBit.restype = c_int
		dll.ECAT_SetSlaveAoProperty.argtypes = [c_ushort, c_ushort, c_ushort, c_ubyte]
		dll.ECAT_SetSlaveAoProperty.restype = c_int
		dll.ECAT_GetSlaveAoProperty.argtypes = [c_ushort, c_ushort, c_ushort, POINTER(c_ubyte)]
		dll.ECAT_GetSlaveAoProperty.restype = c_int
		dll.ECAT_SetSlaveAoRawData.argtypes = [c_ushort, c_ushort, c_ushort, c_ushort]
		dll.ECAT_SetSlaveAoRawData.restype = c_int
		dll.ECAT_GetSlaveAoRawData.argtypes = [c_ushort, c_ushort, c_ushort, POINTER(c_ushort)]
		dll.ECAT_GetSlaveAoRawData.restype = c_int
		dll.ECAT_SetSlaveAoVoltData.argtypes = [c_ushort, c_ushort, c_ushort, c_double]
		dll.ECAT_SetSlaveAoVoltData.restype = c_int        
		dll.ECAT_GetSlaveAoVoltData.argtypes = [c_ushort, c_ushort, c_ushort, POINTER(c_double)]
		dll.ECAT_GetSlaveAoVoltData.restype = c_int
		dll.ECAT_SetSlaveEncProperty.argtypes = [c_ushort, c_ushort, c_ushort, c_ubyte, c_ubyte, c_ubyte]
		dll.ECAT_SetSlaveEncProperty.restype = c_int
		dll.ECAT_GetSlaveEncProperty.argtypes = [c_ushort, c_ushort, c_ushort, POINTER(c_ubyte), POINTER(c_ubyte), POINTER(c_ubyte)]
		dll.ECAT_GetSlaveEncProperty.restype = c_int
		dll.ECAT_GetSlaveEncCount.argtypes = [c_ushort, c_ushort, c_ushort, POINTER(c_uint)]
		dll.ECAT_GetSlaveEncCount.restype = c_int
		dll.ECAT_ResetSlaveEncCount.argtypes = [c_ushort, c_ushort]
		dll.ECAT_ResetSlaveEncCount.restype = c_int           
		dll.ECAT_SetSlaveEncCount.argtypes = [c_ushort, c_ushort, c_ushort, c_int]
		dll.ECAT_SetSlaveEncCount.restype = c_int         
		dll.ECAT_SetSlaveEncIdxLatchProperty.argtypes = [c_ushort, c_ushort, c_ushort, c_ubyte]
		dll.ECAT_SetSlaveEncIdxLatchProperty.restype = c_int        
		dll.ECAT_GetSlaveEncIdxLatchProperty.argtypes = [c_ushort, c_ushort, c_ushort, POINTER(c_ubyte)]
		dll.ECAT_GetSlaveEncIdxLatchProperty.restype = c_int         
		dll.ECAT_GetSlaveEncIdxLatchCnt.argtypes = [c_ushort, c_ushort, c_ushort, POINTER(c_int)]
		dll.ECAT_GetSlaveEncIdxLatchCnt.restype = c_int        
		dll.ECAT_ResetSlaveEncIdxLatchCnt.argtypes = [c_ushort, c_ushort, c_ushort]
		dll.ECAT_ResetSlaveEncIdxLatchCnt.restype = c_int       
		dll.ECAT_SetSlaveEncExtLatchProperty.argtypes = [c_ushort, c_ushort, c_ushort, c_ubyte, c_ubyte, c_ubyte]
		dll.ECAT_SetSlaveEncExtLatchProperty.restype = c_int        
		dll.ECAT_GetSlaveEncExtLatchProperty.argtypes = [c_ushort, c_ushort, c_ushort, POINTER(c_ubyte), POINTER(c_ubyte), POINTER(c_ubyte)]
		dll.ECAT_GetSlaveEncExtLatchProperty.restype = c_int                
		dll.ECAT_GetSlaveEncExtLatchCnt.argtypes = [c_ushort, c_ushort, c_ushort, POINTER(c_int), POINTER(c_int)]
		dll.ECAT_GetSlaveEncExtLatchCnt.restype = c_int        
		dll.ECAT_ResetSlaveEncExtLatchCnt.argtypes = [c_ushort, c_ushort, c_ushort]
		dll.ECAT_ResetSlaveEncExtLatchCnt.restype = c_int        
		dll.ECAT_SetSlaveCmpTrigProperty.argtypes = [c_ushort, c_ushort, c_ushort, c_ushort]
		dll.ECAT_SetSlaveCmpTrigProperty.restype = c_int        
		dll.ECAT_GetSlaveCmpTrigProperty.argtypes = [c_ushort, c_ushort, c_ushort, POINTER(c_ushort)]
		dll.ECAT_GetSlaveCmpTrigProperty.restype = c_int  
		dll.ECAT_SetSlaveCmpTrigData.argtypes = [c_ushort, c_ushort, c_ushort, c_int]
		dll.ECAT_SetSlaveCmpTrigData.restype = c_int               
		dll.ECAT_SetSlaveContCmpTrigData.argtypes = [c_ushort, c_ushort, c_ushort, c_int, c_uint, c_ubyte]
		dll.ECAT_SetSlaveContCmpTrigData.restype = c_int                           
		dll.ECAT_McInit.argtypes = [c_ushort, POINTER(c_ushort), POINTER(c_ushort), c_ushort]
		dll.ECAT_McInit.restype = c_int       
		dll.ECAT_McSetAxisServoOn.argtypes = [c_ushort, c_ushort, c_ushort]
		dll.ECAT_McSetAxisServoOn.restype = c_int
		dll.ECAT_McSetAxisPPU.argtypes = [c_ushort, c_ushort, c_double]
		dll.ECAT_McSetAxisPPU.restype = c_int
		dll.ECAT_McGetAxisPPU.argtypes = [c_ushort, c_ushort, POINTER(c_double)]
		dll.ECAT_McGetAxisPPU.restype = c_int
		dll.ECAT_McSetAxisEncoderPPR.argtypes = [c_ushort, c_ushort, c_uint]
		dll.ECAT_McSetAxisEncoderPPR.restype = c_int
		dll.ECAT_McGetAxisEncoderPPR.argtypes = [c_ushort, c_ushort, POINTER(c_uint)]
		dll.ECAT_McGetAxisEncoderPPR.restype = c_int
		dll.ECAT_McSetAxisMotorPPR.argtypes = [c_ushort, c_ushort, c_uint]
		dll.ECAT_McSetAxisMotorPPR.restype = c_int
		dll.ECAT_McGetAxisMotorPPR.argtypes = [c_ushort, c_ushort, POINTER(c_uint)]
		dll.ECAT_McGetAxisMotorPPR.restype = c_int
		dll.ECAT_McSetAxisHomeMethod.argtypes = [c_ushort, c_ushort, c_int]
		dll.ECAT_McSetAxisHomeMethod.restype = c_int
		dll.ECAT_McGetAxisHomeMethod.argtypes = [c_ushort, c_ushort, POINTER(c_int)]
		dll.ECAT_McGetAxisHomeMethod.restype = c_int
		dll.ECAT_McSetAxisHomeSpeed.argtypes = [c_ushort, c_ushort, c_double, c_double]
		dll.ECAT_McSetAxisHomeSpeed.restype = c_int
		dll.ECAT_McGetAxisHomeSpeed.argtypes = [c_ushort, c_ushort, POINTER(c_double), POINTER(c_double)]
		dll.ECAT_McGetAxisHomeSpeed.restype = c_int
		dll.ECAT_McSetAxisHomeAcc.argtypes = [c_ushort, c_ushort, c_double]
		dll.ECAT_McSetAxisHomeAcc.restype = c_int
		dll.ECAT_McGetAxisHomeAcc.argtypes = [c_ushort, c_ushort, POINTER(c_double)]
		dll.ECAT_McGetAxisHomeAcc.restype = c_int
		dll.ECAT_McSetAxisHomeOffset.argtypes = [c_ushort, c_ushort, c_double]
		dll.ECAT_McSetAxisHomeOffset.restype = c_int
		dll.ECAT_McGetAxisHomeOffset.argtypes = [c_ushort, c_ushort, POINTER(c_double)]
		dll.ECAT_McGetAxisHomeOffset.restype = c_int
		dll.ECAT_McSetAxisAccTime.argtypes = [c_ushort, c_ushort, c_ushort]
		dll.ECAT_McSetAxisAccTime.restype = c_int
		dll.ECAT_McGetAxisAccTime.argtypes = [c_ushort, c_ushort, POINTER(c_ushort)]
		dll.ECAT_McGetAxisAccTime.restype = c_int
		dll.ECAT_McSetAxisAccDecType.argtypes = [c_ushort, c_ushort, c_ushort]
		dll.ECAT_McSetAxisAccDecType.restype = c_int
		dll.ECAT_McGetAxisAccDecType.argtypes = [c_ushort, c_ushort, POINTER(c_ushort)]
		dll.ECAT_McGetAxisAccDecType.restype = c_int
		dll.ECAT_McSetAxisTouchProbeProperty.argtypes = [c_ushort, c_ushort, c_ushort, c_ubyte, c_ubyte]
		dll.ECAT_McSetAxisTouchProbeProperty.restype = c_int
		dll.ECAT_McGetAxisTouchProbeProperty.argtypes = [c_ushort, c_ushort, c_ushort, POINTER(c_ubyte), POINTER(c_ubyte)]
		dll.ECAT_McGetAxisTouchProbeProperty.restype = c_int
		dll.ECAT_McGetAxisTouchProbeValue.argtypes = [c_ushort, c_ushort, c_ushort, POINTER(c_double)]
		dll.ECAT_McGetAxisTouchProbeValue.restype = c_int
		dll.ECAT_McGetAxisState.argtypes = [c_ushort, c_ushort, POINTER(c_uint)]
		dll.ECAT_McGetAxisState.restype = c_int            
		dll.ECAT_McGetAxisLastError.argtypes = [c_ushort, c_ushort, POINTER(c_int)]
		dll.ECAT_McGetAxisLastError.restype = c_int
		dll.ECAT_McGetAxisDriveError.argtypes = [c_ushort, c_ushort, POINTER(c_ushort)]
		dll.ECAT_McGetAxisDriveError.restype = c_int
		dll.ECAT_McGetAxisCommandPos.argtypes = [c_ushort, c_ushort, POINTER(c_double)]
		dll.ECAT_McGetAxisCommandPos.restype = c_int
		dll.ECAT_McGetAxisActualPos.argtypes = [c_ushort, c_ushort, POINTER(c_double)]
		dll.ECAT_McGetAxisActualPos.restype = c_int
		dll.ECAT_McGetAxisActualVel.argtypes = [c_ushort, c_ushort, POINTER(c_double)]
		dll.ECAT_McGetAxisActualVel.restype = c_int        
		dll.ECAT_McGetAxisDI.argtypes = [c_ushort, c_ushort, POINTER(c_uint)]
		dll.ECAT_McGetAxisDI.restype = c_int
		dll.ECAT_McSetProfileData.argtypes = [c_ushort, c_ushort, POINTER(c_double), c_ushort]
		dll.ECAT_McSetProfileData.restype = c_int        
		dll.ECAT_McGetProfileData.argtypes = [c_ushort, c_ushort, POINTER(c_double), c_ushort]
		dll.ECAT_McGetProfileData.restype = c_int
		dll.ECAT_McSetProfileCSV.argtypes = [c_ushort, c_ushort, c_uint, c_char_p, c_uint, c_ubyte]
		dll.ECAT_McSetProfileCSV.restype = c_int        
		dll.ECAT_McGetProfileCSV.argtypes = [c_ushort, c_ushort, POINTER(c_uint), c_char_p, POINTER(c_uint), POINTER(c_ubyte)]
		dll.ECAT_McGetProfileCSV.restype = c_int        
		dll.ECAT_McSetEcamTable.argtypes = [c_ushort, c_ushort, POINTER(c_double), c_ushort]
		dll.ECAT_McSetEcamTable.restype = c_int
		dll.ECAT_McGetEcamTable.argtypes = [c_ushort, c_ushort, POINTER(c_double), c_ushort]
		dll.ECAT_McGetEcamTable.restype = c_int
		dll.ECAT_McConfigEcamTable.argtypes = [c_ushort, c_ushort, c_ubyte]
		dll.ECAT_McConfigEcamTable.restype = c_int
		dll.ECAT_McAxisHome.argtypes = [c_ushort, c_ushort]
		dll.ECAT_McAxisHome.restype = c_int
		dll.ECAT_McAxisErrorReset.argtypes = [c_ushort, c_ushort]
		dll.ECAT_McAxisErrorReset.restype = c_int
		dll.ECAT_McAxisStop.argtypes = [c_ushort, c_ushort]
		dll.ECAT_McAxisStop.restype = c_int
		dll.ECAT_McAxisQuickStop.argtypes = [c_ushort, c_ushort]
		dll.ECAT_McAxisQuickStop.restype = c_int
		dll.ECAT_McAxisMoveAbs.argtypes = [c_ushort, c_ushort, c_double, c_double]
		dll.ECAT_McAxisMoveAbs.restype = c_int
		dll.ECAT_McAxisMoveRel.argtypes = [c_ushort, c_ushort, c_double, c_double]
		dll.ECAT_McAxisMoveRel.restype = c_int
		dll.ECAT_McAxisChangePos.argtypes = [c_ushort, c_ushort, c_double]
		dll.ECAT_McAxisChangePos.restype = c_int
		dll.ECAT_McAxisChangeVel.argtypes = [c_ushort, c_ushort, c_double]
		dll.ECAT_McAxisChangeVel.restype = c_int        
		dll.ECAT_McAxisMoveVel.argtypes = [c_ushort, c_ushort, c_double]
		dll.ECAT_McAxisMoveVel.restype = c_int
		dll.ECAT_McAxisMoveVelByPos.argtypes = [c_ushort, c_ushort, c_double]
		dll.ECAT_McAxisMoveVelByPos.restype = c_int
		dll.ECAT_McAxisGearIn.argtypes = [c_ushort, c_ushort, c_ushort, c_int, c_uint, c_ushort]
		dll.ECAT_McAxisGearIn.restype = c_int
		dll.ECAT_McAxisGearOut.argtypes = [c_ushort, c_ushort, c_ushort]
		dll.ECAT_McAxisGearOut.restype = c_int
		dll.ECAT_McAxisMoveProfile.argtypes = [c_ushort, c_ushort, c_ushort, c_ushort]
		dll.ECAT_McAxisMoveProfile.restype = c_int        
		dll.ECAT_McAxisMoveProfileCSV.argtypes = [c_ushort, c_ushort, c_ushort]
		dll.ECAT_McAxisMoveProfileCSV.restype = c_int
		dll.ECAT_McAxisCamIn.argtypes = [c_ushort, c_ushort, c_ushort, c_ushort, c_ushort, c_double, c_double]
		dll.ECAT_McAxisCamIn.restype = c_int
		dll.ECAT_McAxisCamOut.argtypes = [c_ushort, c_ushort]
		dll.ECAT_McAxisCamOut.restype = c_int
		dll.ECAT_McAxisCamPhaseShift.argtypes = [c_ushort, c_ushort, c_double]
		dll.ECAT_McAxisCamPhaseShift.restype = c_int        
		dll.ECAT_McAxisTangentInGroup.argtypes = [c_ushort, c_ushort, c_ushort, c_double, c_double]
		dll.ECAT_McAxisTangentInGroup.restype = c_int
		dll.ECAT_McAddAxisToGroup.argtypes = [c_ushort, c_ushort, c_ushort]
		dll.ECAT_McAddAxisToGroup.restype = c_int
		dll.ECAT_McRemoveAxisFromGroup.argtypes = [c_ushort, c_ushort, c_ushort]
		dll.ECAT_McRemoveAxisFromGroup.restype = c_int        
		dll.ECAT_McUngroupAllAxes.argtypes = [c_ushort, c_ushort]
		dll.ECAT_McUngroupAllAxes.restype = c_int
		dll.ECAT_McSetGroupCmdMode.argtypes = [c_ushort, c_ushort, c_ushort]
		dll.ECAT_McSetGroupCmdMode.restype = c_int
		dll.ECAT_McGetGroupCmdMode.argtypes = [c_ushort, c_ushort, POINTER(c_ushort)]
		dll.ECAT_McGetGroupCmdMode.restype = c_int
		dll.ECAT_McGetGroupState.argtypes = [c_ushort, c_ushort, POINTER(c_uint)]
		dll.ECAT_McGetGroupState.restype = c_int        
		dll.ECAT_McGetGroupCmdBuffer.argtypes = [c_ushort, c_ushort, POINTER(c_ushort)]
		dll.ECAT_McGetGroupCmdBuffer.restype = c_int
		dll.ECAT_McSetGroupAccTime.argtypes = [c_ushort, c_ushort, c_ushort]
		dll.ECAT_McSetGroupAccTime.restype = c_int
		dll.ECAT_McGetGroupAccTime.argtypes = [c_ushort, c_ushort, POINTER(c_ushort)]
		dll.ECAT_McGetGroupAccTime.restype = c_int        
		dll.ECAT_McSetGroupAccDecType.argtypes = [c_ushort, c_ushort, c_ushort]
		dll.ECAT_McSetGroupAccDecType.restype = c_int
		dll.ECAT_McGetGroupAccDecType.argtypes = [c_ushort, c_ushort, POINTER(c_ushort)]
		dll.ECAT_McGetGroupAccDecType.restype = c_int
		dll.ECAT_McGroupStop.argtypes = [c_ushort, c_ushort]
		dll.ECAT_McGroupStop.restype = c_int
		dll.ECAT_McGroupQuickStop.argtypes = [c_ushort, c_ushort]
		dll.ECAT_McGroupQuickStop.restype = c_int        
		dll.ECAT_McGroupMoveLineAbs.argtypes = [c_ushort, c_ushort, POINTER(c_double), c_double]
		dll.ECAT_McGroupMoveLineAbs.restype = c_int
		dll.ECAT_McGroupMoveLineAbs_PT.argtypes = [c_ushort, c_ushort, POINTER(c_double), c_double]
		dll.ECAT_McGroupMoveLineAbs_PT.restype = c_int
		dll.ECAT_McGroupMoveLineAbs_PVT.argtypes = [c_ushort, c_ushort, POINTER(c_double), POINTER(c_double), c_double]
		dll.ECAT_McGroupMoveLineAbs_PVT.restype = c_int
		dll.ECAT_McGroupMoveLineAbs_PTexT.argtypes = [c_ushort, c_ushort, POINTER(c_double), c_double, c_double]
		dll.ECAT_McGroupMoveLineAbs_PTexT.restype = c_int
		dll.ECAT_McGroupMoveLineAbs_PPT.argtypes = [c_ushort, c_ushort, POINTER(c_double), c_double]
		dll.ECAT_McGroupMoveLineAbs_PPT.restype = c_int                        
		dll.ECAT_McGroupMoveLineRel.argtypes = [c_ushort, c_ushort, POINTER(c_double), c_double]
		dll.ECAT_McGroupMoveLineRel.restype = c_int
		dll.ECAT_McGroupMoveLineRel_PT.argtypes = [c_ushort, c_ushort, POINTER(c_double), c_double]
		dll.ECAT_McGroupMoveLineRel_PT.restype = c_int
		dll.ECAT_McGroupMoveLineRel_PVT.argtypes = [c_ushort, c_ushort, POINTER(c_double), POINTER(c_double), c_double]
		dll.ECAT_McGroupMoveLineRel_PVT.restype = c_int
		dll.ECAT_McGroupMoveLineRel_PTexT.argtypes = [c_ushort, c_ushort, POINTER(c_double), c_double, c_double]
		dll.ECAT_McGroupMoveLineRel_PTexT.restype = c_int
		dll.ECAT_McGroupMoveLineRel_PPT.argtypes = [c_ushort, c_ushort, POINTER(c_double), c_double]
		dll.ECAT_McGroupMoveLineRel_PPT.restype = c_int     
		dll.ECAT_McGroupMoveShaker.argtypes = [c_ushort, c_ushort, c_double, POINTER(c_double), c_double, c_double]
		dll.ECAT_McGroupMoveShaker.restype = c_int        
		dll.ECAT_McGroupMoveCircularAbs_CP_Angle.argtypes = [c_ushort, c_ushort, c_double, c_double, POINTER(c_double)]
		dll.ECAT_McGroupMoveCircularAbs_CP_Angle.restype = c_int
		dll.ECAT_McGroupMoveCircularAbs_CP_EP.argtypes = [c_ushort, c_ushort, c_double, c_ubyte, POINTER(c_double), POINTER(c_double)]
		dll.ECAT_McGroupMoveCircularAbs_CP_EP.restype = c_int
		dll.ECAT_McGroupMoveCircularAbs_BP_EP.argtypes = [c_ushort, c_ushort, c_double, c_ubyte, POINTER(c_double), POINTER(c_double)]
		dll.ECAT_McGroupMoveCircularAbs_BP_EP.restype = c_int     
		dll.ECAT_McGroupMove3DCircularAbs_CP_Angle.argtypes = [c_ushort, c_ushort, c_double, c_double, POINTER(c_double), POINTER(c_double)]
		dll.ECAT_McGroupMove3DCircularAbs_CP_Angle.restype = c_int
		dll.ECAT_McGroupMove3DCircularAbs_CP_EP.argtypes = [c_ushort, c_ushort, c_double, c_ubyte, POINTER(c_double), POINTER(c_double)]
		dll.ECAT_McGroupMove3DCircularAbs_CP_EP.restype = c_int
		dll.ECAT_McGroupMove3DCircularAbs_BP_EP.argtypes = [c_ushort, c_ushort, c_double, c_ubyte, POINTER(c_double), POINTER(c_double)]
		dll.ECAT_McGroupMove3DCircularAbs_BP_EP.restype = c_int    
		dll.ECAT_McGroupMoveCircularRel_CP_Angle.argtypes = [c_ushort, c_ushort, c_double, c_double, POINTER(c_double)]
		dll.ECAT_McGroupMoveCircularRel_CP_Angle.restype = c_int
		dll.ECAT_McGroupMoveCircularRel_CP_EP.argtypes = [c_ushort, c_ushort, c_double, c_ubyte, POINTER(c_double), POINTER(c_double)]
		dll.ECAT_McGroupMoveCircularRel_CP_EP.restype = c_int       
		dll.ECAT_McGroupMoveCircularRel_BP_EP.argtypes = [c_ushort, c_ushort, c_double, c_ubyte, POINTER(c_double), POINTER(c_double)]
		dll.ECAT_McGroupMoveCircularRel_BP_EP.restype = c_int
		dll.ECAT_McGroupMove3DCircularRel_CP_Angle.argtypes = [c_ushort, c_ushort, c_double, c_double, POINTER(c_double), POINTER(c_double)]
		dll.ECAT_McGroupMove3DCircularRel_CP_Angle.restype = c_int
		dll.ECAT_McGroupMove3DCircularRel_CP_EP.argtypes = [c_ushort, c_ushort, c_double, c_ubyte, POINTER(c_double), POINTER(c_double)]
		dll.ECAT_McGroupMove3DCircularRel_CP_EP.restype = c_int
		dll.ECAT_McGroupMove3DCircularRel_BP_EP.argtypes = [c_ushort, c_ushort, c_double, c_ubyte, POINTER(c_double), POINTER(c_double)]
		dll.ECAT_McGroupMove3DCircularRel_BP_EP.restype = c_int       
		dll.ECAT_McGroupMoveHelicalAbs.argtypes = [c_ushort, c_ushort, c_double, POINTER(c_double), c_double, c_double]
		dll.ECAT_McGroupMoveHelicalAbs.restype = c_int
		dll.ECAT_McGroupMove3DHelicalAbs_CP_Angle.argtypes = [c_ushort, c_ushort, c_double, c_double, c_double, POINTER(c_double), POINTER(c_double)]
		dll.ECAT_McGroupMove3DHelicalAbs_CP_Angle.restype = c_int
		dll.ECAT_McGroupMoveHelicalRel.argtypes = [c_ushort, c_ushort, c_double, POINTER(c_double), c_double, c_double]
		dll.ECAT_McGroupMoveHelicalRel.restype = c_int           
		dll.ECAT_McGroupMove3DHelicalRel_CP_Angle.argtypes = [c_ushort, c_ushort, c_double, c_double, c_double, POINTER(c_double), POINTER(c_double)]
		dll.ECAT_McGroupMove3DHelicalRel_CP_Angle.restype = c_int
		dll.ECAT_McGroupMoveConicalHelixAbs.argtypes = [c_ushort, c_ushort, c_double, POINTER(c_double), c_double, c_double, c_double]
		dll.ECAT_McGroupMoveConicalHelixAbs.restype = c_int
		dll.ECAT_McGroupMove3DConicalHelixAbs_CP_Angle.argtypes = [c_ushort, c_ushort, c_double, c_double, c_double, POINTER(c_double), POINTER(c_double), c_double]
		dll.ECAT_McGroupMove3DConicalHelixAbs_CP_Angle.restype = c_int
		dll.ECAT_McGroupMoveConicalHelixRel.argtypes = [c_ushort, c_ushort, c_double, POINTER(c_double), c_double, c_double, c_double]
		dll.ECAT_McGroupMoveConicalHelixRel.restype = c_int           
		dll.ECAT_McGroupMove3DConicalHelixRel_CP_Angle.argtypes = [c_ushort, c_ushort, c_double, c_double, c_double, POINTER(c_double), POINTER(c_double), c_double]
		dll.ECAT_McGroupMove3DConicalHelixRel_CP_Angle.restype = c_int
		dll.ECAT_McGroupMoveProfile.argtypes = [c_ushort, c_ushort, POINTER(c_ushort), c_ushort]
		dll.ECAT_McGroupMoveProfile.restype = c_int
		dll.ECAT_McGroupMoveProfileCSV.argtypes = [c_ushort, c_ushort, c_ushort]
		dll.ECAT_McGroupMoveProfileCSV.restype = c_int       
		dll.ECAT_McGroupMoveDwell.argtypes = [c_ushort, c_ushort, c_uint]
		dll.ECAT_McGroupMoveDwell.restype = c_int
		dll.ECAT_McGroupMoveDO.argtypes = [c_ushort, c_ushort, c_ushort, c_ushort, POINTER(c_uint)]
		dll.ECAT_McGroupMoveDO.restype = c_int      
		dll.ECAT_McGroupMoveBlendingSync.argtypes = [c_ushort, c_ushort]
		dll.ECAT_McGroupMoveBlendingSync.restype = c_int
		dll.ECAT_SetDeviceEncProperty.argtypes = [c_ushort, c_ushort, c_ubyte, c_ubyte, c_ubyte]
		dll.ECAT_SetDeviceEncProperty.restype = c_int       
		dll.ECAT_GetDeviceEncProperty.argtypes = [c_ushort, c_ushort, POINTER(c_ubyte), POINTER(c_ubyte), POINTER(c_ubyte)]
		dll.ECAT_GetDeviceEncProperty.restype = c_int
		dll.ECAT_GetDeviceEncCount.argtypes = [c_ushort, c_ushort, POINTER(c_int)]
		dll.ECAT_GetDeviceEncCount.restype = c_int
		dll.ECAT_ResetDeviceEncCount.argtypes = [c_ushort, c_ushort]
		dll.ECAT_ResetDeviceEncCount.restype = c_int        
		dll.ECAT_SetDeviceCmpTrigProperty.argtypes = [c_ushort, c_ushort, c_uint, c_ubyte]
		dll.ECAT_SetDeviceCmpTrigProperty.restype = c_int
		dll.ECAT_GetDeviceCmpTrigProperty.argtypes = [c_ushort, c_ushort, POINTER(c_uint), POINTER(c_ubyte)]
		dll.ECAT_GetDeviceCmpTrigProperty.restype = c_int                           
		dll.ECAT_SetDeviceCmpTrigData.argtypes = [c_ushort, c_ushort, c_int]
		dll.ECAT_SetDeviceCmpTrigData.restype = c_int 
		dll.ECAT_SetDeviceContCmpTrigData.argtypes = [c_ushort, c_ushort, c_int, c_uint, c_uint, c_ubyte]
		dll.ECAT_SetDeviceContCmpTrigData.restype = c_int          
		dll.ECAT_SetDeviceEmg.argtypes = [c_ushort, c_ubyte, c_ubyte, c_ubyte, c_ushort, c_ushort, c_ubyte]
		dll.ECAT_SetDeviceEmg.restype = c_int
		dll.ECAT_GetDeviceEmg.argtypes = [c_ushort, POINTER(c_ubyte), POINTER(c_ubyte), POINTER(c_ubyte), POINTER(c_ushort), POINTER(c_ushort), POINTER(c_ubyte)]
		dll.ECAT_GetDeviceEmg.restype = c_int
		dll.ECAT_GetDeviceEmgStatus.argtypes = [c_ushort, POINTER(c_ubyte)]
		dll.ECAT_GetDeviceEmgStatus.restype = c_int        
		dll.ECAT_SetDeviceEmgSoftSig.argtypes = [c_ushort]
		dll.ECAT_SetDeviceEmgSoftSig.restype = c_int
		dll.ECAT_SetDeviceMPG.argtypes = [c_ushort, c_ubyte, POINTER(c_ushort), c_ushort]
		dll.ECAT_SetDeviceMPG.restype = c_int                     
		dll.ECAT_GetDeviceMPG.argtypes = [c_ushort, POINTER(c_ubyte), POINTER(c_ushort), POINTER(c_ushort)]
		dll.ECAT_GetDeviceMPG.restype = c_int
		dll.ECAT_OpenMotionConfig.argtypes = [c_char_p, POINTER(c_ushort), POINTER(c_ushort), POINTER(c_ushort), POINTER(c_double), POINTER(c_int), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_uint), POINTER(c_uint)]
		dll.ECAT_OpenMotionConfig.restype = c_int        
		dll.ECAT_McGetMotionRecordState.argtypes = [c_ushort, POINTER(c_ushort), POINTER(c_uint)]
		dll.ECAT_McGetMotionRecordState.restype = c_int  
		dll.ECAT_McSetMotionRecord.argtypes = [c_ushort, c_ushort]
		dll.ECAT_McSetMotionRecord.restype = c_int         
		dll.ECAT_SetTimer.argtypes = [c_ushort, c_uint]
		dll.ECAT_SetTimer.restype = c_int 
		dll.ECAT_SetTimerStop.argtypes = [c_ushort]
		dll.ECAT_SetTimerStop.restype = c_int          
		dll.ECAT_McSetStewartPlatform_M1.argtypes = [c_ushort, c_double, c_double, c_double, c_double, c_double, c_double]
		dll.ECAT_McSetStewartPlatform_M1.restype = c_int  
		dll.ECAT_McGetAxisActualPosVel.argtypes = [c_ushort, c_ushort, POINTER(c_float), POINTER(c_float)]
		dll.ECAT_McGetAxisActualPosVel.restype = c_int          
		dll.ECAT_PidSetSetPointValue.argtypes = [c_ushort, c_uint, c_double]
		dll.ECAT_PidSetSetPointValue.restype = c_int
		dll.ECAT_PidGetControlOutputValue.argtypes = [c_ushort, c_uint, POINTER(c_double)]
		dll.ECAT_PidGetControlOutputValue.restype = c_int
		dll.ECAT_PidSetParameter.argtypes = [c_ushort, c_uint, c_double, c_double, c_double]
		dll.ECAT_PidSetParameter.restype = c_int        
		dll.ECAT_PidSetStatus.argtypes = [c_ushort, c_uint, c_ubyte]
		dll.ECAT_PidSetStatus.restype = c_int
		dll.ECAT_PidSetSimulateMode.argtypes = [c_ushort, c_uint, c_ubyte]
		dll.ECAT_PidSetSimulateMode.restype = c_int
		dll.ECAT_PidSetSampleTime.argtypes = [c_ushort, c_uint, c_uint]
		dll.ECAT_PidSetSampleTime.restype = c_int
		dll.ECAT_PidSetProcessVariableModule.argtypes = [c_ushort, c_uint, c_ushort, c_ushort, c_ushort, c_double, c_double]
		dll.ECAT_PidSetProcessVariableModule.restype = c_int        
		dll.ECAT_PidSetControlOutputModule.argtypes = [c_ushort, c_uint, c_ushort, c_ushort, c_ushort, c_double, c_double, c_double, c_double]
		dll.ECAT_PidSetControlOutputModule.restype = c_int
		dll.ECAT_PidGetParameter.argtypes = [c_ushort, c_uint, POINTER(c_double), POINTER(c_double), POINTER(c_double)]
		dll.ECAT_PidGetParameter.restype = c_int
		dll.ECAT_PidGetStatus.argtypes = [c_ushort, c_uint, POINTER(c_ubyte)]
		dll.ECAT_PidGetStatus.restype = c_int        
		dll.ECAT_PidGetSimulateMode.argtypes = [c_ushort, c_uint, POINTER(c_ubyte)]
		dll.ECAT_PidGetSimulateMode.restype = c_int
		dll.ECAT_PidGetSampleTime.argtypes = [c_ushort, c_uint, POINTER(c_uint)]
		dll.ECAT_PidGetSampleTime.restype = c_int        
		dll.ECAT_PidGetProcessVariableModule.argtypes = [c_ushort, c_uint, POINTER(c_ushort), POINTER(c_ushort), POINTER(c_ushort), POINTER(c_double), POINTER(c_double)]
		dll.ECAT_PidGetProcessVariableModule.restype = c_int
		dll.ECAT_PidGetControlOutputModule.argtypes = [c_ushort, c_uint, POINTER(c_ushort), POINTER(c_ushort), POINTER(c_ushort), POINTER(c_double), POINTER(c_double), POINTER(c_int), POINTER(c_int)]
		dll.ECAT_PidGetControlOutputModule.restype = c_int        
		dll.ECAT_PidGetProcessVariable.argtypes = [c_ushort, c_uint, POINTER(c_int)]
		dll.ECAT_PidGetProcessVariable.restype = c_int
		dll.ECAT_PidGetSimulateFeedback.argtypes = [c_ushort, c_uint, POINTER(c_double)]
		dll.ECAT_PidGetSimulateFeedback.restype = c_int
		dll.ECAT_WaitforTimer.argtypes = [c_ushort]
		dll.ECAT_WaitforTimer.restype = c_int
		dll.ECAT_PidGetSetPointValue.argtypes = [c_ushort, c_uint, POINTER(c_double)]
		dll.ECAT_PidGetSetPointValue.restype = c_int
		dll.ECAT_PidGet_Sp_Err_Op_Pv.argtypes = [c_ushort, c_uint, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double)]
		dll.ECAT_PidGet_Sp_Err_Op_Pv.restype = c_int
		dll.ECAT_McSetAxisVelocityFeedForwardGain.argtypes = [c_ushort, c_ushort, c_double]
		dll.ECAT_McSetAxisVelocityFeedForwardGain.restype = c_int
		dll.ECAT_McGetAxisVelocityFeedForwardGain.argtypes = [c_ushort, c_ushort, POINTER(c_double)]
		dll.ECAT_McGetAxisVelocityFeedForwardGain.restype = c_int
		dll.ECAT_McStewartPlatformMoveAbs_PT.argtypes = [c_ushort, c_ushort, POINTER(c_double), POINTER(c_double), c_double]
		dll.ECAT_McStewartPlatformMoveAbs_PT.restype = c_int
		dll.ECAT_McGetStewartPlatform_M1.argtypes = [c_ushort, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double)]
		dll.ECAT_McGetStewartPlatform_M1.restype = c_int
		dll.ECAT_McSetStewartPlatform_M2.argtypes = [c_ushort, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double)]
		dll.ECAT_McSetStewartPlatform_M2.restype = c_int
		dll.ECAT_McSetGroupVelLimitStatus.argtypes = [c_ushort, c_ushort, c_ushort]
		dll.ECAT_McSetGroupVelLimitStatus.restype = c_int
		dll.ECAT_McGetGroupVelLimitStatus.argtypes = [c_ushort, c_ushort, POINTER(c_ushort)]
		dll.ECAT_McGetGroupVelLimitStatus.restype = c_int
		dll.ECAT_McSetGroupVelLimitValue.argtypes = [c_ushort, c_ushort, c_double]
		dll.ECAT_McSetGroupVelLimitValue.restype = c_int
		dll.ECAT_McGetGroupVelLimitValue.argtypes = [c_ushort, c_ushort, POINTER(c_double)]
		dll.ECAT_McGetGroupVelLimitValue.restype = c_int
		dll.ECAT_McSetMotionRecordParam.argtypes = [c_ushort, c_ushort, c_ushort]
		dll.ECAT_McSetMotionRecordParam.restype = c_int
		dll.ECAT_McGetMotionRecordParam.argtypes = [c_ushort, POINTER(c_ushort), POINTER(c_ushort)]
		dll.ECAT_McGetMotionRecordParam.restype = c_int
		dll.ECAT_McGetMotionRecordValue.argtypes = [c_ushort, c_uint, c_ushort, POINTER(c_float), POINTER(c_float)]
		dll.ECAT_McGetMotionRecordValue.restype = c_int       
		dll.ECAT_McClearMotionRecord.argtypes = [c_ushort]
		dll.ECAT_McClearMotionRecord.restype = c_int
		dll.ECAT_McSetAxisPosSoftwareLimit.argtypes = [c_ushort, c_ushort, c_double, c_double, POINTER(c_uint)]
		dll.ECAT_McSetAxisPosSoftwareLimit.restype = c_int
		dll.ECAT_McGetAxisPosSoftwareLimit.argtypes = [c_ushort, c_ushort, POINTER(c_double), POINTER(c_double), POINTER(c_uint)]
		dll.ECAT_McGetAxisPosSoftwareLimit.restype = c_int
		dll.ECAT_GetProcessTime.argtypes = [c_ushort, POINTER(c_double)]
		dll.ECAT_GetProcessTime.restype = c_int       
		dll.ECAT_McSetGroupHold.argtypes = [c_ushort, c_ushort, c_ushort]
		dll.ECAT_McSetGroupHold.restype = c_int
		dll.ECAT_McSetAxisPosSoftwareLimitStatus.argtypes = [c_ushort, c_ushort, c_ushort, c_ushort]
		dll.ECAT_McSetAxisPosSoftwareLimitStatus.restype = c_int
		dll.ECAT_McGetAxisPosSoftwareLimitStatus.argtypes = [c_ushort, c_ushort, POINTER(c_ushort), POINTER(c_ushort)]
		dll.ECAT_McGetAxisPosSoftwareLimitStatus.restype = c_int
		dll.ECAT_McGetAxisHomeState.argtypes = [c_ushort, c_ushort, POINTER(c_ushort)]
		dll.ECAT_McGetAxisHomeState.restype = c_int
		dll.ECAT_McAxisGantryIn.argtypes = [c_ushort, c_ushort, c_ushort, c_int]
		dll.ECAT_McAxisGantryIn.restype = c_int        
		dll.ECAT_McSetGroupCmdModeEx.argtypes = [c_ushort, c_ushort, c_ushort]
		dll.ECAT_McSetGroupCmdModeEx.restype = c_int
		dll.ECAT_McAxisGantryMaxPosDiff.argtypes = [c_ushort, c_ushort, c_double]
		dll.ECAT_McAxisGantryMaxPosDiff.restype = c_int
		dll.ECAT_McAxisGantryMaxPosDiffStatus.argtypes = [c_ushort, c_ushort, c_ushort]
		dll.ECAT_McAxisGantryMaxPosDiffStatus.restype = c_int
		dll.ECAT_EvEnableEvent.argtypes = [c_ushort, c_ushort]
		dll.ECAT_EvEnableEvent.restype = c_int
		dll.ECAT_EvDisableEvent.argtypes = [c_ushort, c_ushort]
		dll.ECAT_EvDisableEvent.restype = c_int
		dll.ECAT_EvSetComparePositionParameters.argtypes = [c_ushort, c_ushort, c_ushort, c_ushort, c_double]
		dll.ECAT_EvSetComparePositionParameters.restype = c_int
		dll.ECAT_WaitforEvent.argtypes = [c_ushort, c_uint, POINTER(c_uint)]
		dll.ECAT_WaitforEvent.restype = c_int
		dll.ECAT_EvSetCompareDIBitParameters.argtypes = [c_ushort, c_ushort, c_ushort, c_ushort, c_uint]
		dll.ECAT_EvSetCompareDIBitParameters.restype = c_int
		dll.ECAT_EvSetMotionCompleteParameters.argtypes = [c_ushort, c_ushort, c_ushort]
		dll.ECAT_EvSetMotionCompleteParameters.restype = c_int
		dll.ECAT_McGroupMoveAO.argtypes = [c_ushort, c_ushort, c_ushort, c_uint, c_ushort, c_ushort, c_double]
		dll.ECAT_McGroupMoveAO.restype = c_int
		dll.ECAT_McSetGroupAccTimeEx.argtypes = [c_ushort, c_ushort, c_ushort]
		dll.ECAT_McSetGroupAccTimeEx.restype = c_int
		dll.ECAT_McSetGroupBlendingPercent.argtypes = [c_ushort, c_ushort, c_ushort]
		dll.ECAT_McSetGroupBlendingPercent.restype = c_int        
		dll.ECAT_McSetProfileInterval.argtypes = [c_ushort, c_ushort, c_ushort]
		dll.ECAT_McSetProfileInterval.restype = c_int
		dll.ECAT_McAxisMoveSuperimposed.argtypes = [c_ushort, c_ushort, c_double, c_double]
		dll.ECAT_McAxisMoveSuperimposed.restype = c_int
		dll.ECAT_McAxisHaltSuperimposed.argtypes = [c_ushort, c_ushort]
		dll.ECAT_McAxisHaltSuperimposed.restype = c_int
		dll.ECAT_McGetMotionRecordValueEx.argtypes = [c_ushort, c_uint, c_ushort, c_ushort, POINTER(c_float), POINTER(c_float), POINTER(c_ushort)]
		dll.ECAT_McGetMotionRecordValueEx.restype = c_int        
		dll.ECAT_SetHeartBeat.argtypes = [c_ushort, c_uint]
		dll.ECAT_SetHeartBeat.restype = c_int
		dll.ECAT_SetHeartBeatStatus.argtypes = [c_ushort, c_uint]
		dll.ECAT_SetHeartBeatStatus.restype = c_int
		dll.ECAT_McSetGroupBlendingPercentEx.argtypes = [c_ushort, c_ushort, c_ushort]
		dll.ECAT_McSetGroupBlendingPercentEx.restype = c_int        
		dll.ECAT_EvSetCompareDIParameters.argtypes = [c_ushort, c_ushort, c_ushort, c_ushort, c_uint, c_uint]
		dll.ECAT_EvSetCompareDIParameters.restype = c_int
		dll.ECAT_McSetGroupPause.argtypes = [c_ushort, c_ushort, c_ushort]
		dll.ECAT_McSetGroupPause.restype = c_int
		dll.ECAT_EvSetCompareAxisStateParameters.argtypes = [c_ushort, c_ushort, c_ushort, c_uint]
		dll.ECAT_EvSetCompareAxisStateParameters.restype = c_int
		dll.ECAT_SetSlaveAiProperty.argtypes = [c_ushort, c_ushort, c_ushort, c_ubyte]
		dll.ECAT_SetSlaveAiProperty.restype = c_int        
		dll.ECAT_GetSlaveAiProperty.argtypes = [c_ushort, c_ushort, c_ushort, POINTER(c_ubyte)]
		dll.ECAT_GetSlaveAiProperty.restype = c_int
		dll.ECAT_GetSlaveAiRawData.argtypes = [c_ushort, c_ushort, c_ushort, POINTER(c_ushort)]
		dll.ECAT_GetSlaveAiRawData.restype = c_int       
		dll.ECAT_GetSlaveAiVoltData.argtypes = [c_ushort, c_ushort, c_ushort, POINTER(c_double)]
		dll.ECAT_GetSlaveAiVoltData.restype = c_int
		dll.ECAT_GetSlaveAimAData.argtypes = [c_ushort, c_ushort, c_ushort, POINTER(c_double)]
		dll.ECAT_GetSlaveAimAData.restype = c_int        
		dll.ECAT_McAxisHomeEx.argtypes = [c_ushort, c_ushort, c_ushort]
		dll.ECAT_McAxisHomeEx.restype = c_int                        
		dll.ECAT_EvSetCompareCmdPositionParameters.argtypes = [c_ushort, c_ushort, c_ushort, c_ushort, c_double]
		dll.ECAT_EvSetCompareCmdPositionParameters.restype = c_int
		dll.ECAT_EvSetCompareAxisVelStateParameters.argtypes = [c_ushort, c_ushort, c_ushort, c_uint]
		dll.ECAT_EvSetCompareAxisVelStateParameters.restype = c_int        
		dll.ECAT_McSetAxisMaxVelocity.argtypes = [c_ushort, c_ushort, c_double]
		dll.ECAT_McSetAxisMaxVelocity.restype = c_int
		dll.ECAT_McGetAxisMaxVelocity.argtypes = [c_ushort, c_ushort, POINTER(c_double)]
		dll.ECAT_McGetAxisMaxVelocity.restype = c_int        
		dll.ECAT_McGroupMoveLineAbs_P2P.argtypes = [c_ushort, c_ushort, POINTER(c_double)]
		dll.ECAT_McGroupMoveLineAbs_P2P.restype = c_int
		dll.ECAT_McGroupMoveLineRel_P2P.argtypes = [c_ushort, c_ushort, POINTER(c_double)]
		dll.ECAT_McGroupMoveLineRel_P2P.restype = c_int
		dll.ECAT_McAxisMoveAbs_P2P.argtypes = [c_ushort, c_ushort, c_double]
		dll.ECAT_McAxisMoveAbs_P2P.restype = c_int        
		dll.ECAT_McAxisMoveRel_P2P.argtypes = [c_ushort, c_ushort, c_double]
		dll.ECAT_McAxisMoveRel_P2P.restype = c_int
		dll.ECAT_Set_ECAT2016_AiProperty.argtypes = [c_ushort, c_ushort, c_ushort, c_ubyte, POINTER(c_uint)]
		dll.ECAT_Set_ECAT2016_AiProperty.restype = c_int
		dll.ECAT_Get_ECAT2016_AiProperty.argtypes = [c_ushort, c_ushort, c_ushort, POINTER(c_ubyte), POINTER(c_uint)]
		dll.ECAT_Get_ECAT2016_AiProperty.restype = c_int
		dll.ECAT_Get_ECAT2016_AiRawData.argtypes = [c_ushort, c_ushort, c_ushort, POINTER(c_ushort)]
		dll.ECAT_Get_ECAT2016_AiRawData.restype = c_int        
		dll.ECAT_Get_ECAT2016_AiVoltData.argtypes = [c_ushort, c_ushort, c_ushort, POINTER(c_double)]
		dll.ECAT_Get_ECAT2016_AiVoltData.restype = c_int
		dll.ECAT_EvSetCompareAiParameters.argtypes = [c_ushort, c_ushort, c_ushort, c_ushort, c_ushort, c_ushort, c_double, c_double, c_double]
		dll.ECAT_EvSetCompareAiParameters.restype = c_int
		dll.ECAT_McGroupMovePath.argtypes = [c_ushort, c_ushort, c_ushort, c_ubyte, c_ushort, c_ubyte]
		dll.ECAT_McGroupMovePath.restype = c_int
		dll.ECAT_McAddPathData.argtypes = [c_ushort, c_ushort, c_uint, c_ubyte, POINTER(c_double), POINTER(c_double), POINTER(c_double)]
		dll.ECAT_McAddPathData.restype = c_int
		dll.ECAT_McSetPathData.argtypes = [c_ushort, c_ushort, c_ushort, c_uint, c_ubyte, POINTER(c_double), POINTER(c_double), POINTER(c_double)]
		dll.ECAT_McSetPathData.restype = c_int        
		dll.ECAT_McGetPathData.argtypes = [c_ushort, c_ushort, c_ushort, POINTER(c_uint), POINTER(c_ubyte), POINTER(c_double), POINTER(c_double), POINTER(c_double)]
		dll.ECAT_McGetPathData.restype = c_int
		dll.ECAT_McClearPathData.argtypes = [c_ushort, c_ushort]
		dll.ECAT_McClearPathData.restype = c_int
		dll.ECAT_McGetPathDataSize.argtypes = [c_ushort, c_ushort, POINTER(c_ushort)]
		dll.ECAT_McGetPathDataSize.restype = c_int        
		dll.ECAT_McAxisMoveAbsAdv.argtypes = [c_ushort, c_ushort, c_ushort, c_double, c_double, c_double, c_double, c_double, c_double, c_ubyte]
		dll.ECAT_McAxisMoveAbsAdv.restype = c_int
		dll.ECAT_McAxisMoveRelAdv.argtypes = [c_ushort, c_ushort, c_ushort, c_double, c_double, c_double, c_double, c_double, c_double, c_ubyte]
		dll.ECAT_McAxisMoveRelAdv.restype = c_int
		dll.ECAT_McGroupMoveLineAbsAdv.argtypes = [c_ushort, c_ushort, POINTER(c_double), c_double, c_double, c_double, c_double, c_double, c_ubyte]
		dll.ECAT_McGroupMoveLineAbsAdv.restype = c_int
		dll.ECAT_McGroupMoveLineRelAdv.argtypes = [c_ushort, c_ushort, POINTER(c_double), c_double, c_double, c_double, c_double, c_double, c_ubyte]
		dll.ECAT_McGroupMoveLineRelAdv.restype = c_int        
		dll.ECAT_McAxisMove_CiA402_PP.argtypes = [c_ushort, c_ushort, c_ubyte, c_ubyte, c_double, c_double, c_double, c_double]
		dll.ECAT_McAxisMove_CiA402_PP.restype = c_int
		dll.ECAT_McAxisMove_CiA402_PV.argtypes = [c_ushort, c_ushort, c_double, c_double, c_double]
		dll.ECAT_McAxisMove_CiA402_PV.restype = c_int
		dll.ECAT_McAxisMove_CiA402_PT.argtypes = [c_ushort, c_ushort, c_double, c_double]
		dll.ECAT_McAxisMove_CiA402_PT.restype = c_int        
		dll.ECAT_SetTxPdoBufParam.argtypes = [c_ushort, c_ushort, c_ushort, c_ushort, c_ushort, c_ushort]
		dll.ECAT_SetTxPdoBufParam.restype = c_int
		dll.ECAT_GetTxPdoBufParam.argtypes = [c_ushort, c_ushort, POINTER(c_ushort), POINTER(c_ushort), POINTER(c_ushort), POINTER(c_ushort)]
		dll.ECAT_GetTxPdoBufParam.restype = c_int
		dll.ECAT_SetTxPdoBufEnable.argtypes = [c_ushort, c_ushort, c_ushort]
		dll.ECAT_SetTxPdoBufEnable.restype = c_int
		dll.ECAT_GetTxPdoBufEnable.argtypes = [c_ushort, c_ushort, POINTER(c_ushort)]
		dll.ECAT_GetTxPdoBufEnable.restype = c_int        
		dll.ECAT_GetTxPdoBufValue.argtypes = [c_ushort, c_ushort, POINTER(c_float), c_ushort, POINTER(c_ushort)]
		dll.ECAT_GetTxPdoBufValue.restype = c_int
		dll.ECAT_McSetAxisHomeTorque.argtypes = [c_ushort, c_ushort, c_ushort]
		dll.ECAT_McSetAxisHomeTorque.restype = c_int
		dll.ECAT_McGetAxisHomeTorque.argtypes = [c_ushort, c_ushort, POINTER(c_ushort)]
		dll.ECAT_McGetAxisHomeTorque.restype = c_int        
		dll.ECAT_SetAiFilterParam.argtypes = [c_ushort, c_ushort, c_ushort, c_ushort, c_ushort, c_ushort]
		dll.ECAT_SetAiFilterParam.restype = c_int
		dll.ECAT_GetAiFilterParam.argtypes = [c_ushort, c_ushort, POINTER(c_ushort), POINTER(c_ushort), POINTER(c_ushort), POINTER(c_ushort)]
		dll.ECAT_GetAiFilterParam.restype = c_int
		dll.ECAT_SetAiFilterEnable.argtypes = [c_ushort, c_ushort, c_ushort]
		dll.ECAT_SetAiFilterEnable.restype = c_int
		dll.ECAT_GetAiFilterEnable.argtypes = [c_ushort, c_ushort, POINTER(c_ushort)]
		dll.ECAT_GetAiFilterEnable.restype = c_int        
		dll.ECAT_SetAiFilterFreq.argtypes = [c_ushort, c_ushort, c_ushort, c_double]
		dll.ECAT_SetAiFilterFreq.restype = c_int
		dll.ECAT_GetAiFilterFreq.argtypes = [c_ushort, c_ushort, c_ushort, POINTER(c_double)]
		dll.ECAT_GetAiFilterFreq.restype = c_int
		dll.ECAT_GetAiFilterOutput.argtypes = [c_ushort, c_ushort, POINTER(c_int)]
		dll.ECAT_GetAiFilterOutput.restype = c_int        
		dll.ECAT_SetPdoInToOutParam.argtypes = [c_ushort, c_ushort, c_ushort, c_ushort, c_ushort, c_ushort, c_ushort, c_ushort]
		dll.ECAT_SetPdoInToOutParam.restype = c_int
		dll.ECAT_GetPdoInToOutParam.argtypes = [c_ushort, c_ushort, POINTER(c_ushort), POINTER(c_ushort), POINTER(c_ushort), POINTER(c_ushort), POINTER(c_ushort), POINTER(c_ushort), POINTER(c_ushort)]
		dll.ECAT_GetPdoInToOutParam.restype = c_int
		dll.ECAT_SetPdoInToOutCoeff.argtypes = [c_ushort, c_ushort, c_float, c_float]
		dll.ECAT_SetPdoInToOutCoeff.restype = c_int
		dll.ECAT_GetPdoInToOutCoeff.argtypes = [c_ushort, c_ushort, POINTER(c_float), POINTER(c_float)]
		dll.ECAT_GetPdoInToOutCoeff.restype = c_int        
		dll.ECAT_SetPdoInToOutEnable.argtypes = [c_ushort, c_ushort, c_ushort]
		dll.ECAT_SetPdoInToOutEnable.restype = c_int
		dll.ECAT_GetPdoInToOutEnable.argtypes = [c_ushort, c_ushort, POINTER(c_ushort)]
		dll.ECAT_GetPdoInToOutEnable.restype = c_int
		dll.ECAT_McAxisGantryGain.argtypes = [c_ushort, c_ushort, c_double, c_double]
		dll.ECAT_McAxisGantryGain.restype = c_int
		dll.ECAT_SetSlaveNoType.argtypes = [c_ushort, c_ushort]
		dll.ECAT_SetSlaveNoType.restype = c_int
		dll.ECAT_GetSlaveNoType.argtypes = [c_ushort, POINTER(c_ushort)]
		dll.ECAT_GetSlaveNoType.restype = c_int
		dll.ECAT_EvSetMotionCompleteParameters_Grp.argtypes = [c_ushort, c_ushort, c_ushort]
		dll.ECAT_EvSetMotionCompleteParameters_Grp.restype = c_int
		dll.ECAT_McSetGroupPvtDecEnable.argtypes = [c_ushort, c_ushort, c_ushort]
		dll.ECAT_McSetGroupPvtDecEnable.restype = c_int
		dll.ECAT_McGetGroupPvtDecEnable.argtypes = [c_ushort, c_ushort, POINTER(c_ushort)]
		dll.ECAT_McGetGroupPvtDecEnable.restype = c_int        
		dll.ECAT_AbortWaitforEvent.argtypes = [c_ushort]
		dll.ECAT_AbortWaitforEvent.restype = c_int        
		dll.ECAT_McSetGroupCoordinate.argtypes = [c_ushort, c_ushort, c_ushort]
		dll.ECAT_McSetGroupCoordinate.restype = c_int        
		dll.ECAT_McSetGroupCoordinateLimit.argtypes = [c_ushort, c_ushort, c_ushort, c_ushort, c_double, c_double]
		dll.ECAT_McSetGroupCoordinateLimit.restype = c_int        
		dll.ECAT_McGetGroupCoordinateLimit.argtypes = [c_ushort, c_ushort, c_ushort, POINTER(c_ushort), POINTER(c_double), POINTER(c_double)]
		dll.ECAT_McGetGroupCoordinateLimit.restype = c_int                  
		dll.ECAT_SetSlaveArrCmpPos.argtypes = [c_ushort, c_ushort, c_ushort, c_ubyte, c_int]
		dll.ECAT_SetSlaveArrCmpPos.restype = c_int        
		dll.ECAT_GetSlaveArrCmpPos.argtypes = [c_ushort, c_ushort, c_ushort, c_ubyte, POINTER(c_int)]
		dll.ECAT_GetSlaveArrCmpPos.restype = c_int        
		dll.ECAT_SetSlaveArrCmpEnable.argtypes = [c_ushort, c_ushort, c_ushort, c_ubyte, c_ubyte]
		dll.ECAT_SetSlaveArrCmpEnable.restype = c_int                  
		dll.ECAT_GetSlaveArrCmpEnable.argtypes = [c_ushort, c_ushort, c_ushort, c_ubyte, POINTER(c_ubyte)]
		dll.ECAT_GetSlaveArrCmpEnable.restype = c_int        
		dll.ECAT_SetSlaveArrCmpEndIdx.argtypes = [c_ushort, c_ushort, c_ushort, c_ushort]
		dll.ECAT_SetSlaveArrCmpEndIdx.restype = c_int        
		dll.ECAT_GetSlaveArrCmpEndIdx.argtypes = [c_ushort, c_ushort, c_ushort, POINTER(c_ushort)]
		dll.ECAT_GetSlaveArrCmpEndIdx.restype = c_int  
		dll.ECAT_SetSlaveArrCmpTrig.argtypes = [c_ushort, c_ushort, c_ushort]
		dll.ECAT_SetSlaveArrCmpTrig.restype = c_int       
		dll.ECAT_SetSlaveSaveArrCmpData.argtypes = [c_ushort, c_ushort]
		dll.ECAT_SetSlaveSaveArrCmpData.restype = c_int                     
		dll.ECAT_McSetAxisDIActiveLevel.argtypes = [c_ushort, c_ushort, c_ubyte, c_ubyte, c_ubyte]
		dll.ECAT_McSetAxisDIActiveLevel.restype = c_int        
		dll.ECAT_McGetAxisDIActiveLevel.argtypes = [c_ushort, c_ushort, POINTER(c_ubyte), POINTER(c_ubyte), POINTER(c_ubyte)]
		dll.ECAT_McGetAxisDIActiveLevel.restype = c_int  
		dll.ECAT_McGetAxisDI_60FD.argtypes = [c_ushort, c_ushort, POINTER(c_uint)]
		dll.ECAT_McGetAxisDI_60FD.restype = c_int        
		dll.ECAT_SaveSlaveEEPROM.argtypes = [c_ushort, c_ushort]
		dll.ECAT_SaveSlaveEEPROM.restype = c_int          
		dll.ECAT_McSetAxisActualPosition.argtypes = [c_ushort, c_ushort, c_double]
		dll.ECAT_McSetAxisActualPosition.restype = c_int          
		dll.ECAT_McSetAxisCommandPosition.argtypes = [c_ushort, c_ushort, c_double]
		dll.ECAT_McSetAxisCommandPosition.restype = c_int          
		dll.ECAT_McSetAxisAccDecTime_Stepper.argtypes = [c_ushort, c_ushort, c_ushort, c_ushort]
		dll.ECAT_McSetAxisAccDecTime_Stepper.restype = c_int          
		dll.ECAT_McGetAxisAccDecTime_Stepper.argtypes = [c_ushort, c_ushort, POINTER(c_ushort), POINTER(c_ushort)]
		dll.ECAT_McGetAxisAccDecTime_Stepper.restype = c_int          
		dll.ECAT_GetDeviceStateEx.argtypes = [c_ushort, POINTER(c_uint), POINTER(c_uint), POINTER(c_uint), POINTER(c_uint)]
		dll.ECAT_GetDeviceStateEx.restype = c_int          
		dll.ECAT_SetSlaveEncIdxLatchBufferEnable.argtypes = [c_ushort, c_ushort, c_ushort, c_ubyte]
		dll.ECAT_SetSlaveEncIdxLatchBufferEnable.restype = c_int          
		dll.ECAT_GetSlaveEncIdxLatchBufferEnable.argtypes = [c_ushort, c_ushort, c_ushort, POINTER(c_ubyte)]
		dll.ECAT_GetSlaveEncIdxLatchBufferEnable.restype = c_int          
		dll.ECAT_ResetSlaveEncIdxLatchBuffer.argtypes = [c_ushort, c_ushort, c_ushort]
		dll.ECAT_ResetSlaveEncIdxLatchBuffer.restype = c_int          
		dll.ECAT_GetSlaveEncIdxLatchBuffer.argtypes = [c_ushort, c_ushort, c_ushort, c_uint, POINTER(c_uint), POINTER(c_uint)]
		dll.ECAT_GetSlaveEncIdxLatchBuffer.restype = c_int          
		dll.ECAT_McSetAxisAccUnit_Stepper.argtypes = [c_ushort, c_ushort, c_ushort, POINTER(c_uint)]
		dll.ECAT_McSetAxisAccUnit_Stepper.restype = c_int          
		dll.ECAT_McGetAxisAccUnit_Stepper.argtypes = [c_ushort, c_ushort, POINTER(c_ushort), POINTER(c_uint)]
		dll.ECAT_McGetAxisAccUnit_Stepper.restype = c_int          
		dll.ECAT_McSetAxisVelAccScale.argtypes = [c_ushort, c_ushort, c_double, c_double]
		dll.ECAT_McSetAxisVelAccScale.restype = c_int          
		dll.ECAT_McGetAxisVelAccScale.argtypes = [c_ushort, c_ushort, POINTER(c_double), POINTER(c_double)]
		dll.ECAT_McGetAxisVelAccScale.restype = c_int          
		dll.ECAT_McSetAxisInpSignal.argtypes = [c_ushort, c_ushort, c_ushort, c_ushort, c_int]
		dll.ECAT_McSetAxisInpSignal.restype = c_int          
		dll.ECAT_McGetAxisInpSignal.argtypes = [c_ushort, c_ushort, POINTER(c_ushort), POINTER(c_ushort), POINTER(c_int)]
		dll.ECAT_McGetAxisInpSignal.restype = c_int          
		dll.ECAT_McSetAxisInpCompare.argtypes = [c_ushort, c_ushort, c_uint, c_double, c_int]
		dll.ECAT_McSetAxisInpCompare.restype = c_int          
		dll.ECAT_McGetAxisInpCompare.argtypes = [c_ushort, c_ushort, POINTER(c_uint), POINTER(c_double), POINTER(c_int)]
		dll.ECAT_McGetAxisInpCompare.restype = c_int          
		dll.ECAT_McSetAxisInpTimeOut.argtypes = [c_ushort, c_ushort, c_uint]
		dll.ECAT_McSetAxisInpTimeOut.restype = c_int          
		dll.ECAT_McGetAxisInpTimeOut.argtypes = [c_ushort, c_ushort, POINTER(c_uint)]
		dll.ECAT_McGetAxisInpTimeOut.restype = c_int          
		dll.ECAT_McSetAxisWanErrEnable.argtypes = [c_ushort, c_ushort, c_uint]
		dll.ECAT_McSetAxisWanErrEnable.restype = c_int      
		dll.ECAT_McGetAxisWanErrEnable.argtypes = [c_ushort, c_ushort, POINTER(c_uint)]
		dll.ECAT_McGetAxisWanErrEnable.restype = c_int    
		dll.ECAT_SetMultiSlaveDO.argtypes = [c_ushort, POINTER(c_ushort), POINTER(c_uint), c_ushort]
		dll.ECAT_SetMultiSlaveDO.restype = c_int      
		dll.ECAT_GetMultiSlaveDO.argtypes = [c_ushort, POINTER(c_ushort), POINTER(c_uint), c_ushort]
		dll.ECAT_GetMultiSlaveDO.restype = c_int    
		dll.ECAT_McSetAxisDefaultMode.argtypes = [c_ushort, c_ushort, c_byte]
		dll.ECAT_McSetAxisDefaultMode.restype = c_int      
		dll.ECAT_McGetAxisDefaultMode.argtypes = [c_ushort, c_ushort, POINTER(c_byte)]
		dll.ECAT_McGetAxisDefaultMode.restype = c_int    
		dll.ECAT_McInit_Ex.argtypes = [c_ushort, c_char_p, POINTER(c_ushort), POINTER(c_ushort), POINTER(c_ushort)]
		dll.ECAT_McInit_Ex.restype = c_int    
		dll.ECAT_SetMultiSlaveDO_AutoOff.argtypes = [c_ushort, POINTER(c_ushort), POINTER(c_uint), POINTER(c_uint), POINTER(c_uint), c_ushort]
		dll.ECAT_SetMultiSlaveDO_AutoOff.restype = c_int    
		dll.ECAT_McSetAxisAccDecUnit.argtypes = [c_ushort, c_ushort, c_ushort]
		dll.ECAT_McSetAxisAccDecUnit.restype = c_int      
		dll.ECAT_McGetAxisAccDecUnit.argtypes = [c_ushort, c_ushort, POINTER(c_ushort)]
		dll.ECAT_McGetAxisAccDecUnit.restype = c_int    
		dll.ECAT_McSetAxisAccDecRate.argtypes = [c_ushort, c_ushort, c_double]
		dll.ECAT_McSetAxisAccDecRate.restype = c_int      
		dll.ECAT_McGetAxisAccDecRate.argtypes = [c_ushort, c_ushort, POINTER(c_double)]
		dll.ECAT_McGetAxisAccDecRate.restype = c_int    
		dll.ECAT_McGetAxisState_Ex.argtypes = [c_ushort, c_ushort, POINTER(c_uint)]
		dll.ECAT_McGetAxisState_Ex.restype = c_int    
		dll.ECAT_McGetAxisLastError_Ex.argtypes = [c_ushort, c_ushort, POINTER(c_int)]
		dll.ECAT_McGetAxisLastError_Ex.restype = c_int    
		dll.ECAT_McGetAxisDriveError_Ex.argtypes = [c_ushort, c_ushort, POINTER(c_ushort)]
		dll.ECAT_McGetAxisDriveError_Ex.restype = c_int    
		dll.ECAT_McGetAxisActualPos_Ex.argtypes = [c_ushort, c_ushort, POINTER(c_double)]
		dll.ECAT_McGetAxisActualPos_Ex.restype = c_int    
		dll.ECAT_McGetAxisCommandPos_Ex.argtypes = [c_ushort, c_ushort, POINTER(c_double)]
		dll.ECAT_McGetAxisCommandPos_Ex.restype = c_int    
		dll.ECAT_McGetAxisActualVel_Ex.argtypes = [c_ushort, c_ushort, POINTER(c_double)]
		dll.ECAT_McGetAxisActualVel_Ex.restype = c_int    
		dll.ECAT_McGetAxisDI_Ex.argtypes = [c_ushort, c_ushort, POINTER(c_uint)]
		dll.ECAT_McGetAxisDI_Ex.restype = c_int    
		dll.ECAT_McGetAxisActualPos_Directly.argtypes = [c_ushort, c_ushort, POINTER(c_double)]
		dll.ECAT_McGetAxisActualPos_Directly.restype = c_int    
		dll.ECAT_McGetAxisCommandPos_Directly.argtypes = [c_ushort, c_ushort, POINTER(c_double)]
		dll.ECAT_McGetAxisCommandPos_Directly.restype = c_int    
		dll.ECAT_McGetAxisState_Directly.argtypes = [c_ushort, c_ushort, POINTER(c_uint)]
		dll.ECAT_McGetAxisState_Directly.restype = c_int    
		dll.ECAT_McGetAxisDI_Directly.argtypes = [c_ushort, c_ushort, POINTER(c_uint)]
		dll.ECAT_McGetAxisDI_Directly.restype = c_int    
		dll.ECAT_McEnable_Directly.argtypes = [c_ushort]
		dll.ECAT_McEnable_Directly.restype = c_int    
		dll.ECAT_SetSlaveDIMap.argtypes = [c_ushort, c_ushort, POINTER(c_ushort), POINTER(c_ushort)]
		dll.ECAT_SetSlaveDIMap.restype = c_int      
		dll.ECAT_GetSlaveDI_Directly.argtypes = [c_ushort, c_ushort, POINTER(c_uint)]
		dll.ECAT_GetSlaveDI_Directly.restype = c_int    
		dll.ECAT_GetSlaveDIBit_Directly.argtypes = [c_ushort, c_ushort, c_ushort, POINTER(c_uint)]
		dll.ECAT_GetSlaveDIBit_Directly.restype = c_int    
		dll.ECAT_GetSlaveDO_Directly.argtypes = [c_ushort, c_ushort, c_ushort, POINTER(c_uint)]
		dll.ECAT_GetSlaveDO_Directly.restype = c_int    
		dll.ECAT_GetSlaveDOBit_Directly.argtypes = [c_ushort, c_ushort, c_ushort, c_ushort, POINTER(c_uint)]
		dll.ECAT_GetSlaveDOBit_Directly.restype = c_int    
		dll.ECAT_McAxisGantryOut.argtypes = [c_ushort, c_ushort]
		dll.ECAT_McAxisGantryOut.restype = c_int      
		dll.ECAT_SetDeviceCmpDisable.argtypes = [c_ushort, c_ushort]
		dll.ECAT_SetDeviceCmpDisable.restype = c_int
		dll.ECAT_McAxisGantryGainEx.argtypes = [c_ushort, c_ushort, c_double, c_double, c_double]
		dll.ECAT_McAxisGantryGainEx.restype = c_int
		dll.ECAT_McAxisTangentOut.argtypes = [c_ushort, c_ushort, c_ushort]
		dll.ECAT_McAxisTangentOut.restype = c_int
		dll.ECAT_McAxisMoveVelEx.argtypes = [c_ushort, c_ushort, c_double]
		dll.ECAT_McAxisMoveVelEx.restype = c_int
		dll.ECAT_McAxisMoveTorEx.argtypes = [c_ushort, c_ushort, c_double]
		dll.ECAT_McAxisMoveTorEx.restype = c_int
		dll.ECAT_McAxisGearInByPos.argtypes = [c_ushort, c_ushort, c_ushort, c_int, c_uint, c_ushort, c_double, c_double, c_double, c_double, c_double, c_ushort]
		dll.ECAT_McAxisGearInByPos.restype = c_int
		dll.ECAT_GetSlaveTxPdoData_Ex.argtypes = [c_ushort, c_ushort, c_ushort, c_ushort, POINTER(c_ubyte)]
		dll.ECAT_GetSlaveTxPdoData_Ex.restype = c_int
		dll.ECAT_GetSlaveRxPdoData_Ex.argtypes = [c_ushort, c_ushort, c_ushort, c_ushort, POINTER(c_ubyte)]
		dll.ECAT_GetSlaveRxPdoData_Ex.restype = c_int
		dll.ECAT_Get_2074A.argtypes = [c_ushort, c_ushort, c_ushort, POINTER(c_byte), POINTER(c_byte), POINTER(c_uint), POINTER(c_uint)]
		dll.ECAT_Get_2074A.restype = c_int
		dll.ECAT_McSetAxisActualPositionBy35.argtypes = [c_ushort, c_ushort, c_double]
		dll.ECAT_McSetAxisActualPositionBy35.restype = c_int          
		dll.ECAT_McGetGroupPauseState.argtypes = [c_ushort, c_ushort, POINTER(c_ushort)]
		dll.ECAT_McGetGroupPauseState.restype = c_int        
		dll.ECAT_McGroupMoveLineAbs_Ex.argtypes = [c_ushort, c_ushort, POINTER(c_double), c_double]
		dll.ECAT_McGroupMoveLineAbs_Ex.restype = c_int
		dll.ECAT_McGroupMoveLineRel_Ex.argtypes = [c_ushort, c_ushort, POINTER(c_double), c_double]
		dll.ECAT_McGroupMoveLineRel_Ex.restype = c_int
		dll.ECAT_McGroupMoveLineAbsAdv_Ex.argtypes = [c_ushort, c_ushort, POINTER(c_double), c_double, c_double, c_double, c_double, c_double, c_ubyte]
		dll.ECAT_McGroupMoveLineAbsAdv_Ex.restype = c_int
		dll.ECAT_McGroupMoveLineRelAdv_Ex.argtypes = [c_ushort, c_ushort, POINTER(c_double), c_double, c_double, c_double, c_double, c_double, c_ubyte]
		dll.ECAT_McGroupMoveLineRelAdv_Ex.restype = c_int        
		dll.ECAT_McSetAxisHomeStable.argtypes = [c_ushort, c_ushort, c_double, c_double, c_double]
		dll.ECAT_McSetAxisHomeStable.restype = c_int
		dll.ECAT_McGetAxisHomeStable.argtypes = [c_ushort, c_ushort, POINTER(c_double), POINTER(c_double), POINTER(c_double)]
		dll.ECAT_McGetAxisHomeStable.restype = c_int
		dll.ECAT_McAxisMoveAbsAdv_Ex.argtypes = [c_ushort, c_ushort, c_ushort, c_double, c_double, c_double, c_double, c_double, c_double, c_ubyte]
		dll.ECAT_McAxisMoveAbsAdv_Ex.restype = c_int
		dll.ECAT_McAxisMoveRelAdv_Ex.argtypes = [c_ushort, c_ushort, c_ushort, c_double, c_double, c_double, c_double, c_double, c_double, c_ubyte]
		dll.ECAT_McAxisMoveRelAdv_Ex.restype = c_int
		dll.ECAT_SetSlaveDIMap_16bit.argtypes = [c_ushort, c_ushort, POINTER(c_ushort), POINTER(c_ushort), POINTER(c_ushort)]
		dll.ECAT_SetSlaveDIMap_16bit.restype = c_int      
		dll.ECAT_GetSlaveDI_Directly_16bit.argtypes = [c_ushort, POINTER(c_ushort)]
		dll.ECAT_GetSlaveDI_Directly_16bit.restype = c_int    
		dll.ECAT_do_cfg_save.argtypes = [c_ushort, POINTER(c_ushort), POINTER(c_ushort), POINTER(c_ushort), c_ushort]
		dll.ECAT_do_cfg_save.restype = c_int      
		dll.ECAT_do_cfg_load.argtypes = [c_ushort]
		dll.ECAT_do_cfg_load.restype = c_int    
		dll.ECAT_ClearDoQueue.argtypes = [c_ushort]
		dll.ECAT_ClearDoQueue.restype = c_int    
		dll.ECAT_PreSetSlaveDO.argtypes = [c_ushort, c_ushort, c_ushort, c_ushort]
		dll.ECAT_PreSetSlaveDO.restype = c_int      
		dll.ECAT_SetSlaveRxPdoData_Ex.argtypes = [c_ushort, c_ushort, c_ushort, c_ushort, POINTER(c_ubyte)]
		dll.ECAT_SetSlaveRxPdoData_Ex.restype = c_int
		dll.ECAT_McAddAxisToGroup_Ex.argtypes = [c_ushort, c_ushort, c_ushort]
		dll.ECAT_McAddAxisToGroup_Ex.restype = c_int
		dll.ECAT_McRemoveAxisFromGroup_Ex.argtypes = [c_ushort, c_ushort, c_ushort]
		dll.ECAT_McRemoveAxisFromGroup_Ex.restype = c_int        
		dll.ECAT_McUngroupAllAxes_Ex.argtypes = [c_ushort, c_ushort]
		dll.ECAT_McUngroupAllAxes_Ex.restype = c_int
		dll.ECAT_McSetGroupCmdMode_Ex.argtypes = [c_ushort, c_ushort, c_ushort]
		dll.ECAT_McSetGroupCmdMode_Ex.restype = c_int
		dll.ECAT_McSetGroupAccDecType_Ex.argtypes = [c_ushort, c_ushort, c_ushort]
		dll.ECAT_McSetGroupAccDecType_Ex.restype = c_int
		dll.ECAT_McSetGroupAccTime_Ex.argtypes = [c_ushort, c_ushort, c_ushort]
		dll.ECAT_McSetGroupAccTime_Ex.restype = c_int
		dll.ECAT_McGroupMoveProfileCSV_Ex.argtypes = [c_ushort, c_ushort, c_ushort]
		dll.ECAT_McGroupMoveProfileCSV_Ex.restype = c_int       
		dll.ECAT_McGroupMoveDwell_Ex.argtypes = [c_ushort, c_ushort, c_uint]
		dll.ECAT_McGroupMoveDwell_Ex.restype = c_int
		dll.ECAT_McGroupMoveBlendingSync_Ex.argtypes = [c_ushort, c_ushort]
		dll.ECAT_McGroupMoveBlendingSync_Ex.restype = c_int
		dll.ECAT_McGetAxisActualPosVel_Ex.argtypes = [c_ushort, c_ushort, POINTER(c_float), POINTER(c_float)]
		dll.ECAT_McGetAxisActualPosVel_Ex.restype = c_int          
		dll.ECAT_McSetGroupVelLimitStatus_Ex.argtypes = [c_ushort, c_ushort, c_ushort]
		dll.ECAT_McSetGroupVelLimitStatus_Ex.restype = c_int
		dll.ECAT_McSetGroupVelLimitValue_Ex.argtypes = [c_ushort, c_ushort, c_double]
		dll.ECAT_McSetGroupVelLimitValue_Ex.restype = c_int
		dll.ECAT_McGetMotionRecordValue_Ex.argtypes = [c_ushort, c_uint, c_ushort, POINTER(c_float), POINTER(c_float)]
		dll.ECAT_McGetMotionRecordValue_Ex.restype = c_int       
		dll.ECAT_McSetGroupHold_Ex.argtypes = [c_ushort, c_ushort, c_ushort]
		dll.ECAT_McSetGroupHold_Ex.restype = c_int
		dll.ECAT_McSetGroupCmdModeEx_Ex.argtypes = [c_ushort, c_ushort, c_ushort]
		dll.ECAT_McSetGroupCmdModeEx_Ex.restype = c_int
		dll.ECAT_McSetGroupBlendingPercent_Ex.argtypes = [c_ushort, c_ushort, c_ushort]
		dll.ECAT_McSetGroupBlendingPercent_Ex.restype = c_int        
		dll.ECAT_McGetMotionRecordValueEx_Ex.argtypes = [c_ushort, c_uint, c_ushort, c_ushort, POINTER(c_float), POINTER(c_float), POINTER(c_ushort)]
		dll.ECAT_McGetMotionRecordValueEx_Ex.restype = c_int        
		dll.ECAT_McSetGroupBlendingPercentEx_Ex.argtypes = [c_ushort, c_ushort, c_ushort]
		dll.ECAT_McSetGroupBlendingPercentEx_Ex.restype = c_int        
		dll.ECAT_McSetGroupPause_Ex.argtypes = [c_ushort, c_ushort, c_ushort]
		dll.ECAT_McSetGroupPause_Ex.restype = c_int
		dll.ECAT_SetCheckSlaveCnt.argtypes = [c_ushort, c_int]
		dll.ECAT_SetCheckSlaveCnt.restype = c_int
		dll.ECAT_McGroupMoveShaker_Ex.argtypes = [c_ushort, c_ushort, c_double, POINTER(c_double), c_double, c_double]
		dll.ECAT_McGroupMoveShaker_Ex.restype = c_int        
		dll.ECAT_McSetGroupPvtDecEnable_Ex.argtypes = [c_ushort, c_ushort, c_ushort]
		dll.ECAT_McSetGroupPvtDecEnable_Ex.restype = c_int
		dll.ECAT_McSetGroupCoordinate_Ex.argtypes = [c_ushort, c_ushort, c_ushort]
		dll.ECAT_McSetGroupCoordinate_Ex.restype = c_int        
		dll.ECAT_McSetGroupCoordinateLimit_Ex.argtypes = [c_ushort, c_ushort, c_ushort, c_ushort, c_double, c_double]
		dll.ECAT_McSetGroupCoordinateLimit_Ex.restype = c_int        
		dll.ECAT_McGetGroupCoordinateLimit_Ex.argtypes = [c_ushort, c_ushort, c_ushort, POINTER(c_ushort), POINTER(c_double), POINTER(c_double)]
		dll.ECAT_McGetGroupCoordinateLimit_Ex.restype = c_int                  
		dll.ECAT_SetMultiSlaveDO_Ex.argtypes = [c_ushort, POINTER(c_ushort), POINTER(c_uint), c_ushort]
		dll.ECAT_SetMultiSlaveDO_Ex.restype = c_int      
		dll.ECAT_GetMultiSlaveDO_Ex.argtypes = [c_ushort, POINTER(c_ushort), POINTER(c_uint), c_ushort]
		dll.ECAT_GetMultiSlaveDO_Ex.restype = c_int    
		dll.ECAT_SetMultiSlaveDO_AutoOff_Ex.argtypes = [c_ushort, POINTER(c_ushort), POINTER(c_uint), POINTER(c_uint), POINTER(c_uint), c_ushort]
		dll.ECAT_SetMultiSlaveDO_AutoOff_Ex.restype = c_int    
		dll.ECAT_McGroupMoveCircularAbs_CP_Angle_Ex.argtypes = [c_ushort, c_ushort, c_double, c_double, POINTER(c_double)]
		dll.ECAT_McGroupMoveCircularAbs_CP_Angle_Ex.restype = c_int
		dll.ECAT_McGroupMoveCircularAbs_CP_EP_Ex.argtypes = [c_ushort, c_ushort, c_double, c_ubyte, POINTER(c_double), POINTER(c_double)]
		dll.ECAT_McGroupMoveCircularAbs_CP_EP_Ex.restype = c_int
		dll.ECAT_McGroupMoveCircularAbs_BP_EP_Ex.argtypes = [c_ushort, c_ushort, c_double, c_ubyte, POINTER(c_double), POINTER(c_double)]
		dll.ECAT_McGroupMoveCircularAbs_BP_EP_Ex.restype = c_int     
		dll.ECAT_McGroupMove3DCircularAbs_CP_Angle_Ex.argtypes = [c_ushort, c_ushort, c_double, c_double, POINTER(c_double), POINTER(c_double)]
		dll.ECAT_McGroupMove3DCircularAbs_CP_Angle_Ex.restype = c_int
		dll.ECAT_McGroupMove3DCircularAbs_CP_EP_Ex.argtypes = [c_ushort, c_ushort, c_double, c_ubyte, POINTER(c_double), POINTER(c_double)]
		dll.ECAT_McGroupMove3DCircularAbs_CP_EP_Ex.restype = c_int
		dll.ECAT_McGroupMove3DCircularAbs_BP_EP_Ex.argtypes = [c_ushort, c_ushort, c_double, c_ubyte, POINTER(c_double), POINTER(c_double)]
		dll.ECAT_McGroupMove3DCircularAbs_BP_EP_Ex.restype = c_int    
		dll.ECAT_McGroupMoveCircularRel_CP_Angle_Ex.argtypes = [c_ushort, c_ushort, c_double, c_double, POINTER(c_double)]
		dll.ECAT_McGroupMoveCircularRel_CP_Angle_Ex.restype = c_int
		dll.ECAT_McGroupMoveCircularRel_CP_EP_Ex.argtypes = [c_ushort, c_ushort, c_double, c_ubyte, POINTER(c_double), POINTER(c_double)]
		dll.ECAT_McGroupMoveCircularRel_CP_EP_Ex.restype = c_int       
		dll.ECAT_McGroupMoveCircularRel_BP_EP_Ex.argtypes = [c_ushort, c_ushort, c_double, c_ubyte, POINTER(c_double), POINTER(c_double)]
		dll.ECAT_McGroupMoveCircularRel_BP_EP_Ex.restype = c_int
		dll.ECAT_McGroupMove3DCircularRel_CP_Angle_Ex.argtypes = [c_ushort, c_ushort, c_double, c_double, POINTER(c_double), POINTER(c_double)]
		dll.ECAT_McGroupMove3DCircularRel_CP_Angle_Ex.restype = c_int
		dll.ECAT_McGroupMove3DCircularRel_CP_EP_Ex.argtypes = [c_ushort, c_ushort, c_double, c_ubyte, POINTER(c_double), POINTER(c_double)]
		dll.ECAT_McGroupMove3DCircularRel_CP_EP_Ex.restype = c_int
		dll.ECAT_McGroupMove3DCircularRel_BP_EP_Ex.argtypes = [c_ushort, c_ushort, c_double, c_ubyte, POINTER(c_double), POINTER(c_double)]
		dll.ECAT_McGroupMove3DCircularRel_BP_EP_Ex.restype = c_int       
		dll.ECAT_McGroupMoveHelicalAbs_Ex.argtypes = [c_ushort, c_ushort, c_double, POINTER(c_double), c_double, c_double]
		dll.ECAT_McGroupMoveHelicalAbs_Ex.restype = c_int
		dll.ECAT_McGroupMove3DHelicalAbs_CP_Angle_Ex.argtypes = [c_ushort, c_ushort, c_double, c_double, c_double, POINTER(c_double), POINTER(c_double)]
		dll.ECAT_McGroupMove3DHelicalAbs_CP_Angle_Ex.restype = c_int
		dll.ECAT_McGroupMoveHelicalRel_Ex.argtypes = [c_ushort, c_ushort, c_double, POINTER(c_double), c_double, c_double]
		dll.ECAT_McGroupMoveHelicalRel_Ex.restype = c_int           
		dll.ECAT_McGroupMove3DHelicalRel_CP_Angle_Ex.argtypes = [c_ushort, c_ushort, c_double, c_double, c_double, POINTER(c_double), POINTER(c_double)]
		dll.ECAT_McGroupMove3DHelicalRel_CP_Angle_Ex.restype = c_int
		dll.ECAT_McGroupMoveConicalHelixAbs_Ex.argtypes = [c_ushort, c_ushort, c_double, POINTER(c_double), c_double, c_double, c_double]
		dll.ECAT_McGroupMoveConicalHelixAbs_Ex.restype = c_int
		dll.ECAT_McGroupMove3DConicalHelixAbs_CP_Angle_Ex.argtypes = [c_ushort, c_ushort, c_double, c_double, c_double, POINTER(c_double), POINTER(c_double), c_double]
		dll.ECAT_McGroupMove3DConicalHelixAbs_CP_Angle_Ex.restype = c_int
		dll.ECAT_McGroupMoveConicalHelixRel_Ex.argtypes = [c_ushort, c_ushort, c_double, POINTER(c_double), c_double, c_double, c_double]
		dll.ECAT_McGroupMoveConicalHelixRel_Ex.restype = c_int           
		dll.ECAT_McGroupMove3DConicalHelixRel_CP_Angle_Ex.argtypes = [c_ushort, c_ushort, c_double, c_double, c_double, POINTER(c_double), POINTER(c_double), c_double]
		dll.ECAT_McGroupMove3DConicalHelixRel_CP_Angle_Ex.restype = c_int
		dll.ECAT_McGroupMoveProfile_Ex.argtypes = [c_ushort, c_ushort, POINTER(c_ushort), c_ushort]
		dll.ECAT_McGroupMoveProfile_Ex.restype = c_int
		dll.ECAT_McGroupMoveLineAbs_PT_Ex.argtypes = [c_ushort, c_ushort, POINTER(c_double), c_double]
		dll.ECAT_McGroupMoveLineAbs_PT_Ex.restype = c_int
		dll.ECAT_McGroupMoveLineRel_PT_Ex.argtypes = [c_ushort, c_ushort, POINTER(c_double), c_double]
		dll.ECAT_McGroupMoveLineRel_PT_Ex.restype = c_int
		dll.ECAT_McEnable_Directly_Ex.argtypes = [c_ushort]
		dll.ECAT_McEnable_Directly_Ex.restype = c_int    


	#==================================================================
	def GetDllVersion():
		"""Get the dll version."""
		Version = c_char(0)
		Size = c_ushort(0)                
		ret = dll.ECAT_GetDllVersion(byref(Version), byref(Size))
		return ret, Version, Size

	def GetFirmwareVersion():
		"""Get the firmware version."""
		Version = c_char(0)
		Size = c_ushort(0)           
		ret = dll.ECAT_GetFirmwareVersion(byref(Version), byref(Size))
		return ret, Version, Size
        
      
	def GetDeviceCnt():
		"""Get the number of available devices(ECAT-M801/ e-M901)."""
		DeviceCnt = c_ushort(0)
		CardID = (c_ubyte * CARD_DEVICE_NO_MAX)(*[0 for x in range(CARD_DEVICE_NO_MAX)])              
		ret = dll.ECAT_GetDeviceCnt(byref(DeviceCnt), CardID)     
		return ret, DeviceCnt, CardID

	def OpenDevice(DeviceNo):
		"""Open a device (card) with the specified Card ID."""
		ret = dll.ECAT_OpenDevice(DeviceNo)    
		return ret        

	def CloseDevice(DeviceNo):
		"""Close a device (card) with the specified Card ID."""
		ret = dll.ECAT_CloseDevice(DeviceNo)
		return ret        
    
	def GetDeviceSerialNo(DeviceNo):
		"""Get the hardware serial number."""
		SerialNo = (c_ubyte * DEVICE_SERIAL_NO_SIZE)(*[0 for x in range(DEVICE_SERIAL_NO_SIZE)])       
		ret = dll.ECAT_GetDeviceSerialNo(DeviceNo, SerialNo)     
		return ret, SerialNo  
    
	def GetDeviceDI(DeviceNo):
		"""Get  the  on-board  digital  input  data  of  the  specified device.  These digital  inputs  have 
		nothing to do with EtherCAT bus."""
		Value = c_uint(0)
		ret = dll.ECAT_GetDeviceDI(DeviceNo, byref(Value))
		return ret, Value 

	def GetDeviceDIBit(DeviceNo, BitNo):
		"""Get a bit state of a device's on-board digital input."""
		Value = c_uint(0)        
		ret = dll.ECAT_GetDeviceDIBit(DeviceNo, BitNo, byref(Value))
		return ret, Value 

	def GetDeviceDO(DeviceNo):
		"""Get the on-board digital output data of a specified device."""
		Value = c_uint(0)        
		ret = dll.ECAT_GetDeviceDO(DeviceNo, byref(Value))
		return ret, Value  
    
	def GetDeviceDOBit(DeviceNo, BitNo):
		"""Get a bit state of a device's on-board digital output."""
		Value = c_uint(0)
		ret = dll.ECAT_GetDeviceDOBit(DeviceNo, BitNo, byref(Value))
		return ret, Value
    
	def SetDeviceDO(DeviceNo, Value):
		"""Set the on-board digital output data of a device."""
		ret = dll.ECAT_SetDeviceDO(DeviceNo, Value)
		return ret
    
	def SetDeviceDOBit(DeviceNo, BitNo, Value):
		"""Set a bit data of a device's on-board digital output."""
		ret = dll.ECAT_SetDeviceDOBit(DeviceNo, BitNo, Value)  
		return ret
        
	def GetDeviceState(DeviceNo):
		"""Get the EtherCAT network status of a device. This function is always called for checking if the system is running normally."""
		LinkUp = c_uint(0)
		SlavesResp = c_uint(0)
		AlState = c_uint(0)
		Wc = c_uint(0)        
		ret = dll.ECAT_GetDeviceState(DeviceNo, byref(LinkUp), byref(SlavesResp), byref(AlState), byref(Wc))    
		return ret, LinkUp, SlavesResp, AlState, Wc

	def GetSlaveInfo(DeviceNo, SlaveNo):
		"""Get slave information of a slave. 
		When Slavename is blank, it means that the name cannot be read during PreOP, and 
		the name obtained from ESI will be read after OP (V1.0.18 or above)"""
		Alias = c_ushort(0)
		ProductCode = c_uint(0)
		VendorID = c_uint(0)
		RevisionNo = c_uint(0)
		SerialNo = c_uint(0)
		AlState = c_ubyte(0)
		SlaveType = c_uint(0)      
		Slavename = create_string_buffer(MAX_SLAVE_NAME_LENGTH)
		ret = dll.ECAT_GetSlaveInfo(DeviceNo, SlaveNo, byref(Alias), byref(ProductCode), byref(VendorID), byref(RevisionNo), byref(SerialNo), byref(AlState), byref(SlaveType), Slavename)
		return ret, Alias, ProductCode, VendorID, RevisionNo, SerialNo, AlState, SlaveType, Slavename        

	def GetSlaveSdoObject(DeviceNo, SlaveNo, Index, SubIndex, DataSize):
		"""Get SDO data of a slave. Read a data object by means of service data object communication."""
		ObjectVal = c_uint(0)
		AbortCode = c_uint(0)        
		ret = dll.ECAT_GetSlaveSdoObject(DeviceNo, SlaveNo, Index, SubIndex, DataSize, byref(ObjectVal), byref(AbortCode))
		return ret, ObjectVal, AbortCode        
 
	def SetSlaveSdoObject(DeviceNo, SlaveNo, Index, SubIndex, DataSize, ObjectVal):
		"""Set SDO data of a slave. Write a data object by means of service data object communication."""
		AbortCode = c_uint(0)
		ret = dll.ECAT_SetSlaveSdoObject(DeviceNo, SlaveNo, Index, SubIndex, DataSize, ObjectVal, byref(AbortCode))
		return ret, AbortCode

	def StartDeviceOpTask(DeviceNo, NetworkInfoNo, EnumCycleTime, WcErrCnt):
		"""Start the device EtherCAT operation task. At least one network information file must be 
		pre-loaded into this card. This configuration file is used for checking whether the real system 
		is  the  same  as  the  configured  one.  This  function  takes  some  time  to  finish.  Most  of  the 
		motion functions can only be called when the system goes into OP state. After this function 
		is called, users must further use function ECAT_GetDeviceState to check if this operation finishes successfully."""
		ret = dll.ECAT_StartDeviceOpTask(DeviceNo, NetworkInfoNo, EnumCycleTime, WcErrCnt) 
		return ret

	def StopDeviceOpTask(DeviceNo):
		"""Stop the EtherCAT cyclic operation task."""
		ret = dll.ECAT_StopDeviceOpTask(DeviceNo) 
		return ret
    
	def SetSlaveRxPdoData(DeviceNo, SlaveNo, OffsetByte, DataSize, Data):
		"""Set RxPDO data of a slave. Transfer process data to the RxPDO of a slave by means of 
		cyclic communication. Digital outputs and analog outputs of slaves are set by RxPDO data."""
		ret = dll.ECAT_SetSlaveRxPdoData(DeviceNo, SlaveNo, OffsetByte, DataSize, Data)
		return ret
    
	def GetSlaveRxPdoData(DeviceNo, SlaveNo, OffsetByte, DataSize):
		"""Get RxPDO data of a slave. Read process data from the RxPDO of a slave by means of 
		cyclic communication. Digital outputs or analog outputs of slaves are set by RxPDO data. To 
		read the RxPDO data is to read back the status of these outputs."""
		Data = (c_ubyte * DataSize)(*[0 for x in range(DataSize)])              
		ret = dll.ECAT_GetSlaveRxPdoData(DeviceNo, SlaveNo, OffsetByte, DataSize, byref(Data))
		return ret, Data    
    
	def GetSlaveTxPdoData(DeviceNo, SlaveNo, OffsetByte, DataSize):
		"""Get TxPDO data of a slave. Read process data from the TxPDO of a slave by means of 
		cyclic communication. TxPDO data are set by digital inputs or analog inputs of a slave. To 
		read the TxPDO data is to read the status of these inputs."""
		Data = (c_ubyte * DataSize)(*[0 for x in range(DataSize)])
		ret = dll.ECAT_GetSlaveTxPdoData(DeviceNo, SlaveNo, OffsetByte, DataSize, byref(Data))
		return ret, Data
    
	def GetSlaveDI(DeviceNo, SlaveNo):
		"""Get the digital input data of a slave. If a slave is a simple digital input slave, users can 
		use this API to get DI values. Function ECAT_GetSlaveTxPdoData can also do it; but users 
		have to enter more parameters for the same purpose. This function is limited to read up to 
		32 digital inputs of a slave."""
		Value = c_uint(0)
		ret = dll.ECAT_GetSlaveDI(DeviceNo, SlaveNo, byref(Value))
		return ret, Value
    
	def GetSlaveDIBit(DeviceNo, SlaveNo, BitNo):
		"""Get a bit status of a slave's digital input."""
		Value = c_uint(0)
		ret = dll.ECAT_GetSlaveDIBit(DeviceNo, SlaveNo, BitNo, byref(Value))
		return ret, Value
    
	def GetSlaveDO(DeviceNo, SlaveNo):
		"""Get the digital output data of a slave. If a slave is a simple digital output slave, users can 
		use this API to get DO states. Function ECAT_GetSlaveRxPdoData can also do it; but users 
		have to enter more parameters for the same purpose. This function is limited to read up to 
		32 digital outputs of a slave."""
		Value = c_uint(0)
		ret = dll.ECAT_GetSlaveDO(DeviceNo, SlaveNo, byref(Value))
		return ret, Value
    
	def GetSlaveDOBit(DeviceNo, SlaveNo, BitNo):
		"""Get a bit status of a slave's digital output."""
		Value = c_uint(0)
		ret = dll.ECAT_GetSlaveDOBit(DeviceNo, SlaveNo, BitNo, byref(Value))
		return ret, Value
    
	def SetSlaveDO(DeviceNo, SlaveNo, Value):
		"""Set the digital output data of a slave. If a slave is a simple digital input slave, users can 
		use this API to set DO values. Function ECAT_SetSlaveRxPdoData can also do it; but users 
		have to enter more parameters for the same purpose. This function is limited to set up to 32 
		digital outputs of a slave."""
		ret = dll.ECAT_SetSlaveDO(DeviceNo, SlaveNo, Value)
		return ret

	def SetSlaveDOBit(DeviceNo, SlaveNo, BitNo, Value):
		"""Set a bit data of a slave's digital output."""
		ret = c_int(0)
		ret = dll.ECAT_SetSlaveDOBit(DeviceNo, SlaveNo, BitNo, Value)
		return ret
    
	def SetSlaveAoProperty(DeviceNo, SlaveNo, ChannelNo, Range):
		"""Set the AO channel property value. Each AO channel can has different range setting from the others.   
		Note: It supports ECAT-2024 and ECAT-2028."""
		ret = dll.ECAT_SetSlaveAoProperty(DeviceNo, SlaveNo, ChannelNo, Range)
		return ret
    
	def GetSlaveAoProperty(DeviceNo, SlaveNo, ChannelNo):
		"""Get the AO channel property value. Each AO channel can has different range setting from the others. 
		Note: It supports ECAT-2024 and ECAT-2028."""
		Range = c_ubyte(0)
		ret = dll.ECAT_GetSlaveAoProperty(DeviceNo, SlaveNo, ChannelNo, byref(Range))
		return ret, Range
    
	def SetSlaveAoRawData(DeviceNo, SlaveNo, ChannelNo, Data):
		"""Set the 16-bit integer value of an analog output channel. 
		Note: It supports ECAT-2024 and ECAT-2028."""
		ret = dll.ECAT_SetSlaveAoRawData(DeviceNo, SlaveNo, ChannelNo, Data)
		return ret
    
	def GetSlaveAoRawData(DeviceNo, SlaveNo, ChannelNo):
		"""Get the 16-bit integer value of an analog output channel. 
		Note: It supports ECAT-2024 and ECAT-2028."""
		Data = c_ushort(0)
		ret = dll.ECAT_GetSlaveAoRawData(DeviceNo, SlaveNo, ChannelNo, byref(Data))
		return ret, Data
    
	def SetSlaveAoVoltData(DeviceNo, SlaveNo, ChannelNo, Data):
		"""Set the floating-point voltage output value of a specified analog output channel. 
		Note: It supports ECAT-2024 and ECAT-2028."""
		ret = dll.ECAT_SetSlaveAoVoltData(DeviceNo, SlaveNo, ChannelNo, Data)
		return ret
    
	def GetSlaveAoVoltData(DeviceNo, SlaveNo, ChannelNo):
		"""Get the floating-point voltage output value of a specified analog output channel. 
		Note: It supports ECAT-2024 and ECAT-2028."""
		Data = c_double(0)
		ret = dll.ECAT_GetSlaveAoVoltData(DeviceNo, SlaveNo, ChannelNo, byref(Data))
		return ret, Data    
    
	def SetSlaveEncProperty(DeviceNo, SlaveNo, EncNo, Mode, InvertCnt, LPF):
		"""Set the encoder property. This function is designed for encoder module ECAT-2093 and ECAT-2092T."""
		ret = dll.ECAT_SetSlaveEncProperty(DeviceNo, SlaveNo, EncNo, Mode, InvertCnt, LPF)
		return ret

	def GetSlaveEncProperty(DeviceNo, SlaveNo, EncNo):
		"""Get the encoder property settings. This function is designed for encoder module ECAT-2093 and ECAT-2092T."""
		Mode = c_ubyte(0)
		InvertCnt = c_ubyte(0)
		LPF = c_ubyte(0)        
		ret = dll.ECAT_GetSlaveEncProperty(DeviceNo, SlaveNo, EncNo, byref(Mode), byref(InvertCnt), byref(LPF))
		return ret, Mode, InvertCnt, LPF    

	def GetSlaveEncCount(DeviceNo, SlaveNo, EncNo):
		"""Get the encoder counter value. This function is designed for encoder module ECAT-2093 and ECAT-2092T."""
		Cnt = c_int(0)
		ret = dll.ECAT_GetSlaveEncCount(DeviceNo, SlaveNo , EncNo, byref(Cnt))
		return ret, Cnt
    
	def ResetSlaveEncCount(DeviceNo, SlaveNo, EncNo):
		"""Clear the encoder counter value to 0. This function is designed for encoder module ECAT-2093 and ECAT-2092T."""
		ret = dll.ECAT_ResetSlaveEncCount(DeviceNo, SlaveNo , EncNo)
		return ret

	def SetSlaveEncCount(DeviceNo, SlaveNo , EncNo, Value):
		"""Set the encoder counter value. This function is designed for encoder module ECAT-2092T."""
		ret = dll.ECAT_SetSlaveEncCount(DeviceNo, SlaveNo , EncNo, Value)
		return ret

	def SetSlaveEncIdxLatchProperty(DeviceNo, SlaveNo, EncNo, Enable):
		"""Set  the  position  index  latch  function  property  to  be  enabled  or  not.  This  function  is 
		designed for encoder module ECAT-2093 and ECAT-2092T. 
		Note: The index latch of ECAT-2092T is always enabled and can not be disabled."""
		ret = dll.ECAT_SetSlaveEncIdxLatchProperty(DeviceNo, SlaveNo, EncNo, Enable)
		return ret

	def GetSlaveEncIdxLatchProperty(DeviceNo, SlaveNo, EncNo):
		"""Get  the  position  index  latch  function  setting.  This  function  is  designed  for  encoder 
		module ECAT-2093 and ECAT-2092T."""
		Enable = c_ubyte(0)
		ret = dll.ECAT_GetSlaveEncIdxLatchProperty(DeviceNo, SlaveNo, EncNo, byref(Enable))
		return ret    
    
	def GetSlaveEncIdxLatchCnt(DeviceNo, SlaveNo, EncNo):
		"""Read the index latch count. This function is designed for encoder module ECAT-2093 and ECAT-2092T."""
		Cnt = c_int(0)
		ret = dll.ECAT_GetSlaveEncIdxLatchCnt(DeviceNo, SlaveNo, EncNo, byref(Cnt))
		return ret   

	def ResetSlaveEncIdxLatchCnt(DeviceNo, SlaveNo, EncNo):
		"""Reset the index latch count. This function is designed for encoder module ECAT-2092T."""
		ret = dll.ECAT_ResetSlaveEncIdxLatchCnt(DeviceNo, SlaveNo, EncNo)
		return ret
    
	def SetSlaveEncExtLatchProperty(DeviceNo, SlaveNo, EncNo, Enable, Mode, Logic):
		"""Set the position external latch function property. This function is designed for encoder module ECAT-2092T."""
		ret = dll.ECAT_SetSlaveEncExtLatchProperty(DeviceNo, SlaveNo, EncNo, Enable, Mode, Logic)
		return ret
    
	def GetSlaveEncExtLatchProperty(DeviceNo, SlaveNo, EncNo):
		"""Get the position external latch function setting. This function is designed for encoder module ECAT-2092T."""
		Enable = c_ubyte(0)
		Mode = c_ubyte(0)
		Logic = c_ubyte(0)        
		ret = dll.ECAT_GetSlaveEncExtLatchProperty(DeviceNo, SlaveNo, EncNo, byref(Enable), byref(Mode), byref(Logic))
		return ret, Enable, Mode, Logic    
    
	def GetSlaveEncExtLatchCnt(DeviceNo, SlaveNo, EncNo):
		"""Read the external latch count. This function is designed for encoder module ECAT-2092T."""
		Cnt_Rising = c_int(0)
		Cnt_Falling = c_int(0)        
		ret = dll.ECAT_GetSlaveEncExtLatchCnt(DeviceNo, SlaveNo, EncNo, byref(Cnt_Rising), byref(Cnt_Falling))
		return ret, Cnt_Rising, Cnt_Falling    

	def ResetSlaveEncExtLatchCnt(DeviceNo, SlaveNo, EncNo, Cnt):
		"""Reset the external latch count. This function is designed for encoder module ECAT-2092T."""
		ret = dll.ECAT_ResetSlaveEncExtLatchCnt(DeviceNo, SlaveNo, EncNo, Cnt)
		return ret    
    
	def SetSlaveCmpTrigProperty(DeviceNo, SlaveNo, EncNo, PulseWidth):
		"""Set the compare-trigger related properties. This function is designed for encoder module ECAT-2092T."""
		ret = dll.ECAT_SetSlaveCmpTrigProperty(DeviceNo, SlaveNo, EncNo, PulseWidth)
		return ret
    
	def GetSlaveCmpTrigProperty(DeviceNo, SlaveNo, EncNo):
		"""Get the compare-trigger related properties. This function is designed for encoder module ECAT-2092T."""
		PulseWidth = c_ushort(0)
		ret = dll.ECAT_GetSlaveCmpTrigProperty(DeviceNo, SlaveNo, EncNo, byref(PulseWidth))
		return ret, PulseWidth
    
	def SetSlaveCmpTrigData(DeviceNo, SlaveNo, EncNo, CmpData):
		"""According to the setting value, start a single compare-trigger function for an on-board 
		encoder interface channel. This function is designed for encoder module ECAT-2092T."""
		ret = dll.ECAT_SetSlaveCmpTrigData(DeviceNo, SlaveNo, EncNo, CmpData)
		return ret    

	def SetSlaveContCmpTrigData(DeviceNo, SlaveNo, EncNo, Start, Interval, Dir):
		"""Start a continuous or a multiple compare-trigger function. This function is designed for encoder module ECAT-2092T."""
		ret = dll.ECAT_SetSlaveContCmpTrigData(DeviceNo, SlaveNo, EncNo, Start, Interval, Dir)
		return ret        
    
	def McInit(DeviceNo, McSlaveNo, McSubAxisNo, AxisCount):
		"""Initialize parameters for motion control"""
		ret = dll.ECAT_McInit(DeviceNo, McSlaveNo, McSubAxisNo, AxisCount)
		return ret
    
	def McSetAxisServoOn(DeviceNo, AxisNo, State):
		"""Set an axis (a drive) to be servo ON or servo OFF. 
		After  setting  Servo  On,  it  takes  several  ms  to  complete  Servo  On.  You  can  use 
		ECAT_McGetAxisState to get the axis status. When the axis status is MC_AS_STANDSTILL, 
		it means Servo On is completed."""
		ret = dll.ECAT_McSetAxisServoOn(DeviceNo, AxisNo, State)
		return ret
    
	def McSetAxisPPU(DeviceNo, AxisNo, PPU):
		"""Set Pulses Per Unit (PPU) value for an axis. Motion command is based on Unit. Inside 
		the control card, pulses are used for control motors. 
		Pulses Per Unit, pulses of each unit. For example, If you want to set the “user unit” as 
		mm and a mm requires 1000 pulses, then set the PPU to be 1000. If the unit of PDO Entry 
		of the driver is not pulse、pulse/s and pulse/s^2, the PPU needs to be set to 1, at 
		this time, "user unit" is the original unit of the PDO, such as rpm, rpm/s... Etc.."""
		ret = dll.ECAT_McSetAxisPPU(DeviceNo, AxisNo, PPU)
		return ret    
    
	def McGetAxisPPU(DeviceNo, AxisNo):
		"""Get pulses per unit setting of an axis."""
		PPU = c_double(0)
		ret = dll.ECAT_McGetAxisPPU(DeviceNo, AxisNo, byref(PPU))
		return ret, PPU
    
	def McSetAxisEncoderPPR(DeviceNo, AxisNo, PPR):
		"""Set encoder pulses per revolution value of an axis, For encoder and motor scaling. 
		Note: For Encoder module / Stepper motor controller."""
		ret = dll.ECAT_McSetAxisEncoderPPR(DeviceNo, AxisNo, PPR)
		return ret
    
	def McGetAxisEncoderPPR(DeviceNo, AxisNo):
		"""Get encoder pulses per revolution of an axis. 
		Note: For Encoder module / Stepper motor controller."""
		PPR = c_uint(0)
		ret = dll.ECAT_McGetAxisEncoderPPR(DeviceNo, AxisNo, byref(PPR))
		return ret, PPR
    
	def McSetAxisMotorPPR(DeviceNo, AxisNo, PPR):
		"""Set motor pulses per revolution of an axis, For encoder and motor scaling. 
		Note: For Encoder module / Stepper motor controller."""
		ret = dll.ECAT_McSetAxisMotorPPR(DeviceNo, AxisNo, PPR)
		return ret
    
	def McGetAxisMotorPPR(DeviceNo, AxisNo):
		"""Get motor pulses per revolution of an axis. 
		Note: For Encoder module / Stepper motor controller."""
		PPR = c_uint(0)
		ret = dll.ECAT_McGetAxisMotorPPR(DeviceNo, AxisNo, byref(PPR))
		return ret
    
	def McSetAxisHomeMethod(DeviceNo, AxisNo, Method):
		"""Set the homing method of an axis."""
		ret = dll.ECAT_McSetAxisHomeMethod(DeviceNo, AxisNo, Method)
		return ret
    
	def McGetAxisHomeMethod(DeviceNo, AxisNo):
		"""Get the homing method of an axis. Please refer to the user manual of this CiA402 servo drive."""
		Method = c_int(0)
		ret = dll.ECAT_McGetAxisHomeMethod(DeviceNo, AxisNo, byref(Method))
		return ret    
    
	def McSetAxisHomeSpeed(DeviceNo, AxisNo, SeachSw, SeachZr):
		"""Set the homing speed settings of an axis. SeachSw speed is used for searching the 
		home sensor; SeachZr speed is used for searching the encoder index Z signal."""
		ret = dll.ECAT_McSetAxisHomeSpeed(DeviceNo, AxisNo, SeachSw, SeachZr)
		return ret
    
	def McGetAxisHomeSpeed(DeviceNo, AxisNo):
		"""Get the homing speed settings of an axis. SeachSw speed is used for searching the 
		home sensor; SeachZr speed is used for searching the encoder index Z signal."""
		SeachSw = c_double(0)
		SeachZr = c_double(0)        
		ret = dll.ECAT_McGetAxisHomeSpeed(DeviceNo, AxisNo, byref(SeachSw), byref(SeachZr))
		return ret, SeachSw, SeachZr
    
	def McSetAxisHomeAcc(DeviceNo, AxisNo, Acc):
		"""Set homing acceleration of an axis."""
		ret = dll.ECAT_McSetAxisHomeAcc(DeviceNo, AxisNo, Acc)
		return ret
    
	def McGetAxisHomeAcc(DeviceNo, AxisNo):
		"""Get homing acceleration of an axis."""
		Acc = c_double(0)
		ret = dll.ECAT_McGetAxisHomeAcc(DeviceNo, AxisNo, byref(Acc))
		return ret, Acc    
    
	def McSetAxisHomeOffset(DeviceNo, AxisNo, HomeOffset):
		"""Set home offset to an axis."""
		ret = dll.ECAT_McSetAxisHomeOffset(DeviceNo, AxisNo, HomeOffset)
		return ret
  
	def McGetAxisHomeOffset(DeviceNo, AxisNo):
		"""Get home offset of an axis."""
		HomeOffset = c_double(0)
		ret = dll.ECAT_McGetAxisHomeOffset(DeviceNo, AxisNo, byref(HomeOffset))
		return ret, HomeOffset
    
	def McSetAxisAccTime(DeviceNo, AxisNo, Time_ms):
		"""Set acceleration time of an axis."""
		ret = dll.ECAT_McSetAxisAccTime(DeviceNo, AxisNo, Time_ms)
		return ret
 
	def McGetAxisAccTime(DeviceNo, AxisNo):
		"""Get acceleration time of an axis."""
		Time_ms = c_ushort(0)
		ret = dll.ECAT_McGetAxisAccTime(DeviceNo, AxisNo, byref(Time_ms))
		return ret, Time_ms
    
	def McSetAxisAccDecType(DeviceNo, AxisNo, Type):
		"""Set acceleration type of an axis."""
		ret = dll.ECAT_McSetAxisAccDecType(DeviceNo, AxisNo, Type)
		return ret
    
	def McGetAxisAccDecType(DeviceNo, AxisNo):
		"""Get acceleration type of an axis."""
		Type = c_ushort(0)
		ret = dll.ECAT_McGetAxisAccDecType(DeviceNo, AxisNo, byref(Type))
		return ret, Type
    
	def McSetAxisTouchProbeProperty(DeviceNo, AxisNo, ProbeNo, Enable, Logic):
		"""Configure  Touch  Probe  function  of  an  axis.  Servo  drives  can  have  up  to  two  Touch 
		Probe inputs. But some have only one, and some have none."""
		ret = dll.ECAT_McSetAxisTouchProbeProperty(DeviceNo, AxisNo, ProbeNo, Enable, Logic)
		return ret
    
	def McGetAxisTouchProbeProperty(DeviceNo, AxisNo, ProbeNo):
		"""Get the property settings of Touch Probe function of an axis. Servo drives can have up 
		to two Touch Probe inputs. But some have only one, and some have none."""
		Enable = c_ubyte(0)
		Logic = c_ubyte(0)        
		ret = dll.ECAT_McGetAxisTouchProbeProperty(DeviceNo, AxisNo, ProbeNo, byref(Enable), byref(Logic))
		return ret, Enable, Logic    
    
	def McGetAxisTouchProbeValue(DeviceNo, AxisNo, ProbeNo):
		"""Get the Touch Probe value of an axis."""
		Value = c_double(0)
		ret = dll.ECAT_McGetAxisTouchProbeValue(DeviceNo, AxisNo, ProbeNo, byref(Value))
		return ret, Value 
    
	def McGetAxisState(DeviceNo, AxisNo):
		"""Get the state of an axis."""
		if AxisNo == 65535:
			State = (c_uint * MC_AXIS_NO_MAX)(*[0 for x in range(MC_AXIS_NO_MAX)]) 
			ret = dll.ECAT_McGetAxisState(DeviceNo, AxisNo, State) 
			return ret, State
		else:
			State = c_uint(0)
			ret = dll.ECAT_McGetAxisState(DeviceNo, AxisNo, byref(State))
			return ret, State  
    
	def McGetAxisState2(DeviceNo, AxisNo):
		"""Get the state of an axis."""
		AxisState = (c_uint * MC_AXIS_NO_MAX)(*[0 for x in range(MC_AXIS_NO_MAX)])        
		ret = dll.ECAT_McGetAxisState(DeviceNo, AxisNo, AxisState)     
		return ret, AxisState
    
	def McGetAxisLastError(DeviceNo, AxisNo):
		"""Get last error of an axis."""
		if AxisNo == 65535:
			Error = (c_int * MC_AXIS_NO_MAX)(*[0 for x in range(MC_AXIS_NO_MAX)]) 
			ret = dll.ECAT_McGetAxisLastError(DeviceNo, AxisNo, Error) 
			return ret, Error
		else:
			Error = c_int(0)
			ret = dll.ECAT_McGetAxisLastError(DeviceNo, AxisNo, byref(Error))
			return ret, Error        
    
	def McGetAxisDriveError(DeviceNo, AxisNo):
		"""Get the drive error of an axis."""
		if AxisNo == 65535:
			DriveError = (c_ushort * MC_AXIS_NO_MAX)(*[0 for x in range(MC_AXIS_NO_MAX)]) 
			ret = dll.ECAT_McGetAxisDriveError(DeviceNo, AxisNo, DriveError) 
			return ret, DriveError
		else:
			DriveError = c_ushort(0)
			ret = dll.ECAT_McGetAxisDriveError(DeviceNo, AxisNo, byref(DriveError))
			return ret, DriveError        
    
	def McGetAxisCommandPos(DeviceNo, AxisNo):
		"""Get command position of an axis. 
		Note: When AxisNo is set to 65535, command positions of all axes are read back in Pos array pointer."""
		if AxisNo == 65535:
			AxisPos = (c_double * MC_AXIS_NO_MAX)(*[0 for x in range(MC_AXIS_NO_MAX)]) 
			ret = dll.ECAT_McGetAxisCommandPos(DeviceNo, AxisNo, AxisPos) 
			return ret, AxisPos
		else:
			AxisPos = c_double(0)
			ret = dll.ECAT_McGetAxisCommandPos(DeviceNo, AxisNo, byref(AxisPos))
			return ret, AxisPos        
    
	def McGetAxisActualPos(DeviceNo, AxisNo):
		"""Get actual position of an axis. 
		Note: When AxisNo is set to 65535, actual positions of all axes are read back in Pos array pointer."""
		if AxisNo == 65535:
			AxisPos = (c_double * MC_AXIS_NO_MAX)(*[0 for x in range(MC_AXIS_NO_MAX)]) 
			ret = dll.ECAT_McGetAxisActualPos(DeviceNo, AxisNo, AxisPos) 
			return ret, AxisPos
		else:
			AxisPos = c_double(0)
			ret = dll.ECAT_McGetAxisActualPos(DeviceNo, AxisNo, byref(AxisPos))
			return ret, AxisPos
    
	def McGetAxisActualPos2(DeviceNo, AxisNo):
		"""Get actual position of an axis. 
		Note: When AxisNo is set to 65535, actual positions of all axes are read back in Pos array pointer."""
		AxisPos = (c_double * MC_AXIS_NO_MAX)(*[0 for x in range(MC_AXIS_NO_MAX)])        
		ret = dll.ECAT_McGetAxisActualPos(DeviceNo, AxisNo, AxisPos)
		return ret, AxisPos    
    
	def McGetAxisActualVel(DeviceNo, AxisNo):
		"""Get actual velocity of an axis. 
		Note: When AxisNo is set to 65535, the actual velocities of all axes are read back in Vel array pointer."""
		if AxisNo == 65535:
			AxisVel = (c_double * MC_AXIS_NO_MAX)(*[0 for x in range(MC_AXIS_NO_MAX)]) 
			ret = dll.ECAT_McGetAxisActualVel(DeviceNo, AxisNo, AxisVel) 
			return ret, AxisVel
		else:
			AxisVel = c_double(0)
			ret = dll.ECAT_McGetAxisActualVel(DeviceNo, AxisNo, byref(AxisVel))
			return ret, AxisVel        
    
	def McGetAxisActualVel2(DeviceNo, AxisNo):
		"""Get actual velocity of an axis. 
		Note: When AxisNo is set to 65535, the actual velocities of all axes are read back in Vel array pointer."""
		AxisVel = (c_double * MC_AXIS_NO_MAX)(*[0 for x in range(MC_AXIS_NO_MAX)])                
		ret = dll.ECAT_McGetAxisActualVel(DeviceNo, AxisNo, AxisVel)      
		return ret, AxisVel        
    
	def McGetAxisDI(DeviceNo, AxisNo):
		"""Get digital inputs of an axis. Most of digital inputs are available in the drive."""
		if AxisNo == 65535:
			AxisDI = (c_uint * MC_AXIS_NO_MAX)(*[0 for x in range(MC_AXIS_NO_MAX)]) 
			ret = dll.ECAT_McGetAxisDI(DeviceNo, AxisNo, AxisDI) 
			return ret, AxisDI
		else:
			AxisDI = c_uint(0)
			ret = dll.ECAT_McGetAxisDI(DeviceNo, AxisNo, byref(AxisDI))
			return ret, AxisDI         
    
	def McSetProfileData(DeviceNo, ProfileNo, Data, DataSize):
		"""Set a position array data into a buffer number for profile motion(ECAT_McAxisMoveProfile)."""
		ret = dll.ECAT_McSetProfileData(DeviceNo, ProfileNo, Data, DataSize)
		return ret
    
	def McGetProfileData(DeviceNo, ProfileNo, DataSize):
		"""Get a position array data from a profile buffer number."""
		Data = (c_double * DataSize)(*[0 for x in range(DataSize)])
		ret = dll.ECAT_McGetProfileData(DeviceNo, ProfileNo, Data, DataSize)
		return ret, Data

	def McSetProfileCSV(DeviceNo, ProfileNo, Offset, Data, DataSize, LastFlag):
		"""Write position data to a CSV file. This file contain data for a profile motion."""
		ret = dll.ECAT_McSetProfileCSV(DeviceNo, ProfileNo, Offset, Data, DataSize, LastFlag)
		return ret
    
	def McGetProfileCSV(DeviceNo, ProfileNo, DataSize):
		"""Read out position data from a CSV file."""
		Offset = c_uint(0)
		Data = create_string_buffer(DataSize)
		LastFlag = c_ubyte(0)        
		ret = dll.ECAT_McGetProfileCSV(DeviceNo, ProfileNo, byref(Offset), Data, byref(DataSize), byref(LastFlag))
		return ret, Offset, Data, LastFlag
    
	def McSetEcamTable(DeviceNo, TableNo, Data, DataSize):
		"""Set the slave position data for an E-CAM table."""
		ret = dll.ECAT_McSetEcamTable(DeviceNo, TableNo, Data, DataSize)
		return ret

	def McGetEcamTable(DeviceNo, TableNo, DataSize):
		"""Get the slave position data from an E-CAM table."""
		Data = (c_double * DataSize)(*[0 for x in range(DataSize)]) 
		ret = dll.ECAT_McGetEcamTable(DeviceNo, TableNo, Data, DataSize)
		return ret, Data
    
	def McConfigEcamTable(DeviceNo, TableNo, SlaveAbs):
		"""Set relative/absolute position property of an E-CAM table."""
		ret = dll.ECAT_McConfigEcamTable(DeviceNo, TableNo, SlaveAbs)
		return ret
    
	def McAxisHome(DeviceNo, AxisNo):
		"""Start home motion of an axis. 
		Note: Since a few servo drives do not support the dynamic settings of some 
		home-related parameters, such as the home acceleration setting, an error may occur. 
		Another function ECAT_McAxisHomeEx is provided for dealing with this kind of drives."""
		ret = dll.ECAT_McAxisHome(DeviceNo, AxisNo)
		return ret    
    
	def McAxisErrorReset(DeviceNo, AxisNo):
		"""Reset the error state of an axis. 
		After using this function, it takes a few ms to complete the reset axis state, you can use 
		ECAT_McGetAxisState to get the axis state, when the axis state is MC_AS_STANDSTILL(In 
		Servo On state) or MC_AS_DISABLED(In Servo Off state), it means that the reset axis state is completed."""
		ret = dll.ECAT_McAxisErrorReset(DeviceNo, AxisNo)
		return ret    
    
	def McAxisStop(DeviceNo, AxisNo):
		"""Stop an axis with deceleration."""
		ret = dll.ECAT_McAxisStop(DeviceNo, AxisNo)
		return ret
    
	def McAxisQuickStop(DeviceNo, AxisNo):
		"""Stop an axis quickly (immediately)."""
		ret = dll.ECAT_McAxisQuickStop(DeviceNo, AxisNo)
		return ret
    
	def McAxisMoveAbs(DeviceNo, AxisNo, AxisPos, AxisVel):
		"""Start an absolute position motion of an axis."""
		ret = dll.ECAT_McAxisMoveAbs(DeviceNo, AxisNo, AxisPos, AxisVel)
		return ret
    
	def McAxisMoveRel(DeviceNo, AxisNo, AxisPos, AxisVel):
		"""Start an relative position motion of an axis."""
		ret = dll.ECAT_McAxisMoveRel(DeviceNo, AxisNo, AxisPos, AxisVel)
		return ret
    
	def McAxisChangePos(DeviceNo, AxisNo, AxisPos):
		"""When the specified axis is in motion, this motion command can be used to change its end position."""
		ret = dll.ECAT_McAxisChangePos(DeviceNo, AxisNo, AxisPos)
		return ret
 
	def McAxisChangeVel(DeviceNo, AxisNo, AxisVel):
		"""When the specified axis is in motion, this motion command can be used to change the velocity."""
		ret = dll.ECAT_McAxisChangeVel(DeviceNo, AxisNo, AxisVel)
		return ret
    
	def McAxisMoveVel(DeviceNo, AxisNo, AxisVel):
		"""Start a never ending movement with a specified velocity.   
		Note: A velocity control mode (CSV) is used."""
		ret = dll.ECAT_McAxisMoveVel(DeviceNo, AxisNo, AxisVel)
		return ret
    
	def McAxisMoveVelByPos(DeviceNo, AxisNo, AxisVel):
		"""Start a velocity motion with a specified velocity.   
		Note: A position control mode (CSP) is used."""
		ret = dll.ECAT_McAxisMoveVelByPos(DeviceNo, AxisNo, AxisVel)
		return ret
    
	def McAxisGearIn(DeviceNo, MasterNo, SlaveNo, RatioNum, RationDen, SyncSource) :
		"""Start  a  gear  synchronization  motion  with  a  speed  ratio  between  a  slave  axis  and  its master axis."""
		ret = dll.ECAT_McAxisGearIn(DeviceNo, MasterNo, SlaveNo, RatioNum, RationDen, SyncSource) 
		return ret
    
	def McAxisGearOut(DeviceNo, SlaveNo, Stop):
		"""Disengages the slave axis from the master axis. After disengagement, the slave axis 
		can either keep moving with the last velocity, stop slowly, or stop immediately."""
		ret = dll.ECAT_McAxisGearOut(DeviceNo, SlaveNo, Stop)
		return ret    
    
	def McAxisMoveProfile(DeviceNo, AxisNo, ProfileNo, TotalStep):
		"""Start a profile position motion of an axis. A profile buffer is an array that contains a lot of 
		pre-defined motion points. Up to 3000 points can be defined for a single profile. If more than 
		3000  points  are  required,  please  use  function ECAT_McAxisMoveProfileCSV.  
 		Function ECAT_McSetProfileInterval will affect the data consuming rate. 
		Note: Set profile by using ECAT_McSetProfileData."""
		ret = dll.ECAT_McAxisMoveProfile(DeviceNo, AxisNo, ProfileNo, TotalStep)
		return ret    
    
	def McAxisMoveProfileCSV(DeviceNo, AxisNo, ProfileNo):
		"""Start a profile position motion of an axis. A file contains all the position data for a profile 
		motion. ECAT_McSetProfileInterval will affect the data consuming rate. 
		Note: Set profile by using ECAT_McSetProfileCSV."""
		ret = dll.ECAT_McAxisMoveProfileCSV(DeviceNo, AxisNo, ProfileNo)
		return ret
    
	def McAxisCamIn(DeviceNo, MasterNo, SlaveNo, TableNo, SyncSource, MasterInterval, SlaveScaling):
		"""Start E-CAM synchronization motion with a table defining the relationship of a slave axis and its master axis."""
		ret = dll.ECAT_McAxisCamIn(DeviceNo, MasterNo, SlaveNo, TableNo, SyncSource, MasterInterval, SlaveScaling)
		return ret
  
	def McAxisCamOut(DeviceNo, SlaveNo):
		"""Stop a slave axis for performing an E-CAM synchronization motion. After an axis is set to Cam out, it stops immediately."""
		ret = dll.ECAT_McAxisCamOut(DeviceNo, SlaveNo)
		return ret

	def McAxisCamPhaseShift(DeviceNo, SlaveNo, PhaseShift):
		"""Set  the  phase  shift  between  the  master  axis  and  the  slave  axis  for  an  E-CAM 
		synchronization motion. Phase shift changes the starting point of the master axis relative to 
		the slave axis in the CAM table."""
		ret = dll.ECAT_McAxisCamPhaseShift(DeviceNo, SlaveNo, PhaseShift) 
		return ret
    
	def McAxisTangentInGroup(DeviceNo, TangentInAxisNo, GroupNo, AxisAngle, AxisVel):
		"""Start a tangent motion. Tangent motion is a name simplified from tangential following 
		motion. It defines an axis to rotate by following the tangential direction of a continuous profile 
		which is generated by a two-axis group motion. If the vector direction is not continuous for a 
		new group motion, the rotating axis is assigned a new angle to match with the new direction 
		by calling this tangent-in function."""
		ret = dll.ECAT_McAxisTangentInGroup(DeviceNo, TangentInAxisNo, GroupNo, AxisAngle, AxisVel)
		return ret
    
	def McAddAxisToGroup(DeviceNo, GroupNo, AxisNo):
		"""Add one axis to a group."""
		ret = dll.ECAT_McAddAxisToGroup(DeviceNo, GroupNo, AxisNo)
		return ret
 
	def McRemoveAxisFromGroup(DeviceNo, GroupNo, AxisNo):
		"""Remove one axis from a group."""
		ret = dll.ECAT_McRemoveAxisFromGroup(DeviceNo, GroupNo, AxisNo)
		return ret
    
	def McUngroupAllAxes(DeviceNo, GroupNo):
		"""Remove all axes from a group. This group no longer owns any axis."""
		ret = dll.ECAT_McUngroupAllAxes(DeviceNo, GroupNo)
		return ret
    
	def McSetGroupCmdMode(DeviceNo, GroupNo, CmdMode):
		"""This function will set the command mode of a group immediately. The group command 
		mode will decide how a motion command is processed. There are three command modes: 
		aborting, buffered, and blending."""
		ret = dll.ECAT_McSetGroupCmdMode(DeviceNo, GroupNo, CmdMode)
		return ret

	def McGetGroupCmdMode(DeviceNo, GroupNo):
		"""Get the group command mode of a group."""
		CmdMode = c_ushort(0)
		ret = dll.ECAT_McGetGroupCmdMode(DeviceNo, GroupNo, byref(CmdMode))
		return ret, CmdMode
    
	def McGetGroupState(DeviceNo, GroupNo):
		"""Get the state of a group."""
		State = c_uint(0)
		ret = dll.ECAT_McGetGroupState(DeviceNo, GroupNo, byref(State))
		return ret, State
    
	def McGetGroupCmdBuffer(DeviceNo, GroupNo):
		"""Get the number of commands buffered inside a group buffer."""
		BufferCnt = c_ushort(0)
		ret = dll.ECAT_McGetGroupCmdBuffer(DeviceNo, GroupNo, byref(BufferCnt))
		return ret, BufferCnt    

	def McSetGroupAccTime(DeviceNo, GroupNo, Time_ms):
		"""Set the acceleration time of a group."""
		ret = dll.ECAT_McSetGroupAccTime(DeviceNo, GroupNo, Time_ms)
		return ret    

	def McGetGroupAccTime(DeviceNo, GroupNo):
		"""Get the acceleration time of a group."""
		Time_ms = c_ushort(0)
		ret = dll.ECAT_McGetGroupAccTime(DeviceNo, GroupNo, byref(Time_ms))
		return ret, Time_ms
    
	def McSetGroupAccDecType(DeviceNo, GroupNo, Type):
		"""Set the acceleration type of a group. There are two acceleration types: T-Curve (linear acceleration) and S-Curve.   
		Note: The T-curve acceleration time is set by function ECAT_McSetGroupAccTime.
		However, the S-curve acceleration time is twice the acceleration time set by that function."""
		ret = dll.ECAT_McSetGroupAccDecType(DeviceNo, GroupNo, Type)
		return ret

	def McGetGroupAccDecType(DeviceNo, GroupNo):
		"""Get the acceleration type of a group."""
		Type = c_ushort(0)
		ret = dll.ECAT_McGetGroupAccDecType(DeviceNo, GroupNo, byref(Type))
		return ret, Type    
    
	def McGroupStop(DeviceNo, GroupNo):
		"""Stop the motion of a group with deceleration."""
		ret = dll.ECAT_McGroupStop(DeviceNo, GroupNo)
		return ret

	def McGroupQuickStop(DeviceNo, GroupNo):
		"""Stop the motion of a group immediately."""
		ret = dll.ECAT_McGroupQuickStop(DeviceNo, GroupNo)
		return ret    
    
	def McGroupQuickStop(DeviceNo, GroupNo):
		"""Stop the motion of a group immediately."""
		ret = dll.ECAT_McGroupQuickStop(DeviceNo, GroupNo)
		return ret
    
	def McGroupMoveLineAbs(DeviceNo, GroupNo, GroupPos, GroupVel):
		"""Start an absolute linear interpolation motion of a group. An array of position data of axes 
		and a velocity are requested to enter."""
		ret = dll.ECAT_McGroupMoveLineAbs(DeviceNo, GroupNo, GroupPos, GroupVel)
		return ret
    
	def McGroupMoveLineAbs_PT(DeviceNo, GroupNo, GroupPos, GroupTime):
		"""Start an absolute linear interpolation motion of a group. An array of position data of axes 
		and  action  time  are  requested  to  enter.  The  command  speed  of  each  axis  is  calculated 
		according to the position data and the time value."""
		ret = dll.ECAT_McGroupMoveLineAbs_PT(DeviceNo, GroupNo, GroupPos, GroupTime)
		return ret    
    
	def McGroupMoveLineAbs_PVT(DeviceNo, GroupNo, AxisPos, AxisVel, Time):
		"""Start an absolute PVT motion of a group."""
		ret = dll.ECAT_McGroupMoveLineAbs_PVT(DeviceNo, GroupNo, AxisPos, AxisVel, Time)
		return ret 
    
	def McGroupMoveLineAbs_PTexT(DeviceNo, GroupNo, GroupPos, GroupTime, extendTime):
		"""This function is for factory use only   
		Start an absolute linear interpolation motion of a group. An array of position data of axes 
		and action time are requested to enter. The command speed of each axis is calculated 
		according to the position data and the time value. Use command speed to extend action time by extension time. 
		Ex:  Current position of axis is 0, command position is 1, Time is 1, extension Time = 0.5 
		command speed = command position / Time => 1/1 = 1 
		totoal action time = Time + extension Time = 1 + 0.5 = 1.5 
		Actual moving distance = command speed * totoal action time = 1*1.5 = 1.5 ."""
		ret = dll.ECAT_McGroupMoveLineAbs_PTexT(DeviceNo, GroupNo, GroupPos, GroupTime, extendTime)
		return ret    

	def McGroupMoveLineAbs_PPT(DeviceNo, GroupNo, GroupPos, GroupTime):
		"""Start an absolute curve interpolation motion of a group. An array of position data of axes 
		and action time are requested to enter.   
		This command is a cubic smooth curve and will pass through the target point."""
		ret = dll.ECAT_McGroupMoveLineAbs_PPT(DeviceNo, GroupNo, GroupPos, GroupTime)
		return ret    
    
	def McGroupMoveLineRel(DeviceNo, GroupNo, GroupPos, GroupVel):
		"""Start a relative linear interpolation motion of a group. An array of distance data of axes 
		and a velocity are requested to enter."""
		ret = dll.ECAT_McGroupMoveLineRel(DeviceNo, GroupNo, GroupPos, GroupVel)
		return ret
    
	def McGroupMoveLineRel_PT(DeviceNo, GroupNo, GroupPos, GroupTime):
		"""Start a relative linear interpolation motion of a group. An array of distance data of axes 
		and  action  time  are  requested  to  enter.  The  command  speed  of  each  axis  is  calculated 
		according to the distance data and the time value."""
		ret = dll.ECAT_McGroupMoveLineRel_PT(DeviceNo, GroupNo, GroupPos, GroupTime)
		return ret    
    
	def McGroupMoveLineRel_PVT(DeviceNo, GroupNo, AxisPos, AxisVel, Time):
		"""Start an relative PVT motion of a group."""
		ret = dll.ECAT_McGroupMoveLineRel_PVT(DeviceNo, GroupNo, AxisPos, AxisVel, Time)
		return ret

	def McGroupMoveLineRel_PTexT(DeviceNo, GroupNo, GroupPos, GroupTime, extendTime):
		"""This function is for factory use only   
		Start an relative linear interpolation motion of a group. An array of position data of axes 
		and action time are requested to enter. The command speed of each axis is calculated 
		according to the position data and the time value. Use command speed to extend action time by extension time. 
		Ex:  Current position of axis is 0, command position is 1, Time is 1, extension Time = 0.5 
		command speed = command position / Time => 1/1 = 1 
		totoal action time = Time + extension Time = 1 + 0.5 = 1.5 
		Actual moving distance = command speed * totoal action time = 1*1.5 = 1.5 ."""
		ret = dll.ECAT_McGroupMoveLineRel_PTexT(DeviceNo, GroupNo, GroupPos, GroupTime, extendTime)
		return ret    

	def McGroupMoveLineRel_PPT(DeviceNo, GroupNo, GroupPos, GroupTime):
		"""Start an relative curve interpolation motion of a group. An array of position data of axes 
		and action time are requested to enter.   
		This command is a cubic smooth curve and will pass through the target point."""
		ret = dll.ECAT_McGroupMoveLineRel_PPT(DeviceNo, GroupNo, GroupPos, GroupTime)
		return ret

	def McGroupMoveShaker(DeviceNo, GroupNo, Amp, phase, Freq, Time):
		"""This function is for factory use only.         
		Start a relative sine wave motion of a group. 
		Note:(1) Can be used with some group commands 
		(2) Blending or buffer mode is not supported, it can be used in blending or buffer 
		mode, but will abort relative sine wave movement in progress. 
		Y = Amp * sin(2 * pi * Freq * t + phase) 
		t = 0 to Time ."""
		ret = dll.ECAT_McGroupMoveShaker(DeviceNo, GroupNo, Amp, phase, Freq, Time)
		return ret    
    
	def McGroupMoveCircularAbs_CP_Angle(DeviceNo, GroupNo, GroupVel, CircAngle , CircAuxPos):
		"""Start an absolute 2D circular interpolation motion by providing the center position and its angle."""
		ret = dll.ECAT_McGroupMoveCircularAbs_CP_Angle(DeviceNo, GroupNo, GroupVel, CircAngle , CircAuxPos)
		return ret
    
	def McGroupMoveCircularAbs_CP_EP(DeviceNo, GroupNo, GroupVel, CircDir , CircAuxPos, CircEndPos):
		"""Start an absolute 2D circular interpolation motion by providing the center position and the end position."""
		ret = dll.ECAT_McGroupMoveCircularAbs_CP_EP(DeviceNo, GroupNo, GroupVel, CircDir , CircAuxPos, CircEndPos)
		return ret    

	def McGroupMoveCircularAbs_BP_EP(DeviceNo, GroupNo, GroupVel, CircDir , CircAuxPos, CircEndPos):
		"""Start an absolute 2D circular interpolation motion by providing a border position and the end position."""
		ret = dll.ECAT_McGroupMoveCircularAbs_BP_EP(DeviceNo, GroupNo, GroupVel, CircDir , CircAuxPos, CircEndPos)
		return ret
    
	def McGroupMove3DCircularAbs_CP_Angle(DeviceNo, GroupNo, GroupVel , CircAngle, CircAuxPos, NV):
		"""Start an absolute 3D circular interpolation motion by providing the center position and an angle."""
		ret = dll.ECAT_McGroupMove3DCircularAbs_CP_Angle(DeviceNo, GroupNo, GroupVel , CircAngle, CircAuxPos, NV)
		return ret    

	def McGroupMove3DCircularAbs_CP_EP(DeviceNo, GroupNo, GroupVel, CircDir, CircAuxPos, CircEndPos):
		"""Start an absolute 3D circular interpolation motion by providing the center position and the end position."""
		ret = dll.ECAT_McGroupMove3DCircularAbs_CP_EP(DeviceNo, GroupNo, GroupVel, CircDir, CircAuxPos, CircEndPos)
		return ret
    
	def McGroupMove3DCircularAbs_BP_EP(DeviceNo, GroupNo, GroupVel , CircDir, CircAuxPos, CircEndPos):
		"""Start an absolute 3D circular interpolation motion by providing a border position and the end position."""
		ret = dll.ECAT_McGroupMove3DCircularAbs_BP_EP(DeviceNo, GroupNo, GroupVel , CircDir, CircAuxPos, CircEndPos)
		return ret    
    
	def McGroupMoveCircularRel_CP_Angle(DeviceNo, GroupNo, GroupVel, CircAngle , CircAuxPos):
		"""Start a relative 2D circular interpolation motion by providing the center position and its angle."""
		ret = dll.ECAT_McGroupMoveCircularRel_CP_Angle(DeviceNo, GroupNo, GroupVel, CircAngle , CircAuxPos)
		return ret     

	def McGroupMoveCircularRel_CP_EP(DeviceNo, GroupNo, GroupVel, CircDir, CircAuxPos, CircEndPos):
		"""Start a relative 2D circular interpolation motion by providing the center position and the end position."""
		ret = dll.ECAT_McGroupMoveCircularRel_CP_EP(DeviceNo, GroupNo, GroupVel, CircDir, CircAuxPos, CircEndPos)
		return ret
    
	def McGroupMoveCircularRel_BP_EP(DeviceNo, GroupNo, GroupVel, CircDir, CircAuxPos, CircEndPos):
		"""Start a relative 2D circular interpolation motion by providing a border position and the end position."""
		ret = dll.ECAT_McGroupMoveCircularRel_BP_EP(DeviceNo, GroupNo, GroupVel, CircDir, CircAuxPos, CircEndPos)
		return ret    
        
	def McGroupMove3DCircularRel_CP_Angle(DeviceNo, GroupNo, GroupVel, CircAngle, CircAuxPos, NV):
		"""Start a relative 3D circular interpolation motion by providing the center position and an angle."""
		ret = dll.ECAT_McGroupMove3DCircularRel_CP_Angle(DeviceNo, GroupNo, GroupVel, CircAngle, CircAuxPos, NV)
		return ret          

	def McGroupMove3DCircularRel_CP_EP(DeviceNo, GroupNo, GroupVel, CircDir, CircAuxPos, CircEndPos):
		"""Start a relative 3D circular interpolation motion by providing the center position and the end position."""
		ret = dll.ECAT_McGroupMove3DCircularRel_CP_EP(DeviceNo, GroupNo, GroupVel, CircDir, CircAuxPos, CircEndPos)
		return ret
    
	def McGroupMove3DCircularRel_BP_EP(DeviceNo, GroupNo, GroupVel, CircDir, CircAuxPos, CircEndPos):
		"""Start a relative 3D circular interpolation motion by providing a border position and the end position."""
		ret = dll.ECAT_McGroupMove3DCircularRel_BP_EP(DeviceNo, GroupNo, GroupVel, CircDir, CircAuxPos, CircEndPos)
		return ret    
    
	def McGroupMoveHelicalAbs(DeviceNo, GroupNo, CircAngle, CircAuxPos, HelicalPitch, GroupVel):
		"""Start a helical interpolation motion."""
		ret = dll.ECAT_McGroupMoveHelicalAbs(DeviceNo, GroupNo, CircAngle, CircAuxPos, HelicalPitch, GroupVel)
		return ret
    
	def McGroupMove3DHelicalAbs_CP_Angle(DeviceNo, GroupNo, GroupVel, CircAngle, HelicalPitch, CircAuxPos, NV):
		"""Start an absolute 3D helical interpolation motion."""
		ret = dll.ECAT_McGroupMove3DHelicalAbs_CP_Angle(DeviceNo, GroupNo, GroupVel, CircAngle, HelicalPitch, CircAuxPos, NV)
		return ret    
    
	def McGroupMoveHelicalRel(DeviceNo, GroupNo, CircAngle, CircAuxPos, HelicalPitch, GroupVel):
		"""Start a relative helical interpolation motion."""
		ret = dll.ECAT_McGroupMoveHelicalRel(DeviceNo, GroupNo, CircAngle, CircAuxPos, HelicalPitch, GroupVel)
		return ret
    
	def McGroupMove3DHelicalRel_CP_Angle(DeviceNo, GroupNo, GroupVel, CircAngle, HelicalPitch, CircAuxPos, NV):
		"""Start a relative 3D helical interpolation motion."""
		ret = dll.ECAT_McGroupMove3DHelicalRel_CP_Angle(DeviceNo, GroupNo, GroupVel, CircAngle, HelicalPitch, CircAuxPos, NV)
		return ret 
    
	def McGroupMoveConicalHelixAbs(DeviceNo, GroupNo, CircAngle, CircAuxPos, HelicalPitch, GroupVel, EndRadius):
		"""Start an absolute conical helix interpolation motion."""
		ret = dll.ECAT_McGroupMoveConicalHelixAbs(DeviceNo, GroupNo, CircAngle, CircAuxPos, HelicalPitch, GroupVel, EndRadius)
		return ret
    
	def McGroupMove3DConicalHelixAbs_CP_Angle(DeviceNo, GroupNo, GroupVel, CircAngle, HelicalPitch, CircAuxPos, NV, EndRadius):
		"""Start an absolute 3D conical helix interpolation motion."""
		ret = dll.ECAT_McGroupMove3DConicalHelixAbs_CP_Angle(DeviceNo, GroupNo, GroupVel, CircAngle, HelicalPitch, CircAuxPos, NV, EndRadius)
		return ret
    
	def McGroupMoveConicalHelixRel(DeviceNo, GroupNo, CircAngle, CircAuxPos, HelicalPitch, GroupVel, EndRadius):
		"""Start a relative conical helix interpolation motion."""
		ret = dll.ECAT_McGroupMoveConicalHelixRel(DeviceNo, GroupNo, CircAngle, CircAuxPos, HelicalPitch, GroupVel, EndRadius)
		return ret
    
	def McGroupMove3DConicalHelixRel_CP_Angle(DeviceNo, GroupNo, GroupVel, CircAngle, HelicalPitch, CircAuxPos, NV, EndRadius):
		"""Start a relative 3D conical helix interpolation motion."""
		ret = dll.ECAT_McGroupMove3DConicalHelixRel_CP_Angle(DeviceNo, GroupNo, GroupVel, CircAngle, HelicalPitch, CircAuxPos, NV, EndRadius)
		return ret    
    
	def McGroupMoveProfile(DeviceNo, GroupNo, ProfileNo, TotalStep):
		"""Start a profile position motion."""
		ret = dll.ECAT_McGroupMoveProfile(DeviceNo, GroupNo, ProfileNo, TotalStep)
		return ret  
    
	def McGroupMoveProfileCSV(DeviceNo, GroupNo, ProfileNo):
		"""Start a profile position motion. The profile data are read from a CSV file."""
		ret = dll.ECAT_McGroupMoveProfileCSV(DeviceNo, GroupNo, ProfileNo)
		return ret
    
	def McGroupMoveDwell(DeviceNo, GroupNo, DwellTime):
		"""The motion kernel will make a group to wait for the dwell time; after time is up, continue 
		to  load  and  execute  the  next  command.  This  command  can  be  used  for  adjusting  the 
		blending  distance  between  two  motion  commands  in  continuous  blending  motion.  This 
		command behaves just like any other motion commands and is sequentially executed. In 
		Buffered or Blending mode, if a motion command is being executed, it will be pushed into the 
		command buffer. In Aborting mode, motion kernel will stop executing the current command 
		by deceleration and start to wait for the dwell time."""
		ret = dll.ECAT_McGroupMoveDwell(DeviceNo, GroupNo, DwellTime)
		return ret    

	def McGroupMoveDO(DeviceNo, GroupNo, SlaveNo, BitNo, Value):
		"""Add a slave DO output command in the group motion. This command will not execute 
		immediately. It will be put into command buffer and wait for execution."""
		ret = dll.ECAT_McGroupMoveDO(DeviceNo, GroupNo, SlaveNo, BitNo, Value)
		return ret     
    
	def McGroupMoveBlendingSync(DeviceNo, GroupNo):
		"""When the group is in the blending mode, this command will make the motion kernel to 
		wait until the current command is finished before executing the next motion command. After 
		that, the group is still in blending mode and a new current motion command will blend with its 
		next motion command."""
		ret = dll.ECAT_McGroupMoveBlendingSync(DeviceNo, GroupNo)
		return ret    
    
	def SetDeviceEncProperty(DeviceNo, EncNo, Mode, InvertCnt, LPF):
		"""Set the on-board encoder mode of a device."""
		ret = dll.ECAT_SetDeviceEncProperty(DeviceNo, EncNo, Mode, InvertCnt, LPF)
		return ret
    
	def GetDeviceEncProperty(DeviceNo, EncNo):
		"""Get the on-board encoder mode of a device."""
		Mode = c_ubyte(0)
		InvertCnt = c_ubyte(0)
		LPF = c_ubyte(0)        
		ret = dll.ECAT_GetDeviceEncProperty(DeviceNo, EncNo, byref(Mode), byref(InvertCnt), byref(LPF))
		return ret, Mode, InvertCnt, LPF    

	def GetDeviceEncCount(DeviceNo, EncNo):
		"""Get an on board encoder counter value of a device."""
		Cnt = c_int(0)        
		ret = dll.ECAT_GetDeviceEncCount(DeviceNo, EncNo, byref(Cnt))
		return ret, Cnt    

	def ResetDeviceEncCount(DeviceNo, EncNo):
		"""Clear an on-board encoder counter value of a device."""
		ret = dll.ECAT_ResetDeviceEncCount(DeviceNo, EncNo)
		return ret
    
	def SetDeviceCmpTrigProperty(DeviceNo, EncNo, PulseWidth, Source):
		"""Set the on-board device compare-trigger related properties."""
		ret = dll.ECAT_SetDeviceCmpTrigProperty(DeviceNo, EncNo, PulseWidth, Source)
		return ret    
    
	def GetDeviceCmpTrigProperty(DeviceNo, EncNo):
		"""Get the on-board device compare-trigger related properties."""
		PulseWidth = c_uint(0)   
		Source = c_ubyte(0)           
		ret = dll.ECAT_GetDeviceCmpTrigProperty(DeviceNo, EncNo, byref(PulseWidth), byref(Source))
		return ret, PulseWidth, Source
    
	def SetDeviceCmpTrigData(DeviceNo, EncNo, CmpData):
		"""According to the setting value, start a single compare-trigger function for an on-board encoder interface channel."""
		ret = dll.ECAT_SetDeviceCmpTrigData(DeviceNo, EncNo, CmpData)
		return ret
    
	def SetDeviceContCmpTrigData(DeviceNo, EncNo, Start, Interval, Times, Dir):
		"""Start a continuous or a multiple compare-trigger function."""
		ret = dll.ECAT_SetDeviceContCmpTrigData(DeviceNo, EncNo, Start, Interval, Times, Dir)
		return ret
        
	def SetDeviceEmg(DeviceNo, Source, Enable, Logic, SlaveNo, BitNo, ServoOff):
		"""Set the device emergency stop signal related configurations."""
		ret = dll.ECAT_SetDeviceEmg(DeviceNo, Source, Enable, Logic, SlaveNo, BitNo, ServoOff)
		return ret
    
	def GetDeviceEmg(DeviceNo):
		"""Get the configurations of the device emergency stop signal."""
		Source = c_ubyte(0)   
		Enable = c_ubyte(0) 
		Logic = c_ubyte(0)   
		SlaveNo = c_ushort(0) 
		BitNo = c_ushort(0)   
		ServoOff = c_ubyte(0)         
		ret = dll.ECAT_GetDeviceEmg(DeviceNo, byref(Source), byref(Enable), byref(Logic), byref(SlaveNo), byref(BitNo), byref(ServoOff))
		return ret, Source, Enable, Logic, SlaveNo, BitNo, ServoOff

	def GetDeviceEmgStatus(DeviceNo):
		"""Get emergency stop signal status."""
		Status = c_ubyte(0)
		ret = dll.ECAT_GetDeviceEmgStatus(DeviceNo, byref(Status))
		return ret, Status    

	def SetDeviceEmgSoftSig(DeviceNo):
		"""Use this function to produce an emergency stop."""
		ret = dll.ECAT_SetDeviceEmgSoftSig(DeviceNo)
		return ret
    
	def SetDeviceMPG(DeviceNo, Enable, AxisNo, AxisCount):
		"""Configure device local I/O into a manual pulse generator."""
		ret = dll.ECAT_SetDeviceMPG(DeviceNo, Enable, AxisNo, AxisCount)
		return ret
    
	def GetDeviceMPG(DeviceNo):
		"""Get the manual pulse generator (MPG) configuration of this device (card)."""
		Enable = c_ubyte(0)   
		AxisNo = c_ushort(0) 
		AxisCount = c_ushort(0) 
		ret = dll.ECAT_GetDeviceMPG(DeviceNo, byref(Enable), byref(AxisNo), byref(AxisCount))
		return ret, Enable, AxisNo, AxisCount 

	def OpenMotionConfig(FileName):
		"""Read  a  file  which  is  created  by  Axis  configuration  in  the  utility  program  and  save  to 
		variables provided in the arguments. These settings can be further transferred into a control 
		card by calling several different functions. Since this function uses the COM technique of 
		Microsoft to process data, it is not supported in a Linux OS system."""
		AxisCnt = c_ushort(0)
		SlaveNo = (c_ushort * MC_AXIS_NO_MAX)(*[0 for x in range(MC_AXIS_NO_MAX)])         
		SubAxisNo = (c_ushort * MC_AXIS_NO_MAX)(*[0 for x in range(MC_AXIS_NO_MAX)])         
		PPU = (c_double * MC_AXIS_NO_MAX)(*[0 for x in range(MC_AXIS_NO_MAX)])
		HomeMethod = (c_int * MC_AXIS_NO_MAX)(*[0 for x in range(MC_AXIS_NO_MAX)])        
		HomeSpeedSeachSw = (c_double * MC_AXIS_NO_MAX)(*[0 for x in range(MC_AXIS_NO_MAX)])
		HomeSpeedSeachZr = (c_double * MC_AXIS_NO_MAX)(*[0 for x in range(MC_AXIS_NO_MAX)])
		HomeAcc = (c_double * MC_AXIS_NO_MAX)(*[0 for x in range(MC_AXIS_NO_MAX)])
		EncoderPPR = (c_uint * MC_AXIS_NO_MAX)(*[0 for x in range(MC_AXIS_NO_MAX)])        
		MotorPPR = (c_uint * MC_AXIS_NO_MAX)(*[0 for x in range(MC_AXIS_NO_MAX)])        
		ret = dll.ECAT_OpenMotionConfig(FileName, byref(AxisCnt) , SlaveNo, SubAxisNo, PPU, HomeMethod , HomeSpeedSeachSw, HomeSpeedSeachZr, HomeAcc , EncoderPPR, MotorPPR)
		return ret, AxisCnt, SlaveNo, SubAxisNo, PPU, HomeMethod , HomeSpeedSeachSw, HomeSpeedSeachZr, HomeAcc , EncoderPPR, MotorPPR 
    
	def McGetMotionRecordState(DeviceNo):
		"""Get the recording status."""
		state = c_ushort(0) 
		cnt = c_uint(0) 
		ret = dll.ECAT_McGetMotionRecordState(DeviceNo, byref(state), byref(cnt))
		return ret, state, cnt     

	def McSetMotionRecord(DeviceNo, state):
		"""This function can start or stop an ECAT-M801/ e-M901 to record the position and/or 
		velocity of axes. Inside the ECAT-M801/ e-M901, the program can save a record for each 
		cycle time. Up to 100,000 records can be saved. 
		Note: This function will not clear record count to 0. Users can clear record count with function ECAT_McClearMotionRecord."""
		ret = dll.ECAT_McSetMotionRecord(DeviceNo, state)
		return ret
    
	def SetTimer(DeviceNo, Interval):
		"""Set Timer Interval. This timer is only available when system is in OP state. In OP state,   
		there is a cyclic task running inside ECAT_M801/ e-M901. This card takes advantage of this 
		cyclic task to provide the host PC a timer function. ECAT_SetTimer function configures its 
		time interval. A companion function ECAT_WaitforTimer is the one to wait for this interval. A 
		thread calls ECAT_WaitforTimer will be suspended until time up."""
		ret = dll.ECAT_SetTimer(DeviceNo, Interval) 
		return ret    
    
	def SetTimerStop(DeviceNo):
		"""Disable  Timer.  This  timer  is  only  available when  system  is  in  OP  state.  In OP  state,   
		there is a cyclic task running inside ECAT_M801/ e-M901. This card takes advantage of this 
		cyclic task to provide the host PC a timer function. ECAT_SetTimer function configures its 
		time interval. A companion function ECAT_WaitforTimer is the one to wait for this interval. A 
		thread calls ECAT_WaitforTimer will be suspended until time up."""
		ret = dll.ECAT_SetTimerStop(DeviceNo) 
		return ret
    
	def McSetStewartPlatform_M1(DeviceNo, radiusB, angleB, radiusP, angleP, RodLength, Max_RodLength):
		"""Set geometric parameters for a Stewart platform (method 1)."""
		ret = dll.ECAT_McSetStewartPlatform_M1(DeviceNo, radiusB, angleB, radiusP, angleP, RodLength, Max_RodLength)
		return ret

	def McGetAxisActualPosVel(DeviceNo, AxisNo):
		"""Get actual position and velocity of an axis. 
		Note: When AxisNo is set to 65535, both the actual positions and velocities of all axes 
		are read back and saved into Pos and Vel array pointers, respectively."""
		AxisPos = c_float(0)
		AxisVel = c_float(0)        
		ret = dll.ECAT_McGetAxisActualPosVel(DeviceNo, AxisNo, byref(AxisPos), byref(AxisVel))
		return ret, AxisPos, AxisVel
    
	def PidSetSetPointValue(DeviceNo, PidNo, SetPointValue):
		"""Set the Set Point Value."""
		ret = dll.ECAT_PidSetSetPointValue(DeviceNo, PidNo, SetPointValue)
		return ret

	def PidGetControlOutputValue(DeviceNo, PidNo):
		"""Get Control Output Value in a PID control loop."""
		Output = c_double(0)
		ret = dll.ECAT_PidGetControlOutputValue(DeviceNo, PidNo, byref(Output))
		return ret, Output    

	def PidSetParameter(DeviceNo, PidNo, kp, ki, kd):
		"""Set the control parameters of a PID Controller."""
		ret = dll.ECAT_PidSetParameter(DeviceNo, PidNo, kp, ki, kd)
		return ret    
    
	def PidSetStatus(DeviceNo, PidNo, enable):
		"""Set PID Controller Status.   
		Note: Changing the status from Enabled to Disabled will not clear the output of the 
		control output module. Users can set control output to whatever they like by using function 
		ECAT_SetSlaveRxPdoData if PID Controller Status is disabled. However, if the status is 
		changed from Disabled to Enabled, it will set the output of the control output module to 0; 
		then the controller start to work."""
		ret = dll.ECAT_PidSetStatus(DeviceNo, PidNo, enable)
		return ret
  
	def PidSetSimulateMode(DeviceNo, PidNo, Simulate):
		"""Set simulation status. Use it to set whether the system is set for simulation or not. 
		Note: Changing the status from Disable to Enable simulation will clear the output of the 
		control output module which is used for this PID controller. Users can set control output 
		value by using function ECAT_SetSlaveRxPdoData if simulation is disabled."""
		ret = dll.ECAT_PidSetSimulateMode(DeviceNo, PidNo, Simulate)
		return ret
    
	def PidSetSampleTime(DeviceNo, PidNo, Interval):
		"""Set the sampling time."""
		ret = dll.ECAT_PidSetSampleTime(DeviceNo, PidNo, Interval)
		return ret
    
	def PidSetProcessVariableModule(DeviceNo, PidNo, SlaveNo_input, Offset_input, Bitsize_input, Scalegain_input, Scaleoffset_input):
		"""A Process Variable in a PID control loop is measured by an analog input channel in a 
		module.  In  order  to  convert  an  analog  input  value  to  a  physical  value, users  need  to  set 
		scaling  parameters.  SlaveNo,  OffsetByte  and  Bitlength  paramters  are  used  for  the 
		assignment  of  the  module  and  its  analog  input  channel.    ScaleGain  and  ScaleOffset 
		paramters are used for data conversion. This function can set these settings."""
		ret = dll.ECAT_PidSetProcessVariableModule(DeviceNo, PidNo, SlaveNo_input, Offset_input, Bitsize_input, 
Scalegain_input, Scaleoffset_input)
		return ret    

	def PidSetControlOutputModule(DeviceNo, PidNo, SlaveNo_output, Offset_output, Bitsize_output, 
Scalegain_output, Scaleoffset_output, Max_Value, Min_Value):
		"""A Control Output in a PID control loop is sent to an analog output channel in an AO 
		module. In order to convert a physical value to an analog output value, users need to set 
		scaling parameters. SlaveNo, OffsetByte and Bitlength parameters are used for assignment 
		of the module and its analog output channel.    ScaleGain and ScaleOffset parameters are 
		used for data conversion. Output_Max_Value and Output_Min_Value parameters are used 
		to limit the control output value. This function can set these settings."""
		ret = dll.ECAT_PidSetControlOutputModule(DeviceNo, PidNo, SlaveNo_output, Offset_output, Bitsize_output, 
Scalegain_output, Scaleoffset_output, Max_Value, Min_Value)
		return ret
    
	def PidGetParameter(DeviceNo, PidNo):
		"""Get the control parameters of a PID Controller."""
		kp = c_double(0)
		ki = c_double(0)
		kd = c_double(0)        
		ret = dll.ECAT_PidGetParameter(DeviceNo, PidNo, byref(kp), byref(ki), byref(kd))
		return ret, kp, ki, kd    
    
	def PidGetStatus(DeviceNo, PidNo):
		"""Get the controller status. It can be enabled or disabled."""
		Status = c_ubyte(0)
		ret = dll.ECAT_PidGetStatus(DeviceNo, PidNo, byref(Status))
		return ret, Status    
    
	def PidGetSimulateMode(DeviceNo, PidNo):
		"""Get simulation status. Use it to know whether the system is set for simulation or not."""
		Status = c_ubyte(0)
		ret = dll.ECAT_PidGetSimulateMode(DeviceNo, PidNo, byref(Status))
		return ret, Status

	def PidGetSampleTime(DeviceNo, PidNo):
		"""Get the sampling time."""
		Interval = c_uint(0)
		ret = dll.ECAT_PidGetSampleTime(DeviceNo, PidNo, byref(Interval))
		return ret, Interval
    
	def PidGetProcessVariableModule(DeviceNo, PidNo):
		"""A Process Variable in a PID control loop is measured by an analog input channel in a 
		module.  In  order  to  convert  an  analog  input  value  to  a  physical  value, users  need  to  set 
		scaling  parameters.  SlaveNo,  OffsetByte  and  Bitlength  parameters  are  used  for  the 
		assignment  of  the  module  and  its  analog  input  channel.    ScaleGain  and  ScaleOffset 
		paramters  are  used  for  data  conversion.  This  function  can  get  the  settings  of  these parameters."""
		SlaveNo = c_ushort(0)
		Offset = c_ushort(0)        
		Bitsize = c_ushort(0)
		Scalegain = c_double(0)
		Scaleoffset = c_double(0)        
		ret = dll.ECAT_PidGetProcessVariableModule(DeviceNo, PidNo, byref(SlaveNo), byref(Offset), byref(Bitsize), byref(Scalegain), byref(Scaleoffset))
		return ret, SlaveNo, Offset, Bitsize, Scalegain, Scaleoffset    
    
	def PidGetControlOutputModule(DeviceNo, PidNo):
		"""A Control Output in a PID control loop is sent to an analog output channel in an AO 
		module. In order to convert a physical value to an analog output value, users need to set 
		scaling parameters. SlaveNo, OffsetByte and Bitlength parameters are used for assignment 
		of the module and its analog output channel.    ScaleGain and ScaleOffset parameters are 
		used for data conversion. Output_Max_Value and Output_Min_Value parameters are used 
		to limit the control output value. This function can get these settings."""
		SlaveNo = c_ushort(0)
		Offset = c_ushort(0)        
		Bitsize = c_ushort(0)
		Scalegain = c_double(0)
		Scaleoffset = c_double(0) 
		Max_Value = c_double(0)
		Min_Value = c_double(0)         
		ret = dll.ECAT_PidGetControlOutputModule(DeviceNo, PidNo, byref(SlaveNo), byref(Offset), byref(Bitsize), byref(Scalegain), byref(Scaleoffset), byref(Max_Value), byref(Min_Value))
		return ret, SlaveNo, Offset, Bitsize, Scalegain, Scaleoffset, Max_Value, Min_Value
    
	def PidGetProcessVariable(DeviceNo, PidNo):
		"""Get the Process Variable."""
		SetPointValue = c_double(0)
		ret = dll.ECAT_PidGetProcessVariable(DeviceNo, PidNo, byref(SetPointValue))
		return ret, SetPointValue    
    
	def PidGetSimulateFeedback(DeviceNo, PidNo):
		"""If  the  simulation  is  enabled  for  a  PID  control  loop,  this  function  can  get  the  Control 
		Output Value of this loop."""
		Feedback = c_double(0)
		ret = dll.ECAT_PidGetSimulateFeedback(DeviceNo, PidNo, byref(Feedback))
		return ret, Feedback
    
	def WaitforTimer(DeviceNo):
		"""Wait until time up. This timer is only available when system is in OP state. In OP state,   
		there is a cyclic task running inside ECAT_M801/ e-M901. This card takes advantage of this 
		cyclic task to provide the host PC a timer function. ECAT_SetTimer function configures its 
		time interval. A companion function ECAT_WaitforTimer is the one to wait for this interval. 
		A thread calls ECAT_WaitforTimer will be suspended until time up."""
		ret = dll.ECAT_WaitforTimer(DeviceNo)
		return ret
    
	def PidGetSetPointValue(DeviceNo, PidNo):
		"""Get the Set Point Value."""
		SetPointValue = c_double(0)
		ret = dll.ECAT_PidGetSetPointValue(DeviceNo, PidNo, byref(SetPointValue))
		return ret, SetPointValue
    
	def PidGet_Sp_Err_Op_Pv(DeviceNo, PidNo):
		"""Get the Set Point Value, Error, Control Output, and Process Variable of a PID control 
system. Users can use this function to get these values back efficiently."""
		SetPointValue = c_double(0)
		Error = c_double(0)
		OutputValue = c_double(0)
		ProcessVariable = c_double(0)        
		ret = dll.ECAT_PidGet_Sp_Err_Op_Pv(DeviceNo, PidNo, byref(SetPointValue), byref(Error), byref(OutputValue), byref(ProcessVariable))
		return ret, SetPointValue, Error, OutputValue, ProcessVariable
    
	def McSetAxisVelocityFeedForwardGain(DeviceNo, AxisNo, Gain):
		"""Set Velocity Feed Forward Gain of an axis. Note: Only for some CiA402 servo drives. In 
		general, the feed forward velocity can help improving the performance of position tracking 
		control. This function defines the gain of the feed forward velocity for position control."""
		ret = dll.ECAT_McSetAxisVelocityFeedForwardGain(DeviceNo, AxisNo, Gain)
		return ret    
    
	def McGetAxisVelocityFeedForwardGain(DeviceNo, AxisNo):
		"""Get  Velocity  Feed  Forward  Gain  of  an  axis.  Note:  Only  for  CiA402  servo  drives.  In 
		general, the feed forward velocity can help improving the performance of position tracking 
		control. This function defines the gain of the feed forward velocity for position control."""
		Gain = c_double(0)
		ret = dll.ECAT_McGetAxisVelocityFeedForwardGain(DeviceNo, AxisNo, byref(Gain))
		return ret, Gain
    
	def McStwartPlatformMoveAbs_PT(DeviceNo, GroupNo, StewartPlatformPose, Pos, GroupTime):
		"""Start  an  absolute  linear  interpolation  motion  by  providing  world  coordinate  space 
		positions and time for executing this motion command. This is a group motion command. 
		The  pose  includes  the  6-axis  world  coordinate  space  positions.  A  long-distance  linear 
		motion or circular motion can be approximated by many of these short-distance commands. 
		ECAT-M801/  e-M901  has  a  3000-depth  command  buffer.  Users  can  send  commands 
		continuously to this card. If the command mode is Blending, this card will smoothly execute every desired motion command.       
		Note:  At  first,  this  card  will  process  pose  command  obtain  the  targeted  joint  space 
		positions by processing the inverse kinematics. Then a 6-axis linear interpolation motion in 
		joint  space  is  implemented  for  this  motion.  Actually,  the  linear  interpolation  is  not 
		implemented in the world coordinate system. Only continuous short-distance commands can approach nearly linear commands."""
		ret = dll.ECAT_McStwartPlatformMoveAbs_PT(DeviceNo, GroupNo, StewartPlatformPose, Pos, GroupTime)
		return ret    
    
	def McGetStewartPlatform_M1(DeviceNo):
		"""Get geometric parameters of a Stewart platform (method 1)."""
		radiusB = c_double(0)
		angleB = c_double(0)
		radiusP = c_double(0)
		angleP = c_double(0) 
		RodLength = c_double(0)
		Max_RodLength = c_double(0)       
		ret = dll.ECAT_McGetStewartPlatform_M1(DeviceNo, byref(radiusB), byref(angleB), byref(radiusP), byref(angleP), byref(RodLength), byref(Max_RodLength))
		return ret, radiusB, angleB, radiusP, angleP, RodLength, Max_RodLength    
    
	def McSetStewartPlatform_M2(DeviceNo, Bx, By, Px, Py, Z0, RodLength, Max_RodLength):
		"""Set geometric parameters of a Stewart platform (method 2)."""
		ret = dll.ECAT_McSetStewartPlatform_M2(DeviceNo, Bx, By, Px, Py, Z0, RodLength, Max_RodLength)
		return ret
    
	def McSetGroupVelLimitStatus(DeviceNo, GroupNo, State):
		"""Enable or disable the checking of the velocity limit of a group. If state is "Enable", each 
		axis speed in this group will be checked for not over a defined maximum value. If one of 
		these axes is over the speed limit value, this group speed will be recalculated to meet the speed limit requirement.   
		Note: This velocity limit requirement right now is valid only for two functions: 
		ECAT_McGroupMoveLineAbs_PT and ECAT_McGroupMoveLineRel_PT."""
		ret = dll.ECAT_McSetGroupVelLimitStatus(DeviceNo, GroupNo, State)
		return ret    
    
	def McGetGroupVelLimitStatus(DeviceNo, GroupNo):
		"""Get the setting of enabling or disabling the velocity limit of a group. 
		Note: This velocity limit requirement right now is valid only for two functions: 
		ECAT_McGroupMoveLineAbs_PT and ECAT_McGroupMoveLineRel_PT."""
		State = c_ushort(0)
		ret = dll.ECAT_McGetGroupVelLimitStatus(DeviceNo, GroupNo, byref(State))
		return ret, State      
    
	def McSetGroupVelLimitValue(DeviceNo, GroupNo, Value):
		"""Set the velocity limit value of each axis in a group. 
		Note: This velocity limit requirement right now is valid only for two functions: 
		ECAT_McGroupMoveLineAbs_PT and ECAT_McGroupMoveLineRel_PT."""
		ret = dll.ECAT_McSetGroupVelLimitValue(DeviceNo, GroupNo, Value)
		return ret    
   
	def McGetGroupVelLimitValue(DeviceNo, GroupNo):
		"""Get the velocity limit of each of axis in a group. 
		Note: This velocity limit requirement right now is valid only for two functions: 
		ECAT_McGroupMoveLineAbs_PT and ECAT_McGroupMoveLineRel_PT."""
		Value = c_double(0)
		ret = dll.ECAT_McGetGroupVelLimitValue(DeviceNo, GroupNo, byref(Value))
		return ret, Value

	def McSetMotionRecordParam(DeviceNo, Value1, Value2):
		"""Set parameters for deciding which two out of four values are going to be recorded.
		The candidated four values are Actual Position, Actual Velocity, Command Position, and Command Velocity."""
		ret = dll.ECAT_McSetMotionRecordParam(DeviceNo, Value1, Value2)
		return ret

	def McGetMotionRecordParam(DeviceNo):
		"""Get the settings of the recorded parameters."""
		Value1 = c_ushort(0)
		Value2 = c_ushort(0)        
		ret = dll.ECAT_McGetMotionRecordParam(DeviceNo, byref(Value1), byref(Value2))
		return ret, Value1, Value2
    
	def McGetMotionRecordValue(DeviceNo, CountNo, AxisNo):
		"""Get parameter values of an assigned axis at an assigned index number. 
		Note: When the AxisNo is set to 65535, values of all axes at the assigned index number 
		are returned by Value1and Value2 pointers."""
		Value1 = c_float(0)
		Value2 = c_float(0)  
		ret = dll.ECAT_McGetMotionRecordValue(DeviceNo, CountNo, AxisNo, byref(Value1), byref(Value2))
		return ret, Value1, Value2    
    
	def McClearMotionRecord(DeviceNo):
		"""This function can start or stop an ECAT-M801/ e-M901 to record the position and/or 
		velocity of axes. Inside the ECAT-M801/ e-M901, the program can save a record for each 
		cycle time. Up to 100,000 records can be saved. 
		Note: This function will not clear record count to 0. Users can clear record count with 
		function ECAT_McClearMotionRecord."""
		ret = dll.ECAT_McClearMotionRecord(DeviceNo)
		return ret 
    
	def McSetAxisPosSoftwareLimit(DeviceNo,AxisNo, Maximum, Minimum):
		"""Set position software limits to a specific axis.   
		Notice: Only for CiA402 and Virtual axis."""
		ret = dll.ECAT_McSetAxisPosSoftwareLimit(DeviceNo,AxisNo, Maximum, Minimum)
		return ret    
    
	def McGetAxisPosSoftwareLimit(DeviceNo,AxisNo):
		"""Get position software limits to an axis.   
		Notice: Only for CiA402 and Virtual axis."""
		Maximum = c_double(0)
		Minimum = c_double(0)  
		ret = dll.ECAT_McGetAxisPosSoftwareLimit(DeviceNo,AxisNo, byref(Maximum), byref(Minimum))
		return ret, Maximum, Minimum    

	def GetProcessTime(DeviceNo, Time):
		"""Get the processing time of an EtherCAT communication cycle. This is an average time 
		for successive 1000 cycles; the unit is in micro-second. 
		Warn:  the  processing  time  may  change  according  to  the  quantity  of  slaves  and  the 
		called APIs. It is better to keep this value under 50% of EtherCAT cycle time."""
		ret = dll.ECAT_GetProcessTime(DeviceNo, Time)
		return ret    
    
	def McSetGroupHold(DeviceNo, GroupNo, Status):
		"""The group state becomes MC_GS_HOLD and the motion kernel will stop loading new 
		commands after current command is done. After disable the holding, the motion kernel will 
		load a new command from command buffer and execute commands sequentially. 
		Notice: The PVT motion command and other group motion commands cannot be used together."""
		ret = dll.ECAT_McSetGroupHold(DeviceNo, GroupNo, Status)
		return ret     

	def McSetAxisPosSoftwareLimitStatus(DeviceNo,AxisNo, Status, ErrorStop):
		"""Set position software limit status to be enabled or not for an axis.   
		Notice: Only for CiA402 and Virtual axis."""
		ret = dll.ECAT_McSetAxisPosSoftwareLimitStatus(DeviceNo,AxisNo, Status, ErrorStop)
		return ret    
    
	def McGetAxisPosSoftwareLimitStatus(DeviceNo,AxisNo):
		"""Get position software limit status to a specific axis.   
		Notice: Only for CiA402 and Virtual axis."""
		Status = c_ushort(0)
		ErrorStop = c_ushort(0)         
		ret = dll.ECAT_McGetAxisPosSoftwareLimitStatus(DeviceNo,AxisNo, byref(Status), byref(ErrorStop))
		return ret, Status, ErrorStop    
    
	def McGetAxisHomeState(DeviceNo, AxisNo):
		"""Get Home state of an axis. Check if this axis has already executed home action successfully."""
		State = c_ushort(0) 
		ret = dll.ECAT_McGetAxisHomeState(DeviceNo, AxisNo, byref(State))
		return ret, State
    
	def McAxisGantryIn(DeviceNo, MasterNo, SlaveNo, Direction):
		"""Start a gantry control synchronization motion. Similar to an electrical gear control, the 
		slave axis will follow command position of master axis motion with gear ratio 1 or -1. Enable 
		the  feedforward  velocity  gain  will  make  the  servo  response  for  both  axes  faster;  thus, 
		improve the synchronization performance. 
		Note: (1) Use ECAT_McAxisStop to stop a gantry control synchronization motion 
		(2)  Use  ECAT_McSetAxisVelocityFeedForwardGain  to  adjust  the  feedforward 
		gain to get better synchronization performance. 
		(3) The Master axis number must be smaller than the slave axis number, otherwise 
		the position command of Slave axis will lag behind a CycleTime."""
		ret = dll.ECAT_McAxisGantryIn(DeviceNo, MasterNo, SlaveNo, Direction)
		return ret    
    
	def McSetGroupCmdModeEx(DeviceNo, GroupNo, CmdMode):
		"""This function is a little different from ECAT_McSetGroupCmdMode at the timing for 
		setting group command mode. It will be pushed into the command buffer first and wait for 
		executing if the current command mode in Buffered mode or Blending mode. However, in 
		Aborting mode it will change command mode immediately. A group command mode decides 
		how a motion command is processed by motion kernel. There are three command modes: 
		Aborting, Buffered, and Blending.   
		Aborting:  A  new  command  will  abort  the  current  executing  command;  then  the  new 
		command executes immediately. However, the motion kernel still provides a smooth velocity 
		transition for this mode."""
		ret = dll.ECAT_McSetGroupCmdModeEx(DeviceNo, GroupNo, CmdMode)
		return ret    
    
	def McAxisGantryMaxPosDiff(DeviceNo, SlaveNo, Value):
		"""Set the limitation of position deviation of the master axis and the slave axis. If the 
		position deviation is greater than the set value, it will trigger the error stop."""
		ret = dll.ECAT_McAxisGantryMaxPosDiff(DeviceNo, SlaveNo, Value)
		return ret    
 
	def McAxisGantryMaxPosDiffStatus(DeviceNo, SlaveNo, Status):
		"""Enable or disable the checking of maximum position deviation status for gantry control."""
		ret = dll.ECAT_McAxisGantryMaxPosDiffStatus(DeviceNo, SlaveNo, Status) 
		return ret
    
	def EvEnableEvent(DeviceNo, EventID):
		"""Enable an event. 
		Note: After an event is triggered, it will become disabled."""
		ret = dll.ECAT_EvEnableEvent(DeviceNo, EventID)
		return ret    
    
	def EvDisableEvent(DeviceNo, EventID):
		"""Disable an event."""
		ret = dll.ECAT_EvDisableEvent(DeviceNo, EventID)
		return ret
    
	def EvSetComparePositionParameters(DeviceNo, EventID, AxisNo, Operator, ComparePosition):
		"""Set event parameters for a position comparison event."""
		ret = dll.ECAT_EvSetComparePositionParameters(DeviceNo, EventID, AxisNo, Operator, ComparePosition)
		return ret
    
	def WaitforEvent(DeviceNo, INFINITE, Value):
		"""Program is blocked until the specified event is triggered or time out occurs."""
		ret = dll.ECAT_WaitforEvent(DeviceNo, INFINITE, Value)
		return ret    
    
	def EvSetCompareDIBitParameters(DeviceNo, EventID, SlaveNo, BitNo, CompareValue):
		"""Set event parameters for a DI-BIT comparison event."""
		ret = dll.ECAT_EvSetCompareDIBitParameters(DeviceNo, EventID, SlaveNo, BitNo, CompareValue)
		return ret    

	def EvSetMotionCompleteParameters(DeviceNo, EventID, AxisNo):
		"""Set event parameters as for checking an Axis motion done."""
		ret = dll.ECAT_EvSetMotionCompleteParameters(DeviceNo, EventID, AxisNo)
		return ret    

	def McGroupMoveAO(DeviceNo, GroupNo, SlaveNo, RunMode, ChannelNo, RawData, VoltData):
		"""Add a slave AO output command in the group motion. This command will not execute 
		immediately. It will be put into command buffer and wait for execution. 
		Note:    Please use ECAT_SetSlaveAoProperty to configure an AO slave before setting its value."""
		ret = dll.ECAT_McGroupMoveAO(DeviceNo, GroupNo, SlaveNo, RunMode, ChannelNo, RawData, VoltData)
		return ret    

	def McSetGroupAccTimeEx(DeviceNo, GroupNo, Time_ms):
		"""Set the acceleration time of a group. This command will be pushed into command buffer 
		and wait for execution if group command mode is in Buffered mode or Blending mode. The 
		motion kernel will wait the previous motion to be finished and then set the acceleration time."""
		ret = dll.ECAT_McSetGroupAccTimeEx(DeviceNo, GroupNo, Time_ms)
		return ret
    
	def McSetGroupBlendingPercent(DeviceNo, GroupNo, Value):
		"""Set the blending percent of a group. In the Blending mode, a "100" blending percent 
		means to blend the next motion command from the starting of deceleration of the pervious 
		motion  command.  A  "0"  blending  percent  means  no  blending  part;  and  the  behavior  is 
		similar to the Buffered command mode. Blending will introduce a smooth transition from one 
		command to another; however, it will produce corner error."""
		ret = dll.ECAT_McSetGroupBlendingPercent(DeviceNo, GroupNo, Value)
		return ret
    
	def McSetProfileInterval(DeviceNo, ProfileNo, Value):
		"""Set  interval  of  position  array  data  for  profile  motion.  It  controls  the  data  consuming 
		speed as well as the motion speed. For example, if the interval value is 2, the system will 
		consume each position value for every 2 cycles, i.e., the increment for each cycle is half of 
		the original defined value."""
		ret = dll.ECAT_McSetProfileInterval(DeviceNo, ProfileNo, Value)
		return ret

	def McAxisMoveSuperimposed(DeviceNo, AxisNo, AxisPos, AxisVel):
		"""Start a relative position motion of an axis additional to an existing motion. 
		Note: ECAT_McAxisChangePos and ECAT_McAxisChangeVel cannot be used during this command execution."""
		ret = dll.ECAT_McAxisMoveSuperimposed(DeviceNo, AxisNo, AxisPos, AxisVel)
		return ret    
    
	def McAxisHaltSuperimposed(DeviceNo, AxisNo):
		"""Stop a Superimposed motion of an axis with deceleration."""
		ret = dll.ECAT_McAxisHaltSuperimposed(DeviceNo, AxisNo)
		return ret    

	def McGetMotionRecordValueEx(DeviceNo, CountNo, Count, AxisNo):
		"""Get parameter values of an axis starting from an assigned index number. This function 
		is able to get more records than ECAT_McGetMotionRecordValue. This function can get up 
		to 64 records each time rather than only one record by ECAT_McGetMotionRecordValue."""
		Value1 = c_float(0)
		Value2 = c_float(0)
		ActualCount = c_ushort(0)        
		ret = dll.ECAT_McGetMotionRecordValueEx(DeviceNo, CountNo, Count, AxisNo, byref(Value1), byref(Value2), byref(ActualCount))
		return ret, Value1, Value2, ActualCount    
    
	def SetHeartBeat(DeviceNo, Value):
		"""Set heartbeat value. 
		After entering the OP and executing ECAT_McInit, If no command is executed for more 
		than heartbeat value, the software emergency stop signal will be triggered."""
		ret = dll.ECAT_SetHeartBeat(DeviceNo, Value)
		return ret    
    
	def SetHeartBeatStatus(DeviceNo, Value):
		"""Set heartbeat function to be enabled or not. 
		After entering the OP and executing ECAT_McInit, If no command is executed for more 
		than heartbeat value, the software emergency stop signal will be triggered."""
		ret = dll.ECAT_SetHeartBeatStatus(DeviceNo, Value)
		return ret  

	def McSetGroupBlendingPercentEx(DeviceNo, GroupNo, Value):
		"""Set the blending percent of a group. It is different from ECAT_McSetGroupBlendingPercent
		at the executing time. This command will be pushed 
		into command buffer first in Buffered mode or Blending mode and wait for execution."""
		ret = dll.ECAT_McSetGroupBlendingPercentEx(DeviceNo, GroupNo, Value) 
		return ret     
    
	def EvSetCompareDIParameters(DeviceNo, EventID, SlaveNo, OffsetByte, CompareValue, Mask):
		"""Set event parameters for multiple DI comparison event."""
		ret = dll.ECAT_EvSetCompareDIParameters(DeviceNo, EventID, SlaveNo, OffsetByte, CompareValue, Mask)
		return ret    
    
	def McSetGroupPause(DeviceNo, GroupNo, Status):
		"""The group state becomes MC_GS_PAUSE and the motion kernel will pause the current 
		group motion with deceleration immediately. The current command is just partially done, and 
		some remaining part is held. After the pause state becomes disabled, the motion kernel will 
		execute  the  remaining  part  of  the  unfinished  command  and  other  commands  in  the 
		command buffer sequentially."""
		ret = dll.ECAT_McSetGroupPause(DeviceNo, GroupNo, Status)
		return ret
    
	def EvSetCompareAxisStateParameters(DeviceNo, EventID, AxisNo, CompareState):
		"""Set event parameters as for checking an Axis state."""
		ret = dll.ECAT_EvSetCompareAxisStateParameters(DeviceNo, EventID, AxisNo, CompareState)
		return ret    
    
	def SetSlaveAiProperty(DeviceNo, SlaveNo, ChannelNo, Range):
		"""Set the AI channel property value. Each AI channel can has different range setting from others. 
		Note: It supports ECAT-2011H、ECAT-2012H."""
		ret = dll.ECAT_SetSlaveAiProperty(DeviceNo, SlaveNo, ChannelNo, Range)
		return ret    
    
	def GetSlaveAiProperty(DeviceNo, SlaveNo, ChannelNo):
		"""Get the AI channel property value. Each AI channel can has different range setting from others. 
		Note: It supports ECAT-2011H、ECAT-2012H."""
		Range = c_ushort(0)
		ret = dll.ECAT_GetSlaveAiProperty(DeviceNo, SlaveNo, ChannelNo, byref(Range))
		return ret, Range    

	def GetSlaveAiRawData(DeviceNo, SlaveNo, ChannelNo):
		"""Get the 16-bit integer value of an analog output channel. 
		Note: It supports ECAT-2011H、ECAT-2012H."""
		Data = c_int(0)
		ret = dll.ECAT_GetSlaveAiRawData(DeviceNo, SlaveNo, ChannelNo, byref(Data))
		return ret, Data
    
	def GetSlaveAiVoltData(DeviceNo, SlaveNo, ChannelNo):
		"""Get the floating-point voltage value of an analog input channel. Please make sure the channel property is set correctly. 
		Note: It supports ECAT-2011H、ECAT-2012H."""
		Data = c_double(0)
		ret = dll.ECAT_GetSlaveAiVoltData(DeviceNo, SlaveNo, ChannelNo, byref(Data))
		return ret, Data    
    
	def GetSlaveAimAData(DeviceNo, SlaveNo, ChannelNo):
		"""Get the floating-point milliampere value of an analog input channel. Please make sure the channel property is set correctly. 
		Note: It supports ECAT-2011H、ECAT-2012H."""
		Data = c_double(0)
		ret = dll.ECAT_GetSlaveAimAData(DeviceNo, SlaveNo, ChannelNo, byref(Data))
		return ret, Data    
    
	def McAxisHomeEx(DeviceNo, AxisNo, Settings):
		"""Start home motion of an axis. 
		Note: Since a few servo drives do not support the dynamic settings of some 
		home-related parameters, such as the home acceleration setting, an error may occur. 
		Another function ECAT_McAxisHomeEx is provided for dealing with this kind of drives."""
		ret = dll.ECAT_McAxisHomeEx(DeviceNo, AxisNo, Settings)
		return ret
    
	def EvSetCompareCmdPositionParameters(DeviceNo, EventID, AxisNo, Operator, ComparePosition):
		"""Set event parameters for a command position comparison event."""
		ret = dll.ECAT_EvSetCompareCmdPositionParameters(DeviceNo, EventID, AxisNo, Operator, ComparePosition)
		return ret     
    
	def EvSetCompareAxisVelStateParameters(DeviceNo, EventID, AxisNo, CompareState):
		"""Set event parameters for a command position comparison event."""
		ret = dll.ECAT_EvSetCompareAxisVelStateParameters(DeviceNo, EventID, AxisNo, CompareState)
		return ret
    
	def McSetAxisMaxVelocity(DeviceNo, AxisNo, MaxVelocity):
		"""Set maximum velocity of an axis for the following functions. 
		ECAT_McAxisMoveAbs_P2P 
		ECAT_McAxisMoveRel_P2P 
		ECAT_McGroupMoveLineAbs_P2P 
		ECAT_McGroupMoveLineRel_P2P."""
		ret = dll.ECAT_McSetAxisMaxVelocity(DeviceNo, AxisNo, MaxVelocity)
		return ret
    
	def McGetAxisMaxVelocity(DeviceNo, AxisNo):
		"""Get maximum velocity of an axis."""
		MaxVelocity = c_double(0)
		ret = dll.ECAT_McGetAxisMaxVelocity(DeviceNo, AxisNo, byref(MaxVelocity))
		return ret, MaxVelocity    
    
	def McGroupMoveLineAbs_P2P(DeviceNo, GroupNo, GroupPos):
		"""Start an absolute position motion of each axis in a group. 
		Note: Use Maximum velocity of each axis (ECAT_McSetAxisMaxVelocity)."""
		ret = dll.ECAT_McGroupMoveLineAbs_P2P(DeviceNo, GroupNo, GroupPos)
		return ret    
    
	def McGroupMoveLineRel_P2P(DeviceNo, GroupNo, GroupPos):
		"""Start a relative position motion of each axis in a group. 
		Note: Use Maximum velocity of each axis (ECAT_McSetAxisMaxVelocity)."""
		ret = dll.ECAT_McGroupMoveLineRel_P2P(DeviceNo, GroupNo, GroupPos)
		return ret    

	def McAxisMoveAbs_P2P(DeviceNo, AxisNo, AxisPos):
		"""Start a point-to-point absolute position motion of an axis.   
		Note: This motion will use the maximum velocity of the spcified axis, which is defined by ECAT_McSetAxisMaxVelocity."""
		ret = dll.ECAT_McAxisMoveAbs_P2P(DeviceNo, AxisNo, AxisPos)
		return ret

	def McAxisMoveRel_P2P(DeviceNo, AxisNo, AxisPos):
		"""Start a point-to-point relative position motion of an axis.   
		Note: This motion will use the maximum velocity of the spcified axis, which is defined by ECAT_McSetAxisMaxVelocity."""
		ret = dll.ECAT_McAxisMoveRel_P2P(DeviceNo, AxisNo, AxisPos)
		return ret    
    
	def Set_ECAT2016_AiProperty(DeviceNo, SlaveNo, ChannelNo, Range):
		"""Set the AI channel property value. Each AI channel can has different range setting from others. 
		Note: (1) It supports ECAT-2016N. (2) Change any one channel property value will change the property 
		values in the other remaining channels."""
		Abortcode = c_uint(0)        
		ret = dll.ECAT_Set_ECAT2016_AiProperty(DeviceNo, SlaveNo, ChannelNo, Range, byref(Abortcode))
		return ret, Abortcode
    
	def Get_ECAT2016_AiProperty(DeviceNo, SlaveNo, ChannelNo):
		"""Get the AI channel property value. Each AI channel can has different range setting from others. 
		Note: It supports ECAT-2016N."""
		Abortcode = c_uint(0)
		Range = c_ubyte(0)        
		ret = dll.ECAT_Get_ECAT2016_AiProperty(DeviceNo, SlaveNo, ChannelNo, byref(Range), byref(Abortcode))
		return ret, Range, Abortcode    
    
	def Get_ECAT2016_AiRawData(DeviceNo, SlaveNo, ChannelNo):
		"""Get the 16-bit integer value of an analog input channel. 
		Note: It supports ECAT-2016N."""
		Data = c_int(0)
		ret = dll.ECAT_Get_ECAT2016_AiRawData(DeviceNo, SlaveNo, ChannelNo, byref(Data))
		return ret, Data    
    
	def Get_ECAT2016_AiVoltData(DeviceNo, SlaveNo, ChannelNo):
		"""Get the floating-point voltage value of an analog input channel. Please make sure the channel property is set correctly. 
		Note: It supports ECAT-2016N."""
		Data = c_double(0)
		ret = dll.ECAT_Get_ECAT2016_AiVoltData(DeviceNo, SlaveNo, ChannelNo, byref(Data))
		return ret, Data    
    
	def EvSetCompareAiParameters(DeviceNo, EventID, Operator, SlaveNo, OffsetByte, DataSize, ScaleGain, ScaleOffset, CompareValue):
		"""Set event parameters for Ai comparison event. 
		Get TxPDO data of a slave by using OffsetByte and DataSize. 
		Compare Value to “CompareValue” after Scaling (Value = int(TxPDO data) * ScaleGain + ScaleOffset)."""
		ret = dll.ECAT_EvSetCompareAiParameters(DeviceNo, EventID, Operator, SlaveNo, OffsetByte, DataSize, ScaleGain, ScaleOffset, CompareValue)
		return ret     
    
	def McGroupMovePath(DeviceNo, GroupNo, PathDataNo, Restart, DataIndex, Repeat):
		"""Start Path Motion Control."""
		ret = dll.ECAT_McGroupMovePath(DeviceNo, GroupNo, PathDataNo, Restart, DataIndex, Repeat) 
		return ret
    
	def McAddPathData(DeviceNo, PathDataNo, CmdType, AbsMove, EndPos, AuxPos, Args):
		"""Add a Path data to the queue."""
		ret = dll.ECAT_McAddPathData(DeviceNo, PathDataNo, CmdType, AbsMove, EndPos, AuxPos, Args)
		return ret    
    
	def McSetPathData(DeviceNo, PathDataNo, DataIndex, CmdType, AbsMove, EndPos, AuxPos, Args):
		"""Modify the specified index Path data."""
		ret = dll.ECAT_McSetPathData(DeviceNo, PathDataNo, DataIndex, CmdType, AbsMove, EndPos, AuxPos, Args)
		return ret
    
	def McGetPathData(DeviceNo, PathDataNo, DataIndex):
		"""Get the specified index Path data."""
		CmdType = c_uint(0)
		AbsMove = c_ubyte(0)   
		EndPos = (c_double * MC_AXIS_NO_MAX)(*[0 for x in range(MC_AXIS_NO_MAX)])         
		AuxPos = (c_double * MC_PATH_DATA_ARGS_MAX)(*[0 for x in range(MC_PATH_DATA_ARGS_MAX)])
		Args = (c_double * MC_PATH_DATA_ARGS_MAX)(*[0 for x in range(MC_PATH_DATA_ARGS_MAX)])        
		ret = dll.ECAT_McGetPathData(DeviceNo, PathDataNo, DataIndex, byref(CmdType), byref(AbsMove), EndPos, AuxPos, Args)
		return ret, CmdType, AbsMove, EndPos, AuxPos, Args    
    
	def McClearPathData(DeviceNo, PathDataNo):
		"""Clear the path data in the queue."""
		ret = dll.ECAT_McClearPathData(DeviceNo, PathDataNo) 
		return ret    

	def McGetPathDataSize(DeviceNo, PathDataNo):
		"""Get the number of Path data in the queue."""
		Size = c_ushort(0)
		ret = dll.ECAT_McGetPathDataSize(DeviceNo, PathDataNo, byref(Size))
		return ret, Size    
    
	def McAxisMoveAbsAdv(DeviceNo, AxisNo, EndPos, StartVel, ReqVel, FinalVel, Accel, Decel, AccDecMode):
		"""Start an absolute position motion of an axis."""
		ret = dll.ECAT_McAxisMoveAbsAdv(DeviceNo, AxisNo, EndPos, StartVel, ReqVel, FinalVel, Accel, Decel, AccDecMode) 
		return ret    
    
	def McAxisMoveRelAdv(DeviceNo, AxisNo, EndPos, StartVel, ReqVel, FinalVel, Accel, Decel, AccDecMode):
		"""Start an relative position motion of an axis."""
		ret = dll.ECAT_McAxisMoveRelAdv(DeviceNo, AxisNo, EndPos, StartVel, ReqVel, FinalVel, Accel, Decel, AccDecMode) 
		return ret    
    
	def McGroupMoveLineAbsAdv(DeviceNo, GroupNo, EndPos, StartVel, ReqVel, FinalVel, Accel, Decel, AccDecMode): 
		"""Start an absolute linear interpolation motion of a group."""
		ret = dll.ECAT_McGroupMoveLineAbsAdv(DeviceNo, GroupNo, EndPos, StartVel, ReqVel, FinalVel, Accel, Decel, AccDecMode) 
		return ret
    
	def McGroupMoveLineRelAdv(DeviceNo, GroupNo, EndPos, StartVel, ReqVel, FinalVel, Accel, Decel, AccDecMode): 
		"""Start an relative linear interpolation motion of a group."""
		ret = dll.ECAT_McGroupMoveLineRelAdv(DeviceNo, GroupNo, EndPos, StartVel, ReqVel, FinalVel, Accel, Decel, AccDecMode) 
		return ret    
    
	def McAxisMove_CiA402_PP(DeviceNo, AxisNo, Abort, AbsMove, EndPos, Vel, Accel, Decel):
		"""Start the single-axis motion in CiA402 profile position mode. 
		Note: This function contains SDO commands."""
		ret = dll.ECAT_McAxisMove_CiA402_PP(DeviceNo, AxisNo, Abort, AbsMove, EndPos, Vel, Accel, Decel) 
		return ret    
    
	def McAxisMove_CiA402_PV(DeviceNo, AxisNo, Vel, Accel, Decel):
		"""Start the single-axis motion in CiA402 profile vecolity mode. 
		Note: This function contains SDO commands."""
		ret = dll.ECAT_McAxisMove_CiA402_PV(DeviceNo, AxisNo, Vel, Accel, Decel)
		return ret
    
	def McAxisMove_CiA402_PT(DeviceNo, AxisNo, Torque, Slope):
		"""Start the single-axis motion in CiA402 profile torque mode. 
		Note: This function contains SDO commands."""
		ret = dll.ECAT_McAxisMove_CiA402_PT(DeviceNo, AxisNo, Torque, Slope) 
		return ret    
    
	def SetTxPdoBufParam(DeviceNo, ChannelNo, SlaveNo, OffsetByte, DataSize, Enable):
		"""Set parameters of a TxPdo buffer. 
		TxPdo buffer will store the values of the last PDO_BUFFER_DATA_MAX specified TxPdo. 
		After reading the buffer, the data inside the buffer will be cleared."""
		ret = dll.ECAT_SetTxPdoBufParam(DeviceNo, ChannelNo, SlaveNo, OffsetByte, DataSize, Enable)
		return ret    

	def GetTxPdoBufParam(DeviceNo, ChannelNo):
		"""Read parameters of a TxPdo buffer."""
		SlaveNo = c_ushort(0)
		OffsetByte = c_ushort(0)
		DataSize = c_ushort(0)
		Enable = c_ushort(0)        
		ret = dll.ECAT_GetTxPdoBufParam(DeviceNo, ChannelNo, byref(SlaveNo), byref(OffsetByte), byref(DataSize), byref(Enable))
		return ret, SlaveNo, OffsetByte, DataSize, Enable    
    
	def SetTxPdoBufEnable(DeviceNo, ChannelNo, Enable):
		"""Set enable/disable of a TxPdo buffer."""
		ret = dll.ECAT_SetTxPdoBufEnable(DeviceNo, ChannelNo, Enable)
		return ret
    
	def GetTxPdoBufEnable(DeviceNo, ChannelNo):
		"""Get enable/disable of a TxPdo buffer."""
		Enable = c_ushort(0)
		ret = dll.ECAT_GetTxPdoBufEnable(DeviceNo, ChannelNo, byref(Enable))
		return ret, Enable    
    
	def GetTxPdoBufValue(DeviceNo, ChannelNo, Size):
		"""Get Data of a TxPdo buffer. 
		TxPdo buffer will store the values of the last PDO_BUFFER_DATA_MAX specified TxPdo. 
		After reading the buffer, the data inside the buffer will be cleared."""
		Data = (c_float * Size)(*[0 for x in range(Size)])  
		ActualGetSize = c_ushort(0)        
		ret = dll.ECAT_GetTxPdoBufValue(DeviceNo, ChannelNo, Data, Size, byref(ActualGetSize))
		return ret, Data, ActualGetSize
    
	def McSetAxisHomeTorque(DeviceNo, AxisNo, Torque):
		"""Set homing torque of an axis. 
		Note:(1)for homing mode 38、39 
		(2) supporting the CiA402 module (requires 6072h (Max torque), 6077h (Torque 
		actual value), need to support homing mode 37)."""
		ret = dll.ECAT_McSetAxisHomeTorque(DeviceNo, AxisNo, Torque)
		return ret
    
	def McGetAxisHomeTorque(DeviceNo, AxisNo):
		"""Set homing torque of an axis."""
		Torque = c_ushort(0)
		ret = dll.ECAT_McGetAxisHomeTorque(DeviceNo, AxisNo, byref(Torque))
		return ret, Torque    
    
	def SetAiFilterParam(DeviceNo, ChannelNo, SlaveNo, OffsetByte, DataSize, Enable):
		"""Set a TxPdo filter, supporting notch filter、high pass filter and low pass filter."""
		ret = dll.ECAT_SetAiFilterParam(DeviceNo, ChannelNo, SlaveNo, OffsetByte, DataSize, Enable)
		return ret
    
	def GetAiFilterParam(DeviceNo, ChannelNo):
		"""Get settings of a TxPdo filter."""
		SlaveNo = c_ushort(0)
		OffsetByte = c_ushort(0)
		DataSize = c_ushort(0)
		Enable = c_ushort(0) 
		ret = dll.ECAT_GetAiFilterParam(DeviceNo, ChannelNo, byref(SlaveNo), byref(OffsetByte), byref(DataSize), byref(Enable))
		return ret, SlaveNo, OffsetByte, DataSize, Enable    
    
	def SetAiFilterEnable(DeviceNo, ChannelNo, Enable):
		"""Set enable/disable of a TxPdo filter."""
		ret = dll.ECAT_SetAiFilterEnable(DeviceNo, ChannelNo, Enable)
		return ret    
    
	def GetAiFilterEnable(DeviceNo, ChannelNo):
		"""Get enable/disable of a TxPdo filter."""
		Enable = c_ushort(0)
		ret = dll.ECAT_GetAiFilterEnable(DeviceNo, ChannelNo, byref(Enable))
		return ret, Enable    
    
	def SetAiFilterFreq(DeviceNo, ChannelNo, FilterType, Frequency):
		"""Set frequency of a TxPdo filter."""
		ret = dll.ECAT_SetAiFilterFreq(DeviceNo, ChannelNo, FilterType, Frequency)
		return ret    
 
	def GetAiFilterFreq(DeviceNo, ChannelNo, FilterType):
		"""Get frequency of a TxPdo filter."""
		Frequency = c_double(0)
		ret = dll.ECAT_GetAiFilterFreq(DeviceNo, ChannelNo, FilterType, byref(Frequency))
		return ret, Frequency
    
	def GetAiFilterOutput(DeviceNo, ChannelNo):
		"""Get output of a TxPdo filter."""
		Output = c_int(0)
		ret = dll.ECAT_GetAiFilterOutput(DeviceNo, ChannelNo, byref(Output))
		return ret, Output    
    
	def SetPdoInToOutParam(DeviceNo, ChannelNo, SlaveNoIn, OffsetByteIn, DataSizeIn, SlaveNoOut, OffsetByteOut, DataSizeOut):
		"""Set a Pdo input Output. Write TxPDO data (Input) to RxPDO (Output) every EtherCAT cycle."""
		ret = dll.ECAT_SetPdoInToOutParam(DeviceNo, ChannelNo, SlaveNoIn, OffsetByteIn, DataSizeIn, SlaveNoOut, OffsetByteOut, DataSizeOut)
		return ret
    
	def GetPdoInToOutParam(DeviceNo, ChannelNo):
		"""Get settings of a Pdo input Output."""
		SlaveNoIn = c_ushort(0)
		OffsetByteIn = c_ushort(0)
		DataSizeIn = c_ushort(0)
		SlaveNoOut = c_ushort(0)
		OffsetByteOut = c_ushort(0)
		DataSizeOut = c_ushort(0)       
		ret = dll.ECAT_GetPdoInToOutParam(DeviceNo, ChannelNo, byref(SlaveNoIn), byref(OffsetByteIn), byref(DataSizeIn), byref(SlaveNoOut), byref(OffsetByteOut), byref(DataSizeOut))
		return ret, SlaveNoIn, OffsetByteIn, DataSizeIn, SlaveNoOut, OffsetByteOut, DataSizeOut
    
	def SetPdoInToOutCoeff(DeviceNo, ChannelNo, gain, offset):
		"""Set coefficient of a Pdo input Output. Output = input * gain + offset."""
		ret = dll.ECAT_SetPdoInToOutCoeff(DeviceNo, ChannelNo, gain, offset)
		return ret    
    
	def GetPdoInToOutCoeff(DeviceNo, ChannelNo):
		"""Get coefficient of a Pdo input Output."""
		gain = c_float(0)
		offset = c_float(0)        
		ret = dll.ECAT_GetPdoInToOutCoeff(DeviceNo, ChannelNo, byref(gain), byref(offset))
		return ret, gain, offset
    
	def SetPdoInToOutEnable(DeviceNo, ChannelNo, Enable):
		"""Set enable/disable of a Pdo input Output."""
		ret = dll.ECAT_SetPdoInToOutEnable(DeviceNo, ChannelNo, Enable)
		return ret
    
	def GetPdoInToOutEnable(DeviceNo, ChannelNo):
		"""Get enable/disable of a Pdo input Output."""
		Enable = c_ushort(0)
		ret = dll.ECAT_GetPdoInToOutEnable(DeviceNo, ChannelNo, byref(Enable))
		return ret, Enable
    
	def McAxisGantryGain(DeviceNo, SlaveNo, Kp, Ki):
		"""Please use McSetAxisVelocityFeedForwardGain to adjust the feedforward gain first. If 
		you are not satisfied with the result, use this function. If the parameter is set badly, it may 
		cause oscillation. Please set it carefully. You can use ECAT_McAxisGantryMaxPosDiff to 
		set the maximum position error or use emergency stop. When the position is 
		oscillating/diverge, you can stop it in time."""
		ret = dll.ECAT_McAxisGantryGain(DeviceNo, SlaveNo, Kp, Ki)
		return ret        
    
	def SetSlaveNoType(DeviceNo, Type):
		"""Define the slaveNo, When the slaveNo type is SLAVE_NO_TYPE_POSITION, the slaveNo is the position of 
		the module; When the slaveNo type is SLAVE_NO_TYPE_ALIAS, the slaveNo is the module alias."""
		ret = dll.ECAT_SetSlaveNoType(DeviceNo, Type)
		return ret
    
	def GetSlaveNoType(DeviceNo):
		"""Get define the slaveNo, When the slaveNo type is SLAVE_NO_TYPE_POSITION, the slaveNo is the position of 
		the module; When the slaveNo type is SLAVE_NO_TYPE_ALIAS, the slaveNo is the module alias."""
		Type = c_ushort(0)
		ret = dll.ECAT_GetSlaveNoType(DeviceNo, byref(Type))
		return ret, Type    
    
	def EvSetMotionCompleteParameters_Grp(DeviceNo, EventID, GroupNo):
		"""Set event parameters as for checking a Group motion done."""
		ret = dll.ECAT_EvSetMotionCompleteParameters_Grp(DeviceNo, EventID, GroupNo)
		return ret    

	def McSetGroupPvtDecEnable(DeviceNo, GroupNo, Enable):
		"""Set whether to decelerate or not after the PVT motion is finished of a group."""
		ret = dll.ECAT_McSetGroupPvtDecEnable(DeviceNo, GroupNo, Enable)
		return ret
        
	def McGetGroupPvtDecEnable(DeviceNo, GroupNo):
		"""Get whether to decelerate or not after the PVT motion is finished of a group."""
		Enable = c_ushort(0)
		ret = dll.ECAT_McGetGroupPvtDecEnable(DeviceNo, GroupNo, byref(Enable))
		return ret, Enable
    
	def AbortWaitforEvent(DeviceNo):
		"""Use this function to force ECAT_WaitforEvent to return 0 directly."""
		ret = dll.ECAT_AbortWaitforEvent(DeviceNo)
		return ret
    
	def McSetGroupCoordinate(DeviceNo, GroupNo, Type):
		"""Set whether the group motion control performs coordinate conversion."""
		ret = dll.ECAT_McSetGroupCoordinate(DeviceNo, GroupNo, Type)
		return ret
    
	def McSetGroupCoordinateLimit(DeviceNo, GroupNo, Type, Enable, Min, Max):
		"""Set the position software limit of a group."""
		ret = dll.ECAT_McSetGroupCoordinateLimit(DeviceNo, GroupNo, Type, Enable, Min, Max)
		return ret    
    
	def McGetGroupCoordinateLimit(DeviceNo, GroupNo, Type):
		"""Get the position software limit of a group."""
		Enable = c_ushort(0)
		Min = c_double(0)
		Max = c_double(0)        
		ret = dll.ECAT_McGetGroupCoordinateLimit(DeviceNo, GroupNo, Type, byref(Enable), byref(Min), byref(Max))
		return ret, Enable, Min, Max
    
	def SetSlaveArrCmpPos(DeviceNo, SlaveNo, EncNo, Index, Position):
		"""Set the array comparison position. After enabling the array comparison trigger function, 
		the comparison will start from the comparison position stored in index 0. When the encoder 
		reaches the comparison position, it will trigger the digital output. 
		Note: (1) This function is designed for encoder module ECAT-2092T. 
		(2) The first compare value has to be stored in index 0, the second in index 1, the third in index 2, etc..   
		(3) The  value  stored  in  index 0 as the  first  compare position and  it  remains  in  the 
		compare register until the encoder counter reaches this position before using the index 1 
		value as the next compare position."""
		ret = dll.ECAT_SetSlaveArrCmpPos(DeviceNo, SlaveNo, EncNo, Index, Position)
		return ret
    
	def GetSlaveArrCmpPos(DeviceNo, SlaveNo, EncNo, Index):
		"""Get the array comparison position. 
		Note: This function is designed for encoder module ECAT-2092T."""
		Position = c_int(0)
		ret = dll.ECAT_GetSlaveArrCmpPos(DeviceNo, SlaveNo, EncNo, Index, byref(Position))
		return ret, Position
    
	def SetSlaveArrCmpEnable(DeviceNo, SlaveNo, EncNo, Index, Enable):
		"""Set  “DO  trigger”  flag,  which  indicates  whether  a  digital  output  will  be  triggered  if  the 
		encoder value reaches this position value. 
		Note: This function is designed for encoder module ECAT-2092T."""
		ret = dll.ECAT_SetSlaveArrCmpEnable(DeviceNo, SlaveNo, EncNo, Index, Enable)
		return ret
    
	def GetSlaveArrCmpEnable(DeviceNo, SlaveNo, EncNo, Index):
		"""Get “DO trigger” flag. 
		Note: This function is designed for encoder module ECAT-2092T."""
		Enable = c_ubyte(0)
		ret = dll.ECAT_GetSlaveArrCmpEnable(DeviceNo, SlaveNo, EncNo, Index, byref(Enable))
		return ret, Enable    
  
	def SetSlaveArrCmpEndIdx(DeviceNo, SlaveNo, EncNo, End_Index):
		"""Set the end index to be used for the compare process. 
		When the encoder position has reached the compare position stored in end index then 
		the next compare position will start again with the position stored index 0. 
		Note: This function is designed for encoder module ECAT-2092T."""
		ret = dll.ECAT_SetSlaveArrCmpEndIdx(DeviceNo, SlaveNo, EncNo, End_Index)
		return ret    
        
	def GetSlaveArrCmpEndIdx(DeviceNo, SlaveNo, EncNo):
		"""Get the end index. 
		Note: This function is designed for encoder module ECAT-2092T."""
		End_Index = c_ushort(0)
		ret = dll.ECAT_GetSlaveArrCmpEndIdx(DeviceNo, SlaveNo, EncNo, byref(End_Index))
		return ret, End_Index    

	def SetSlaveArrCmpTrig(DeviceNo, SlaveNo, EncNo):
		"""Start Array compare-trigger function. 
		Note: This function is designed for encoder module ECAT-2092T."""
		ret = dll.ECAT_SetSlaveArrCmpTrig(DeviceNo, SlaveNo, EncNo)
		return ret    
    
	def SetSlaveSaveArrCmpData(DeviceNo, SlaveNo):
		"""Store the array comparison position and the “DO trigger” flag in the EEPROM of the 
		module, and the data will not be lost when the power is turned off. 
		Note: (1) This function is designed for encoder module ECAT-2092T. 
		(2) It needs to be set in the PreOP state. If the module is in the OP state, an error will be returned. 
		(3) This function is a blocking function. It returns about 10 seconds after execution. 
		If the return value is not zero, please re-power the module and set the data again."""
		ret = dll.ECAT_SetSlaveSaveArrCmpData(DeviceNo, SlaveNo)
		return ret
    
	def McSetAxisDIActiveLevel(DeviceNo, AxisNo, POT, NOT, ORG):
		"""Set limit active level and home active level of an axis. 
		For CiA402 driver, it is usually "active high", means that the di bit is set to high when the switch is logically active. 
		When the settings of drive is “active low ", user need to use this function to inform the 
		EtherCAT Master that the active level is "active low” . 
		Note:  Regardless  of  the  setting  of  "active  high  "  or  "active  high",  when  using 
		ECAT_McGetAxisDI to read the signal, 1 means that the signal is active."""
		ret = dll.ECAT_McSetAxisDIActiveLevel(DeviceNo, AxisNo, POT, NOT, ORG)
		return ret    
  
	def McGetAxisDIActiveLevel(DeviceNo, AxisNo):
		"""Get limit active level and home active level of an axis."""
		POT = c_ushort(0)
		NOT = c_double(0)
		ORG = c_double(0) 
		ret = dll.ECAT_McGetAxisDIActiveLevel(DeviceNo, AxisNo, byref(POT), byref(NOT), byref(ORG))
		return ret, POT, NOT, ORG
    
	def McGetAxisDI_60FD(DeviceNo, AxisNo):
		"""Get digital inputs(object 0x60FD) of an axis."""
		DI = c_uint(0)
		ret = dll.ECAT_McGetAxisDI_60FD(DeviceNo, AxisNo, byref(DI))
		return ret, DI        
    
	def SaveSlaveEEPROM(DeviceNo, SlaveNo):
		"""Save slave data to EEPROM.
		Note: This function is designed for ECAT-2091S/ ECAT-2094S."""        
		ret = dll.ECAT_SaveSlaveEEPROM(DeviceNo, SlaveNo);
		return ret    
    
	def McSetAxisActualPosition(DeviceNo, AxisNo, Position):
		"""Set actual position of an axis.
		Note: (1) The execution of this function will switch the state to MC_AS_HOMING. After 
		the function returns, it is necessary to check that the state is 
		MC_AS_STANDSTILL to be completed.
		(2) ECAT_McAxisHome already contains this command, no need to use this function."""
		ret = dll.ECAT_McSetAxisActualPosition(DeviceNo, AxisNo, Position)
		return ret
    
	def McSetAxisCommandPositio(DeviceNo, AxisNo, Position):
		"""Set command position of an axis. 
		Note: ECAT_McSetAxisActualPosition already contains this command, no need to use this function"""
		ret = dll.ECAT_McSetAxisCommandPosition(DeviceNo, AxisNo, Position)
		return ret
    
	def McSetAxisAccDecTime_Stepper(DeviceNo, AxisNo, AccTime_ms, DecTime_ms):
		"""Set acceleration time and deceleration time of an axis. 
		Note: This function is designed for ECAT-2091S/ ECAT-2094S."""
		ret = dll.ECAT_McSetAxisAccDecTime_Stepper(DeviceNo, AxisNo, AccTime_ms, DecTime_ms)
		return ret
    
	def McGetAxisAccDecTime_Stepper(DeviceNo, AxisNo):
		"""Get acceleration time and deceleration time of an axis. 
		Note: This function is designed for ECAT-2091S/ ECAT-2094S."""
		AccTime_ms = c_ushort(0)
		DecTime_ms = c_ushort(0)
		ret = dll.ECAT_McGetAxisAccDecTime_Stepper(DeviceNo, AxisNo, byref(AccTime_ms), byref(DecTime_ms))
		return ret, AccTime_ms, DecTime_ms
    
	def GetDeviceStateEx(DeviceNo):
		"""Get the EtherCAT network status of a device. This function is always called for checking if the system is running normally. 
		When AlStates is in ECAT_AS_OP, if the EtherCAT communication is abnormal, Wc will change. 
		At this time, AlStates may still remain at ECAT_AS_OP. The difference between this function and 
		ECAT_GetDeviceState is that when Wc is abnormal, bit4 of AlStates will be changed to 1."""
		LinkUp = c_uint(0)
		SlavesResp = c_uint(0)
		AlState = c_uint(0)
		Wc = c_uint(0)
		ret = dll.ECAT_GetDeviceStateEx(DeviceNo, byref(LinkUp), byref(SlavesResp), byref(AlState), byref(Wc))
		return ret, LinkUp, SlavesResp, AlState, Wc
	
	def SetSlaveEncIdxLatchBufferEnable(DeviceNo, SlaveNo, EncNo, Enable):
		"""Set the position index latch buffer(FIFO) function property to be enabled or not. This 
		function is designed for encoder module ECAT-2093 
		Note: The latch frequency is related to the EtherCAT communication cycle. For example: 
		if the communication cycle is 1ms, the maximum latch frequency is 1kHz."""
		ret = dll.ECAT_SetSlaveEncIdxLatchBufferEnable(DeviceNo, SlaveNo, EncNo, Enable)
		return ret
    
	def GetSlaveEncIdxLatchBufferEnable(DeviceNo, SlaveNo, EncNo):
		"""Get the position index latch buffer(FIFO) function setting. This function is designed for encoder module ECAT-2093."""
		Enable = c_ubyte(0)
		ret = dll.ECAT_GetSlaveEncIdxLatchBufferEnable(DeviceNo, SlaveNo, EncNo, byref(Enable))
		return ret, Enable
    
	def ResetSlaveEncIdxLatchBuffer(DeviceNo, SlaveNo, EncNo):
		"""Reset the index latch buffer(FIFO). This function is designed for encoder module ECAT-2093."""
		ret = dll.ECAT_ResetSlaveEncIdxLatchBuffer(DeviceNo, SlaveNo, EncNo)
		return ret
    
	def GetSlaveEncIdxLatchBuffer(DeviceNo, SlaveNo, EncNo, GetCount):
		"""Read the index latch buffer(FIFO). This function is designed for encoder module ECAT-2093. 
		Note:(1) After reading the data, the data will be removed from the buffer 
		(2) When the buffer is full, the buffer will no longer update data until there is room in the buffer."""
		ActualGetCount = c_int(0)
		Data = c_int(0)        
		ret = dll.ECAT_GetSlaveEncIdxLatchBuffer(DeviceNo, SlaveNo, EncNo, GetCount, byref(ActualGetCount), byref(Data))
		return ret, ActualGetCount, Data    
    
	def McSetAxisAccUnit_Stepper(DeviceNo, AxisNo, Unit):
		"""Set the acc unit of an axis.This setting is only to correctly convert the acc unit to milliseconds. 
		Note: (1)Support ECAT-2094S、ECAT-2091S 
		(2) No matter how much this value is set, when using ECAT_McSetAxisAccDecTime_Stepper or ECAT_McSetAxisAccTime  
		to set acceleration and deceleration, the unit is milliseconds 
		(3) Don't set this value unless necessary   
		(4) This setting will modify the value of 8x20:08."""
		AbortCode = c_uint(0)        
		ret = dll.ECAT_McSetAxisAccUnit_Stepper(DeviceNo, AxisNo, Unit, byref(AbortCode))
		return ret, AbortCode 
    
	def McGetAxisAccUnit_Stepper(DeviceNo, AxisNo):
		"""Get acc unit of an axis. 
		Note: Support ECAT-2094S、ECAT-2091S."""
		Unit = c_ushort(0)
		AbortCode = c_uint(0)        
		ret = dll.ECAT_McGetAxisAccUnit_Stepper(DeviceNo, AxisNo, byref(Unit), byref(AbortCode))
		return ret, Unit, AbortCode
    
	def McSetAxisVelAccScale(DeviceNo, AxisNo, VelScal, AccScal):
		"""Set the velocity/acceleration scaling parameters of an axis. 
		When the unit of PDO Entry of the drive is not pulse/s or pulse/s^2, this parameter can be used for unit scaling. 
		Example: 
		The original velocity unit of PDO is rpm. To convert pulse/s to rpm, the formula is 
		(1/1 revolution) * 60. Assuming that the number of pulses required for one revolution is 10000, 
		the velocity scaling parameter is set as 1/10000*60 = 0.006  
		The original acceleration unit of PDO is rpm. To convert pulse/s to rpm, the formula is 
		(1/1 revolution) * 60. Assuming that the number of pulses required for one revolution is 10000, 
		the acceleration scaling parameter is set as 1/10000*60 = 0.006."""
		ret = dll.ECAT_McSetAxisVelAccScale(DeviceNo, AxisNo, VelScal, AccScal)
		return ret

	def McGetAxisVelAccScale(DeviceNo, AxisNo):
		"""Get velocity/acceleration scaling parameters of an axis."""
		VelScal = c_double(0)
		AccScal = c_double(0)        
		ret = dll.ECAT_McGetAxisVelAccScale(DeviceNo, AxisNo, byref(VelScal), byref(AccScal))
		return ret, VelScal, AccScal
    
	def McSetAxisInpSignal(DeviceNo, AxisNo, Offset, Bit, Enable):
		"""Set the INP signal source of the specified axis. 
		When this function is enabled, MC_AS_STANDSTILL and MC_GS_STANDBY represent the completion of positioning. 
		When group commands are used and there are commands in the command buffer 
		(BUFFERED mode), the next command will not start until the positioning is completed. 
		Note: (1) It can be used together with ECAT_McSetAxisInpCompare. 
		(2) When the motion command speed is too high and the time is too short, it may 
		cause the motion command has been finished but the motor has not started to run. In this 
		case, it may cause the misjudgment of the positioning signal. It is recommended to use it 
		with ECAT_McSetAxisInpCompare."""
		ret = dll.ECAT_McSetAxisInpSignal(DeviceNo, AxisNo, Offset, Bit, Enable)
		return ret    
    
	def McGetAxisInpSignal(DeviceNo, AxisNo):
		"""Get INP signal source of an axis."""
		Offset = c_ushort(0)
		Bit = c_ushort(0)
		Enable = c_int(0)        
		ret = dll.ECAT_McGetAxisInpSignal(DeviceNo, AxisNo, byref(Offset), byref(Bit), byref(Enable))
		return ret, Offset, Bit, Enable
    
	def McSetAxisInpCompare(DeviceNo, AxisNo, Time_ms, PosWindow, Enable):
		"""Set the INP signal conditions of the specified Axis. 
		When this function is enabled, MC_AS_STANDSTILL and MC_GS_STANDBY represent the completion of positioning. 
		When group commands are used and there are commands in the command buffer 
		(BUFFERED mode), the next command will not start until the positioning is completed.  
		When a command is executed, the difference between the command position and the 
		actual  position  is  within  the  position  window,  and  the  stabilization  time  has  elapsed,  the 
		status will be changed to MC_AS_STANDSTILL/ MC_GS_STANDBY. 
		Note: It can be used together with ECAT_McSetAxisInpSignal."""
		ret = dll.ECAT_McSetAxisInpCompare(DeviceNo, AxisNo, Time_ms, PosWindow, Enable)
		return ret
    
	def McGetAxisInpCompare(DeviceNo, AxisNo):
		"""Get the INP signal conditions of the specified Axis."""
		Time_ms = c_uint(0)
		PosWindow = c_double(0)
		Enable = c_int(0)
		ret = dll.ECAT_McGetAxisInpCompare(DeviceNo, AxisNo, byref(Time_ms), byref(PosWindow), byref(Enable))
		return ret, Time_ms, PosWindow, Enable
    
	def McSetAxisInpTimeOut(DeviceNo, AxisNo, Time_ms):
		"""Set the INP timeout of the specified axis. 
		When the INP function is enabled, if the motion command has ended, but no INP signal 
		is detected after a period of time, the axis state will change to MC_AS_ERRORSTOP."""
		ret = dll.ECAT_McSetAxisInpTimeOut(DeviceNo, AxisNo, Time_ms)
		return ret
    
	def McGetAxisInpTimeOut(DeviceNo, AxisNo):
		"""Get the INP timeout of the specified axis."""
		Time_ms = c_uint(0)
		ret = dll.ECAT_McGetAxisInpTimeOut(DeviceNo, AxisNo, byref(Time_ms))
		return ret, Time_ms
    
	def McSetAxisWanErrEnable(DeviceNo, AxisNo, Enable):
		"""Set whether the specified Axis will change the status to MC_AS_ERRORSTOP when encountering a WAN (warning) signal. 
		Note: WAN (Warning) signal can be read using ECAT_McGetAxisDI."""
		ret = dll.ECAT_McSetAxisWanErrEnable(DeviceNo, AxisNo, Enable)
		return ret    
 
	def McGetAxisWanErrEnable(DeviceNo, AxisNo):
		"""Get the WAN (warning) signal function setting of the specified Axis number."""
		Enable = c_int(0)
		ret = dll.ECAT_McGetAxisWanErrEnable(DeviceNo, AxisNo, byref(Enable))
		return ret, Enable
	
	def SetMultiSlaveDO(DeviceNo, SlaveNo, Value, SlaveCnt):
		"""Set the digital output data of multiple slaves at the same time."""
		ret = dll.ECAT_SetMultiSlaveDO(DeviceNo, SlaveNo, Value, SlaveCnt)
		return ret
	
	def GetMultiSlaveDO(DeviceNo, SlaveNo, SlaveCnt):
		"""Get the WAN (warning) signal function setting of the specified Axis number."""
		Value = (c_uint * SlaveCnt)(*[0 for x in range(SlaveCnt)])              
		ret = dll.GetMultiSlaveDO(DeviceNo, SlaveNo, byref(Value), SlaveCnt)
		return ret, Value    
	
	def McSetAxisDefaultMode(DeviceNo, AxisNo, Mode):
		"""Set the default motion mode of the specified axis.
		The initial default motion mode is CSP mode. When ServoOn or ErrorReset, the motion mode will be switched to the default motion mode."""
		ret = dll.ECAT_McSetAxisDefaultMode(DeviceNo, AxisNo, Mode)
		return ret
	
	def McGetAxisDefaultMode(DeviceNo, AxisNo):
		"""Get default motion mode of an axis."""
		Mode = c_byte(0)
		ret = dll.ECAT_McGetAxisDefaultMode(DeviceNo, AxisNo, byref(Mode))
		return ret, Mode
	
	def McInit_Ex(DeviceNo, FileName):
		"""Use Motion Control Parameter File to initialize motion control, please refer to "Motion Control Parameter File Editing Steps" for file creation."""
		SlaveNo = (c_ushort * MC_AXIS_NO_MAX)(*[0 for x in range(MC_AXIS_NO_MAX)])         
		SubAxisNo = (c_ushort * MC_AXIS_NO_MAX)(*[0 for x in range(MC_AXIS_NO_MAX)])         
		AxisCnt = c_ushort(0)
		ret = dll.ECAT_McInit_Ex(DeviceNo, FileName, SlaveNo, SubAxisNo, byref(AxisCnt))
		return ret, SlaveNo, SubAxisNo, AxisCnt
	
	def SetMultiSlaveDO_AutoOff(DeviceNo, SlaveNo, Value, Width, Mask, SlaveCnt):
		"""Set the digital output data of multiple slaves at the same time.
		After the specified time (Width), the specified DO bit (Mask) will be turned off."""
		ret = dll.ECAT_SetMultiSlaveDO_AutoOff(DeviceNo, SlaveNo, Value, Width, Mask, SlaveCnt)
		return ret
	
	def McSetAxisAccDecUnit(DeviceNo, AxisNo, Unit):
		"""Set acceleration unit of an axis.
		When the setting unit is MC_ACC_DEC_MODE_RATE, use the setting value of ECAT_McSetAxisAccDecRate
		When the setting unit is MC_ACC_DEC_MODE_TIME, use the setting value of ECAT_McSetAxisAccTime."""
		ret = dll.ECAT_McSetAxisAccDecUnit(DeviceNo, AxisNo, Unit)
		return ret
	
	def McGetAxisAccDecUnit(DeviceNo, AxisNo):
		"""Get acceleration unit of an axis."""
		Unit = c_ushort(0)
		ret = dll.ECAT_McGetAxisAccDecUnit(DeviceNo, AxisNo, byref(Unit))
		return ret, Unit
	
	def McSetAxisAccDecRate(DeviceNo, AxisNo, Rate):
		"""Set acceleration rate of an axis."""
		ret = dll.ECAT_McSetAxisAccDecRate(DeviceNo, AxisNo, Rate)
		return ret
	
	def McGetAxisAccDecRate(DeviceNo, AxisNo):
		"""Get acceleration rate of an axis."""
		Rate = c_double(0)
		ret = dll.ECAT_McGetAxisAccDecRate(DeviceNo, AxisNo, byref(Rate))
		return ret, Rate
	
	def McGetAxisState_Ex(DeviceNo, AxisNo):
		"""Reduce the time spent using ECAT_McGetAxisState."""
		if AxisNo == 65535:
			State = (c_uint * MC_AXIS_NO_MAX)(*[0 for x in range(MC_AXIS_NO_MAX)]) 
			ret = dll.ECAT_McGetAxisState_Ex(DeviceNo, AxisNo, State) 
			return ret, State
		else:
			State = c_uint(0)
			ret = dll.ECAT_McGetAxisState_Ex(DeviceNo, AxisNo, byref(State))
			return ret, State  

	def McGetAxisLastError_Ex(DeviceNo, AxisNo):
		"""Reduce the time spent using ECAT_McGetAxisLastError."""
		if AxisNo == 65535:
			Error = (c_int * MC_AXIS_NO_MAX)(*[0 for x in range(MC_AXIS_NO_MAX)]) 
			ret = dll.ECAT_McGetAxisLastError_Ex(DeviceNo, AxisNo, Error) 
			return ret, Error
		else:
			Error = c_int(0)
			ret = dll.ECAT_McGetAxisLastError_Ex(DeviceNo, AxisNo, byref(Error))
			return ret, Error        

	def McGetAxisDriveError_Ex(DeviceNo, AxisNo):
		"""Reduce the time spent using ECAT_McGetAxisDriveError."""
		if AxisNo == 65535:
			DriveError = (c_ushort * MC_AXIS_NO_MAX)(*[0 for x in range(MC_AXIS_NO_MAX)]) 
			ret = dll.ECAT_McGetAxisDriveError_Ex(DeviceNo, AxisNo, DriveError) 
			return ret, DriveError
		else:
			DriveError = c_ushort(0)
			ret = dll.ECAT_McGetAxisDriveError_Ex(DeviceNo, AxisNo, byref(DriveError))
			return ret, DriveError        

	def McGetAxisActualPos_Ex(DeviceNo, AxisNo):
		"""Reduce the time spent using ECAT_McGetAxisActualPos."""
		if AxisNo == 65535:
			AxisPos = (c_double * MC_AXIS_NO_MAX)(*[0 for x in range(MC_AXIS_NO_MAX)]) 
			ret = dll.ECAT_McGetAxisActualPos_Ex(DeviceNo, AxisNo, AxisPos) 
			return ret, AxisPos
		else:
			AxisPos = c_double(0)
			ret = dll.ECAT_McGetAxisActualPos_Ex(DeviceNo, AxisNo, byref(AxisPos))
			return ret, AxisPos
	
	def McGetAxisActualVel_Ex(DeviceNo, AxisNo):
		"""Reduce the time spent using ECAT_McGetAxisActualVel."""
		if AxisNo == 65535:
			AxisVel = (c_double * MC_AXIS_NO_MAX)(*[0 for x in range(MC_AXIS_NO_MAX)]) 
			ret = dll.ECAT_McGetAxisActualVel_Ex(DeviceNo, AxisNo, AxisVel) 
			return ret, AxisVel
		else:
			AxisVel = c_double(0)
			ret = dll.ECAT_McGetAxisActualVel_Ex(DeviceNo, AxisNo, byref(AxisVel))
			return ret, AxisVel        

	def McGetAxisDI_Ex(DeviceNo, AxisNo):
		"""Reduce the time spent using ECAT_McGetAxisDI."""
		if AxisNo == 65535:
			AxisDI = (c_uint * MC_AXIS_NO_MAX)(*[0 for x in range(MC_AXIS_NO_MAX)]) 
			ret = dll.ECAT_McGetAxisDI_Ex(DeviceNo, AxisNo, AxisDI) 
			return ret, AxisDI
		else:
			AxisDI = c_uint(0)
			ret = dll.ECAT_McGetAxisDI_Ex(DeviceNo, AxisNo, byref(AxisDI))
			return ret, AxisDI         
	
	def McGetAxisActualPos_Directly(DeviceNo, AxisNo):
		"""Reduce the time spent using ECAT_McGetAxisActualPos_Ex.
		Note: ECAT_McEnable_Directly needs to be enabled."""
		Pos = c_double(0)
		ret = dll.ECAT_McGetAxisActualPos_Directly(DeviceNo, AxisNo, byref(Pos))
		return ret, Pos
	
	def McGetAxisCommandPos_Directly(DeviceNo, AxisNo):
		"""Reduce the time spent using ECAT_McGetAxisCommandPos_Ex.
		Note: ECAT_McEnable_Directly needs to be enabled."""
		Pos = c_double(0)
		ret = dll.ECAT_McGetAxisCommandPos_Directly(DeviceNo, AxisNo, byref(Pos))
		return ret, Pos
	
	def McGetAxisState_Directly(DeviceNo, AxisNo):
		"""Reduce the time spent using ECAT_McGetAxisState.
		Note: ECAT_McEnable_Directly needs to be enabled."""
		State = c_uint(0)
		ret = dll.ECAT_McGetAxisState_Directly(DeviceNo, AxisNo, byref(State))
		return ret, State
	
	def McEnable_Directly(DeviceNo):
		"""Enable _Directly related functions, please use this function before ECAT_StartDeviceOpTask.
		This feature is automatically disabled when using ECAT_StopDeviceOpTask with ECAT_OpenDevice.
		Note: When EnumCycleTime is 1ms, the number of axes must be less than 64 axes
		    When EnumCycleTime is 0.5ms, the number of axes must be less than 32 axes."""
		ret = dll.ECAT_McEnable_Directly(DeviceNo)
		return ret
	
	def SetSlaveDIMap(DeviceNo, ChannelCnt, SlaveNo, Dir):
		"""Enable _Directly related functions, please use this function before ECAT_StartDeviceOpTask.
		This feature is automatically disabled when using ECAT_StopDeviceOpTask with ECAT_OpenDevice.
		_Directly related functions can reduce the time spent by general functions, but there is a limit on the number of modules. 
		You need to use this function to set modules. Modules that have not been set cannot use _Directly related functions."""
		ret = dll.ECAT_SetSlaveDIMap(DeviceNo, ChannelCnt, SlaveNo, Dir)
		return ret
	
	def GetSlaveDI_Directly(DeviceNo, SlaveNo):
		"""Reduce the time spent using ECAT_McGeSlaveDI.
		Note: ECAT_SetSlaveDIMap needs to be enabled."""
		Value = c_uint(0)
		ret = dll.ECAT_GetSlaveDI_Directly(DeviceNo, SlaveNo, byref(Value))
		return ret, Value
	
	def GetSlaveDIBit_Directly(DeviceNo, SlaveNo, BitNo):
		"""Reduce the time spent using ECAT_McGeSlaveDIBit.
		Note: ECAT_SetSlaveDIMap needs to be enabled."""
		Value = c_uint(0)
		ret = dll.ECAT_GetSlaveDIBit_Directly(DeviceNo, SlaveNo, BitNo, byref(Value))
		return ret, Value
	
	def GetSlaveDO_Directly(DeviceNo, SlaveNo):
		"""Reduce the time spent using ECAT_McGeSlaveDO.
		Note: ECAT_SetSlaveDIMap needs to be enabled."""
		Value = c_uint(0)
		ret = dll.ECAT_GetSlaveDO_Directly(DeviceNo, SlaveNo, byref(Value))
		return ret, Value
	
	def GetSlaveDOBit_Directly(DeviceNo, SlaveNo, BitNo):
		"""Reduce the time spent using ECAT_McGeSlaveDOBit.
		Note: ECAT_SetSlaveDIMap needs to be enabled."""
		Value = c_uint(0)
		ret = dll.ECAT_GetSlaveDOBit_Directly(DeviceNo, SlaveNo, BitNo, byref(Value))
		return ret, Value

	def McAxisGantryOut(DeviceNo, AxisNo):
		"""Disengages the slave axis from the master axis."""
		ret = dll.ECAT_McAxisGantryOut(DeviceNo, AxisNo)
		return ret

	def SetDeviceCmpDisable(DeviceNo, CmpNo):
		"""Disable compare-trigger function."""
		ret = dll.ECAT_SetDeviceCmpDisable(DeviceNo, CmpNo)
		return ret

	def McAxisGantryGainEx(DeviceNo, SlaveNo, Kp, Ki, Kd):
		"""Set Gantrt gain."""
		ret = dll.ECAT_McAxisGantryGainEx(DeviceNo, SlaveNo, Kp, Ki, Kd)
		return ret

	def McAxisTangentOut(DeviceNo, AxisNo, GroupNo):
		"""Stop Tangent In."""
		ret = dll.ECAT_McAxisTangentOut(DeviceNo, AxisNo, GroupNo)
		return ret

	def McAxisMoveVelEx(DeviceNo, AxisNo, Vel):
		"""Start a never ending movement with a specified velocity.
		Note: A velocity control mode (CSV) is used."""
		ret = dll.ECAT_McAxisMoveVelEx(DeviceNo, AxisNo, Vel)
		return ret

	def McAxisMoveTorEx(DeviceNo, AxisNo, Torque):
		"""Start a never ending movement with a specified torque.
		Note: A torque control mode (CST) is used."""
		ret = dll.ECAT_McAxisMoveTorEx(DeviceNo, AxisNo, Torque)
		return ret

	def McAxisGearInByPos(DeviceNo, MasterNo, SlaveNo, RatioNum, RationDen, SyncSource, MasterSynPos, SlaveSynPos, GearOutPos, MaterialLength, ReturnVel, ReturnAcctime):
		"""Start a flying saw motion."""
		ret = dll.ECAT_McAxisGearInByPos(DeviceNo, MasterNo, SlaveNo, RatioNum, RationDen, SyncSource, MasterSynPos, SlaveSynPos, GearOutPos, MaterialLength, ReturnVel, ReturnAcctime)
		return ret
    
	def GetSlaveTxPdoData_Ex(DeviceNo, SlaveNo, OffsetByte, DataSize):
		"""Get TxPDO data of a slave. Read process data from the TxPDO of a slave by means of 
		cyclic communication. TxPDO data are set by digital inputs or analog inputs of a slave. To 
		read the TxPDO data is to read the status of these inputs."""
		Data = (c_ubyte * DataSize)(*[0 for x in range(DataSize)])
		ret = dll.ECAT_GetSlaveTxPdoData_Ex(DeviceNo, SlaveNo, OffsetByte, DataSize, byref(Data))
		return ret, Data
    
	def GetSlaveRxPdoData_Ex(DeviceNo, SlaveNo, OffsetByte, DataSize):
		"""Get RxPDO data of a slave. Read process data from the RxPDO of a slave by means of 
		cyclic communication. Digital outputs or analog outputs of slaves are set by RxPDO data. To 
		read the RxPDO data is to read back the status of these outputs."""
		Data = (c_ubyte * DataSize)(*[0 for x in range(DataSize)])              
		ret = dll.ECAT_GetSlaveRxPdoData_Ex(DeviceNo, SlaveNo, OffsetByte, DataSize, byref(Data))
		return ret, Data    
    
	def Get_2074A(DeviceNo, SlaveNo, ChannelNo):
		"""Get the encoder values. This function is designed for encoder module ECAT-2074A."""
		Status = c_byte(0)
		InputCycleCounter = c_byte(0)
		MultiTurn = c_uint(0)
		SingleTurn = c_uint(0)

		ret = dll.ECAT_Get_2074A(DeviceNo, SlaveNo, ChannelNo, byref(Status), byref(InputCycleCounter), byref(MultiTurn), byref(SingleTurn))
		return ret, Status, InputCycleCounter, MultiTurn, SingleTurn
	
	def McSetAxisActualPositionBy35(DeviceNo, AxisNo, Position):
		"""Set actual position of an axis by home method 35.
		Note: (1) The execution of this function will switch the state to MC_AS_HOMING. After 
		the function returns, it is necessary to check that the state is 
		MC_AS_STANDSTILL to be completed.
		(2) ECAT_McAxisHome already contains this command, no need to use this function."""
		ret = dll.ECAT_McSetAxisActualPositionBy35(DeviceNo, AxisNo, Position)
		return ret
    
	def McGetGroupPauseState(DeviceNo, GroupNo):
		"""Get pausing state of a group."""
		State = c_ushort(0)
		ret = dll.ECAT_McGetGroupPauseState(DeviceNo, GroupNo, byref(State))
		return ret, State
    
	def McGroupMoveLineAbs_Ex(DeviceNo, GroupNo, GroupPos, GroupVel):
		"""Start an absolute linear interpolation motion of a group. An array of position data of axes 
		and a velocity are requested to enter."""
		ret = dll.ECAT_McGroupMoveLineAbs_Ex(DeviceNo, GroupNo, GroupPos, GroupVel)
		return ret
    
	def McGroupMoveLineRel_Ex(DeviceNo, GroupNo, GroupPos, GroupVel):
		"""Start a relative linear interpolation motion of a group. An array of distance data of axes 
		and a velocity are requested to enter."""
		ret = dll.ECAT_McGroupMoveLineRel_Ex(DeviceNo, GroupNo, GroupPos, GroupVel)
		return ret
    
	def McGroupMoveLineAbsAdv_Ex(DeviceNo, GroupNo, EndPos, StartVel, ReqVel, FinalVel, Accel, Decel, AccDecMode): 
		"""Start an absolute linear interpolation motion of a group."""
		ret = dll.ECAT_McGroupMoveLineAbsAdv_Ex(DeviceNo, GroupNo, EndPos, StartVel, ReqVel, FinalVel, Accel, Decel, AccDecMode) 
		return ret
    
	def McGroupMoveLineRelAdv_Ex(DeviceNo, GroupNo, EndPos, StartVel, ReqVel, FinalVel, Accel, Decel, AccDecMode): 
		"""Start an relative linear interpolation motion of a group."""
		ret = dll.ECAT_McGroupMoveLineRelAdv_Ex(DeviceNo, GroupNo, EndPos, StartVel, ReqVel, FinalVel, Accel, Decel, AccDecMode) 
		return ret    
    
	def McSetAxisHomeStable(DeviceNo, AxisNo, Time_s, TimeOut_s, PosRange):
		"""Set the homing stable settings of an axis.
		After homing, it will wait for the actual position to be within the PosRange and maintain Time_s before changing the axis status MC_AS_HOMING to MC_AS_STANDSTILL.
		"""
		ret = dll.ECAT_McSetAxisHomeStable(DeviceNo, AxisNo, Time_s, TimeOut_s, PosRange)
		return ret
    
	def McGetAxisHomeStable(DeviceNo, AxisNo):
		"""Get the homing stable settings of an axis."""
		Time_s = c_double(0)
		TimeOut_s = c_double(0)
		PosRange = c_double(0)        
		ret = dll.ECAT_McGetAxisHomeStable(DeviceNo, AxisNo, byref(Time_s), byref(TimeOut_s), byref(PosRange))
		return ret, Time_s, TimeOut_s, PosRange
	    
	def McAxisMoveAbsAdv_Ex(DeviceNo, AxisNo, EndPos, StartVel, ReqVel, FinalVel, Accel, Decel, AccDecMode):
		"""Start an absolute position motion of an axis."""
		ret = dll.ECAT_McAxisMoveAbsAdv_Ex(DeviceNo, AxisNo, EndPos, StartVel, ReqVel, FinalVel, Accel, Decel, AccDecMode) 
		return ret    
    
	def McAxisMoveRelAdv_Ex(DeviceNo, AxisNo, EndPos, StartVel, ReqVel, FinalVel, Accel, Decel, AccDecMode):
		"""Start an relative position motion of an axis."""
		ret = dll.ECAT_McAxisMoveRelAdv_Ex(DeviceNo, AxisNo, EndPos, StartVel, ReqVel, FinalVel, Accel, Decel, AccDecMode) 
		return ret    
	
	def SetSlaveDIMap_16bit(DeviceNo, ChannelCnt, SlaveNo, Offset, Dir):
		"""Enable _Directly related functions, please use this function before ECAT_StartDeviceOpTask.
		This feature is automatically disabled when using ECAT_StopDeviceOpTask with ECAT_OpenDevice.
		_Directly related functions can reduce the time spent by general functions, but there is a limit on the number of modules. 
		You need to use this function to set modules. Modules that have not been set cannot use _Directly related functions."""
		ret = dll.ECAT_SetSlaveDIMap_16bit(DeviceNo, ChannelCnt, SlaveNo, Offset, Dir)
		return ret
	
	def GetSlaveDI_Directly_16bit(DeviceNo):
		"""Reduce the time spent using ECAT_McGeSlaveDI.
		Note: ECAT_SetSlaveDIMap_16bit needs to be enabled."""
		Value = (c_ushort * SLAVE_DIO_MAX_EX)(*[0 for x in range(SLAVE_DIO_MAX_EX)])    
		ret = dll.ECAT_GetSlaveDI_Directly_16bit(DeviceNo, byref(Value))
		return ret, Value
	
	def do_cfg_save(DeviceNo, SlaveNo, Offset, Value, SlaveCnt):
		"""Save the SubDevice DO data to the PC.
		The path is C:\icpdas\Ecat-M801\ECAT_DO_Config.xml."""
		ret = dll.ECAT_do_cfg_save(DeviceNo, SlaveNo, Offset, Value, SlaveCnt)
		return ret
	
	def do_cfg_load(DeviceNo):
		"""Load ECAT_DO_Config.xml and set the digital output data of SubDevices.
		The path is C:\icpdas\Ecat-M801\ECAT_DO_Config.xml."""
		ret = dll.ECAT_do_cfg_load(DeviceNo)
		return ret
		
	def ClearDoQueue(DeviceNo):
		"""Set default digital output data to 0.
		Note: Support ECAT-2045K-32."""
		ret = dll.ECAT_ClearDoQueue(DeviceNo)
		return ret
	
	def PreSetSlaveDO(DeviceNo, SlaveNo, Offset, Value):
		"""Set the digital output data of a SubDevice."""
		ret = dll.ECAT_PreSetSlaveDO(DeviceNo, SlaveNo, Offset, Value)
		return ret
    
	def SetSlaveRxPdoData_Ex(DeviceNo, SlaveNo, OffsetByte, DataSize, Data):
		"""Set RxPDO data of a slave. Transfer process data to the RxPDO of a slave by means of 
		cyclic communication. Digital outputs and analog outputs of slaves are set by RxPDO data."""
		ret = dll.ECAT_SetSlaveRxPdoData_Ex(DeviceNo, SlaveNo, OffsetByte, DataSize, Data)
		return ret
    
	def McAddAxisToGroup_Ex(DeviceNo, GroupNo, AxisNo):
		"""Add one axis to a group."""
		ret = dll.ECAT_McAddAxisToGroup_Ex(DeviceNo, GroupNo, AxisNo)
		return ret
  
	def McRemoveAxisFromGroup_Ex(DeviceNo, GroupNo, AxisNo):
		"""Remove one axis from a group."""
		ret = dll.ECAT_McRemoveAxisFromGroup_Ex(DeviceNo, GroupNo, AxisNo)
		return ret
    
	def McUngroupAllAxes_Ex(DeviceNo, GroupNo):
		"""Remove all axes from a group. This group no longer owns any axis."""
		ret = dll.ECAT_McUngroupAllAxes_Ex(DeviceNo, GroupNo)
		return ret
    
	def McSetGroupCmdMode_Ex(DeviceNo, GroupNo, CmdMode):
		"""This function will set the command mode of a group immediately. The group command 
		mode will decide how a motion command is processed. There are three command modes: 
		aborting, buffered, and blending."""
		ret = dll.ECAT_McSetGroupCmdMode_Ex(DeviceNo, GroupNo, CmdMode)
		return ret
    
	def McSetGroupAccDecType_Ex(DeviceNo, GroupNo, Type):
		"""Set the acceleration type of a group. There are two acceleration types: T-Curve (linear acceleration) and S-Curve.   
		Note: The T-curve acceleration time is set by function ECAT_McSetGroupAccTime.
		However, the S-curve acceleration time is twice the acceleration time set by that function."""
		ret = dll.ECAT_McSetGroupAccDecType_Ex(DeviceNo, GroupNo, Type)
		return ret

	def McSetGroupAccTime_Ex(DeviceNo, GroupNo, Time_ms):
		"""Set the acceleration time of a group."""
		ret = dll.ECAT_McSetGroupAccTime_Ex(DeviceNo, GroupNo, Time_ms)
		return ret    
    
	def McGroupMoveProfileCSV_Ex(DeviceNo, GroupNo, ProfileNo):
		"""Start a profile position motion. The profile data are read from a CSV file."""
		ret = dll.ECAT_McGroupMoveProfileCSV_Ex(DeviceNo, GroupNo, ProfileNo)
		return ret
    
	def McGroupMoveDwell_Ex(DeviceNo, GroupNo, DwellTime):
		"""The motion kernel will make a group to wait for the dwell time; after time is up, continue 
		to  load  and  execute  the  next  command.  This  command  can  be  used  for  adjusting  the 
		blending  distance  between  two  motion  commands  in  continuous  blending  motion.  This 
		command behaves just like any other motion commands and is sequentially executed. In 
		Buffered or Blending mode, if a motion command is being executed, it will be pushed into the 
		command buffer. In Aborting mode, motion kernel will stop executing the current command 
		by deceleration and start to wait for the dwell time."""
		ret = dll.ECAT_McGroupMoveDwell_Ex(DeviceNo, GroupNo, DwellTime)
		return ret    
    
	def McGroupMoveBlendingSync_Ex(DeviceNo, GroupNo):
		"""When the group is in the blending mode, this command will make the motion kernel to 
		wait until the current command is finished before executing the next motion command. After 
		that, the group is still in blending mode and a new current motion command will blend with its 
		next motion command."""
		ret = dll.ECAT_McGroupMoveBlendingSync_Ex(DeviceNo, GroupNo)
		return ret    

	def McGetAxisActualPosVel_Ex(DeviceNo, AxisNo):
		"""Get actual position and velocity of an axis. 
		Note: When AxisNo is set to 65535, both the actual positions and velocities of all axes 
		are read back and saved into Pos and Vel array pointers, respectively."""
		AxisPos = c_float(0)
		AxisVel = c_float(0)        
		ret = dll.ECAT_McGetAxisActualPosVel_Ex(DeviceNo, AxisNo, byref(AxisPos), byref(AxisVel))
		return ret, AxisPos, AxisVel
    
	def McSetGroupVelLimitStatus_Ex(DeviceNo, GroupNo, State):
		"""Enable or disable the checking of the velocity limit of a group. If state is "Enable", each 
		axis speed in this group will be checked for not over a defined maximum value. If one of 
		these axes is over the speed limit value, this group speed will be recalculated to meet the speed limit requirement.   
		Note: This velocity limit requirement right now is valid only for two functions: 
		ECAT_McGroupMoveLineAbs_PT and ECAT_McGroupMoveLineRel_PT."""
		ret = dll.ECAT_McSetGroupVelLimitStatus_Ex(DeviceNo, GroupNo, State)
		return ret    
    
	def McSetGroupVelLimitValue_Ex(DeviceNo, GroupNo, Value):
		"""Set the velocity limit value of each axis in a group. 
		Note: This velocity limit requirement right now is valid only for two functions: 
		ECAT_McGroupMoveLineAbs_PT and ECAT_McGroupMoveLineRel_PT."""
		ret = dll.ECAT_McSetGroupVelLimitValue_Ex(DeviceNo, GroupNo, Value)
		return ret    
    
	def McGetMotionRecordValue_Ex(DeviceNo, CountNo, AxisNo):
		"""Get parameter values of an assigned axis at an assigned index number. 
		Note: When the AxisNo is set to 65535, values of all axes at the assigned index number 
		are returned by Value1and Value2 pointers."""
		Value1 = c_float(0)
		Value2 = c_float(0)  
		ret = dll.ECAT_McGetMotionRecordValue_Ex(DeviceNo, CountNo, AxisNo, byref(Value1), byref(Value2))
		return ret, Value1, Value2    
    
	def McSetGroupHold_Ex(DeviceNo, GroupNo, Status):
		"""The group state becomes MC_GS_HOLD and the motion kernel will stop loading new 
		commands after current command is done. After disable the holding, the motion kernel will 
		load a new command from command buffer and execute commands sequentially. 
		Notice: The PVT motion command and other group motion commands cannot be used together."""
		ret = dll.ECAT_McSetGroupHold_Ex(DeviceNo, GroupNo, Status)
		return ret     
    
	def McSetGroupCmdModeEx_Ex(DeviceNo, GroupNo, CmdMode):
		"""This function is a little different from ECAT_McSetGroupCmdMode at the timing for 
		setting group command mode. It will be pushed into the command buffer first and wait for 
		executing if the current command mode in Buffered mode or Blending mode. However, in 
		Aborting mode it will change command mode immediately. A group command mode decides 
		how a motion command is processed by motion kernel. There are three command modes: 
		Aborting, Buffered, and Blending.   
		Aborting:  A  new  command  will  abort  the  current  executing  command;  then  the  new 
		command executes immediately. However, the motion kernel still provides a smooth velocity 
		transition for this mode."""
		ret = dll.ECAT_McSetGroupCmdModeEx_Ex(DeviceNo, GroupNo, CmdMode)
		return ret    
    
	def McSetGroupBlendingPercent_Ex(DeviceNo, GroupNo, Value):
		"""Set the blending percent of a group. In the Blending mode, a "100" blending percent 
		means to blend the next motion command from the starting of deceleration of the pervious 
		motion  command.  A  "0"  blending  percent  means  no  blending  part;  and  the  behavior  is 
		similar to the Buffered command mode. Blending will introduce a smooth transition from one 
		command to another; however, it will produce corner error."""
		ret = dll.ECAT_McSetGroupBlendingPercent_Ex(DeviceNo, GroupNo, Value)
		return ret

	def McGetMotionRecordValueEx_Ex(DeviceNo, CountNo, Count, AxisNo):
		"""Get parameter values of an axis starting from an assigned index number. This function 
		is able to get more records than ECAT_McGetMotionRecordValue. This function can get up 
		to 64 records each time rather than only one record by ECAT_McGetMotionRecordValue."""
		Value1 = c_float(0)
		Value2 = c_float(0)
		ActualCount = c_ushort(0)        
		ret = dll.ECAT_McGetMotionRecordValueEx_Ex(DeviceNo, CountNo, Count, AxisNo, byref(Value1), byref(Value2), byref(ActualCount))
		return ret, Value1, Value2, ActualCount    

	def McSetGroupBlendingPercentEx_Ex(DeviceNo, GroupNo, Value):
		"""Set the blending percent of a group. It is different from ECAT_McSetGroupBlendingPercent
		at the executing time. This command will be pushed 
		into command buffer first in Buffered mode or Blending mode and wait for execution."""
		ret = dll.ECAT_McSetGroupBlendingPercentEx_Ex(DeviceNo, GroupNo, Value) 
		return ret     
    
	def McSetGroupPause_Ex(DeviceNo, GroupNo, Status):
		"""The group state becomes MC_GS_PAUSE and the motion kernel will pause the current 
		group motion with deceleration immediately. The current command is just partially done, and 
		some remaining part is held. After the pause state becomes disabled, the motion kernel will 
		execute  the  remaining  part  of  the  unfinished  command  and  other  commands  in  the 
		command buffer sequentially."""
		ret = dll.ECAT_McSetGroupPause_Ex(DeviceNo, GroupNo, Status)
		return ret
    
	def SetCheckSlaveCnt(DeviceNo, Enable):
		"""Enable/disable Network information check when entering OP."""
		ret = dll.ECAT_SetCheckSlaveCnt(DeviceNo, Enable)
		return ret

	def McGroupMoveShaker_Ex(DeviceNo, GroupNo, Amp, phase, Freq, Time):
		"""This function is for factory use only.         
		Start a relative sine wave motion of a group. 
		Note:(1) Can be used with some group commands 
		(2) Blending or buffer mode is not supported, it can be used in blending or buffer 
		mode, but will abort relative sine wave movement in progress. 
		Y = Amp * sin(2 * pi * Freq * t + phase) 
		t = 0 to Time ."""
		ret = dll.ECAT_McGroupMoveShaker_Ex(DeviceNo, GroupNo, Amp, phase, Freq, Time)
		return ret    

	def McSetGroupPvtDecEnable_Ex(DeviceNo, GroupNo, Enable):
		"""Set whether to decelerate or not after the PVT motion is finished of a group."""
		ret = dll.ECAT_McSetGroupPvtDecEnable_Ex(DeviceNo, GroupNo, Enable)
		return ret
    
	def McSetGroupCoordinate_Ex(DeviceNo, GroupNo, Type):
		"""Set whether the group motion control performs coordinate conversion."""
		ret = dll.ECAT_McSetGroupCoordinate_Ex(DeviceNo, GroupNo, Type)
		return ret
    
	def McSetGroupCoordinateLimit_Ex(DeviceNo, GroupNo, Type, Enable, Min, Max):
		"""Set the position software limit of a group."""
		ret = dll.ECAT_McSetGroupCoordinateLimit_Ex(DeviceNo, GroupNo, Type, Enable, Min, Max)
		return ret    
    
	def McGetGroupCoordinateLimit_Ex(DeviceNo, GroupNo, Type):
		"""Get the position software limit of a group."""
		Enable = c_ushort(0)
		Min = c_double(0)
		Max = c_double(0)        
		ret = dll.ECAT_McGetGroupCoordinateLimit_Ex(DeviceNo, GroupNo, Type, byref(Enable), byref(Min), byref(Max))
		return ret, Enable, Min, Max
	
	def SetMultiSlaveDO_Ex(DeviceNo, SlaveNo, Value, SlaveCnt):
		"""Set the digital output data of multiple slaves at the same time."""
		ret = dll.ECAT_SetMultiSlaveDO_Ex(DeviceNo, SlaveNo, Value, SlaveCnt)
		return ret
	
	def GetMultiSlaveDO_Ex(DeviceNo, SlaveNo, SlaveCnt):
		"""Get the WAN (warning) signal function setting of the specified Axis number."""
		Value = (c_uint * SlaveCnt)(*[0 for x in range(SlaveCnt)])              
		ret = dll.ECAT_GetMultiSlaveDO_Ex(DeviceNo, SlaveNo, byref(Value), SlaveCnt)
		return ret, Value    
	
	def SetMultiSlaveDO_AutoOff_Ex(DeviceNo, SlaveNo, Value, Width, Mask, SlaveCnt):
		"""Set the digital output data of multiple slaves at the same time.
		After the specified time (Width), the specified DO bit (Mask) will be turned off."""
		ret = dll.ECAT_SetMultiSlaveDO_AutoOff_Ex(DeviceNo, SlaveNo, Value, Width, Mask, SlaveCnt)
		return ret
    
	def McGroupMoveCircularAbs_CP_Angle_Ex(DeviceNo, GroupNo, GroupVel, CircAngle , CircAuxPos):
		"""Start an absolute 2D circular interpolation motion by providing the center position and its angle."""
		ret = dll.ECAT_McGroupMoveCircularAbs_CP_Angle_Ex(DeviceNo, GroupNo, GroupVel, CircAngle , CircAuxPos)
		return ret
    
	def McGroupMoveCircularAbs_CP_EP_Ex(DeviceNo, GroupNo, GroupVel, CircDir , CircAuxPos, CircEndPos):
		"""Start an absolute 2D circular interpolation motion by providing the center position and the end position."""
		ret = dll.ECAT_McGroupMoveCircularAbs_CP_EP_Ex(DeviceNo, GroupNo, GroupVel, CircDir , CircAuxPos, CircEndPos)
		return ret    

	def McGroupMoveCircularAbs_BP_EP_Ex(DeviceNo, GroupNo, GroupVel, CircDir , CircAuxPos, CircEndPos):
		"""Start an absolute 2D circular interpolation motion by providing a border position and the end position."""
		ret = dll.ECAT_McGroupMoveCircularAbs_BP_EP_Ex(DeviceNo, GroupNo, GroupVel, CircDir , CircAuxPos, CircEndPos)
		return ret
    
	def McGroupMove3DCircularAbs_CP_Angle_Ex(DeviceNo, GroupNo, GroupVel , CircAngle, CircAuxPos, NV):
		"""Start an absolute 3D circular interpolation motion by providing the center position and an angle."""
		ret = dll.ECAT_McGroupMove3DCircularAbs_CP_Angle_Ex(DeviceNo, GroupNo, GroupVel , CircAngle, CircAuxPos, NV)
		return ret    

	def McGroupMove3DCircularAbs_CP_EP_Ex(DeviceNo, GroupNo, GroupVel, CircDir, CircAuxPos, CircEndPos):
		"""Start an absolute 3D circular interpolation motion by providing the center position and the end position."""
		ret = dll.ECAT_McGroupMove3DCircularAbs_CP_EP_Ex(DeviceNo, GroupNo, GroupVel, CircDir, CircAuxPos, CircEndPos)
		return ret
    
	def McGroupMove3DCircularAbs_BP_EP_Ex(DeviceNo, GroupNo, GroupVel , CircDir, CircAuxPos, CircEndPos):
		"""Start an absolute 3D circular interpolation motion by providing a border position and the end position."""
		ret = dll.ECAT_McGroupMove3DCircularAbs_BP_EP_Ex(DeviceNo, GroupNo, GroupVel , CircDir, CircAuxPos, CircEndPos)
		return ret    
    
	def McGroupMoveCircularRel_CP_Angle_Ex(DeviceNo, GroupNo, GroupVel, CircAngle , CircAuxPos):
		"""Start a relative 2D circular interpolation motion by providing the center position and its angle."""
		ret = dll.ECAT_McGroupMoveCircularRel_CP_Angle_Ex(DeviceNo, GroupNo, GroupVel, CircAngle , CircAuxPos)
		return ret     

	def McGroupMoveCircularRel_CP_EP_Ex(DeviceNo, GroupNo, GroupVel, CircDir, CircAuxPos, CircEndPos):
		"""Start a relative 2D circular interpolation motion by providing the center position and the end position."""
		ret = dll.ECAT_McGroupMoveCircularRel_CP_EP_Ex(DeviceNo, GroupNo, GroupVel, CircDir, CircAuxPos, CircEndPos)
		return ret
    
	def McGroupMoveCircularRel_BP_EP_Ex(DeviceNo, GroupNo, GroupVel, CircDir, CircAuxPos, CircEndPos):
		"""Start a relative 2D circular interpolation motion by providing a border position and the end position."""
		ret = dll.ECAT_McGroupMoveCircularRel_BP_EP_Ex(DeviceNo, GroupNo, GroupVel, CircDir, CircAuxPos, CircEndPos)
		return ret    
        
	def McGroupMove3DCircularRel_CP_Angle_Ex(DeviceNo, GroupNo, GroupVel, CircAngle, CircAuxPos, NV):
		"""Start a relative 3D circular interpolation motion by providing the center position and an angle."""
		ret = dll.ECAT_McGroupMove3DCircularRel_CP_Angle_Ex(DeviceNo, GroupNo, GroupVel, CircAngle, CircAuxPos, NV)
		return ret          

	def McGroupMove3DCircularRel_CP_EP_Ex(DeviceNo, GroupNo, GroupVel, CircDir, CircAuxPos, CircEndPos):
		"""Start a relative 3D circular interpolation motion by providing the center position and the end position."""
		ret = dll.ECAT_McGroupMove3DCircularRel_CP_EP_Ex(DeviceNo, GroupNo, GroupVel, CircDir, CircAuxPos, CircEndPos)
		return ret
    
	def McGroupMove3DCircularRel_BP_EP_Ex(DeviceNo, GroupNo, GroupVel, CircDir, CircAuxPos, CircEndPos):
		"""Start a relative 3D circular interpolation motion by providing a border position and the end position."""
		ret = dll.ECAT_McGroupMove3DCircularRel_BP_EP_Ex(DeviceNo, GroupNo, GroupVel, CircDir, CircAuxPos, CircEndPos)
		return ret    
    
	def McGroupMoveHelicalAbs_Ex(DeviceNo, GroupNo, CircAngle, CircAuxPos, HelicalPitch, GroupVel):
		"""Start a helical interpolation motion."""
		ret = dll.ECAT_McGroupMoveHelicalAbs_Ex(DeviceNo, GroupNo, CircAngle, CircAuxPos, HelicalPitch, GroupVel)
		return ret
    
	def McGroupMove3DHelicalAbs_CP_Angle_Ex(DeviceNo, GroupNo, GroupVel, CircAngle, HelicalPitch, CircAuxPos, NV):
		"""Start an absolute 3D helical interpolation motion."""
		ret = dll.ECAT_McGroupMove3DHelicalAbs_CP_Angle_Ex(DeviceNo, GroupNo, GroupVel, CircAngle, HelicalPitch, CircAuxPos, NV)
		return ret    
    
	def McGroupMoveHelicalRel_Ex(DeviceNo, GroupNo, CircAngle, CircAuxPos, HelicalPitch, GroupVel):
		"""Start a relative helical interpolation motion."""
		ret = dll.ECAT_McGroupMoveHelicalRel_Ex(DeviceNo, GroupNo, CircAngle, CircAuxPos, HelicalPitch, GroupVel)
		return ret
    
	def McGroupMove3DHelicalRel_CP_Angle_Ex(DeviceNo, GroupNo, GroupVel, CircAngle, HelicalPitch, CircAuxPos, NV):
		"""Start a relative 3D helical interpolation motion."""
		ret = dll.ECAT_McGroupMove3DHelicalRel_CP_Angle_Ex(DeviceNo, GroupNo, GroupVel, CircAngle, HelicalPitch, CircAuxPos, NV)
		return ret 
    
	def McGroupMoveConicalHelixAbs_Ex(DeviceNo, GroupNo, CircAngle, CircAuxPos, HelicalPitch, GroupVel, EndRadius):
		"""Start an absolute conical helix interpolation motion."""
		ret = dll.ECAT_McGroupMoveConicalHelixAbs_Ex(DeviceNo, GroupNo, CircAngle, CircAuxPos, HelicalPitch, GroupVel, EndRadius)
		return ret
    
	def McGroupMove3DConicalHelixAbs_CP_Angle_Ex(DeviceNo, GroupNo, GroupVel, CircAngle, HelicalPitch, CircAuxPos, NV, EndRadius):
		"""Start an absolute 3D conical helix interpolation motion."""
		ret = dll.ECAT_McGroupMove3DConicalHelixAbs_CP_Angle_Ex(DeviceNo, GroupNo, GroupVel, CircAngle, HelicalPitch, CircAuxPos, NV, EndRadius)
		return ret
    
	def McGroupMoveConicalHelixRel_Ex(DeviceNo, GroupNo, CircAngle, CircAuxPos, HelicalPitch, GroupVel, EndRadius):
		"""Start a relative conical helix interpolation motion."""
		ret = dll.ECAT_McGroupMoveConicalHelixRel_Ex(DeviceNo, GroupNo, CircAngle, CircAuxPos, HelicalPitch, GroupVel, EndRadius)
		return ret
    
	def McGroupMove3DConicalHelixRel_CP_Angle_Ex(DeviceNo, GroupNo, GroupVel, CircAngle, HelicalPitch, CircAuxPos, NV, EndRadius):
		"""Start a relative 3D conical helix interpolation motion."""
		ret = dll.ECAT_McGroupMove3DConicalHelixRel_CP_Angle_Ex(DeviceNo, GroupNo, GroupVel, CircAngle, HelicalPitch, CircAuxPos, NV, EndRadius)
		return ret    
        
	def McGroupMoveProfile_Ex(DeviceNo, GroupNo, ProfileNo, TotalStep):
		"""Start a profile position motion."""
		ret = dll.ECAT_McGroupMoveProfile_Ex(DeviceNo, GroupNo, ProfileNo, TotalStep)
		return ret  
    
	def McGroupMoveLineAbs_PT_Ex(DeviceNo, GroupNo, GroupPos, GroupTime):
		"""Start an absolute linear interpolation motion of a group. An array of position data of axes 
		and  action  time  are  requested  to  enter.  The  command  speed  of  each  axis  is  calculated 
		according to the position data and the time value."""
		ret = dll.ECAT_McGroupMoveLineAbs_PT_Ex(DeviceNo, GroupNo, GroupPos, GroupTime)
		return ret    
    
	def McGroupMoveLineRel_PT(DeviceNo, GroupNo, GroupPos, GroupTime):
		"""Start a relative linear interpolation motion of a group. An array of distance data of axes 
		and  action  time  are  requested  to  enter.  The  command  speed  of  each  axis  is  calculated 
		according to the distance data and the time value."""
		ret = dll.ECAT_McGroupMoveLineRel_PT_Ex(DeviceNo, GroupNo, GroupPos, GroupTime)
		return ret    
	
	def McEnable_Directly_Ex(DeviceNo):
		"""Enable _Directly related functions, please use this function before ECAT_StartDeviceOpTask.
		This feature is automatically disabled when using ECAT_StopDeviceOpTask with ECAT_OpenDevice.
		Note: When EnumCycleTime is 1ms, the number of axes must be less than 64 axes
		    When EnumCycleTime is 0.5ms, the number of axes must be less than 32 axes."""
		ret = dll.ECAT_McEnable_Directly_Ex(DeviceNo)
		return ret


